package com.jpsoft.bus.modules.base.entity;

import java.io.Serializable;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.math.BigDecimal;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiModel;
import lombok.Data;

/**
 * 描述:base_order_info的实体类
 */
@Data
@ApiModel(value = "base_order_info的实体类")
public class OrderInfo {
    @ApiModelProperty(value = "")
    private String id;
    @ApiModelProperty(value = "账单金额")
    private BigDecimal totalFee = BigDecimal.ZERO;
    @ApiModelProperty(value = "商品描述")
    private String body;
    @ApiModelProperty(value = "支付状态（10：未支付，15：部分支付，20：支付完成，30：关闭，40：全额退款，45：部分退款）")
    private Integer payStatus;
    @ApiModelProperty(value = "支付方式（wechat：微信，alipay：支付宝，cash：现金，ticket：车票）")
    private String payName;
    @ApiModelProperty(value = "支付金额")
    private BigDecimal payFee = BigDecimal.ZERO;
    @ApiModelProperty(value = "退款金额")
    private BigDecimal refundFee = BigDecimal.ZERO;
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm", timezone = "GMT+8")
    @ApiModelProperty(value = "支付时间")
    private Date payTime;
    @ApiModelProperty(value = "系统订单号")
    private String outOrderNo;
    @ApiModelProperty(value = "微信支付订单号")
    private String transactionId;
    @ApiModelProperty(value = "用户标识")
    private String openId;
    @ApiModelProperty(value = "收款账户id")
    private String paymentId;
    @ApiModelProperty(value = "创建人")
    private String createBy;
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm", timezone = "GMT+8")
    @ApiModelProperty(value = "创建时间")
    private Date createTime;
    @ApiModelProperty(value = "更新人")
    private String updateBy;
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm", timezone = "GMT+8")
    @ApiModelProperty(value = "更新时间")
    private Date updateTime;
    @ApiModelProperty(value = "是否删除")
    private Boolean delFlag =false;
    @ApiModelProperty(value = "乘客记录表")
    private String passengerId;
    @ApiModelProperty(value = "购票类型,1：儿童票，2：成人票")
    private String ticketType;
    @ApiModelProperty(value = "货票金额")
    private BigDecimal goodsTicket = BigDecimal.ZERO;
    @ApiModelProperty(value = "综合账单id")
    private String mergeOrderId;

    @ApiModelProperty(value = "购票的上车站点")
    private String ticketUpStationId;

    @ApiModelProperty(value = "购票的下车站点")
    private String ticketDownStationId;

    @ApiModelProperty(value = "结算状态")
    private Boolean settlementFlag = false;


    public String getPayNameStr(String payName){
        String payNameStr = "";
        if ("wechat".equals(payName)){
            payNameStr = "微信";
        }
        if ("alipay".equals(payName)){
            payNameStr = "支付宝";
        }
        if ("cash".equals(payName)){
            payNameStr = "现金";
        }
        if ("ticket".equals(payName)){
            payNameStr = "车票";
        }
        return payNameStr;
    }

    public String getTicketTypeStr(String ticketType){
        String ticketTypeStr = "";
        if ("1".equals(ticketType)){
            ticketTypeStr = "儿童票";
        }
        if ("2".equals(ticketType)){
            ticketTypeStr = "成人票";
        }
        return ticketTypeStr;
    }
}