package com.jpsoft.bus.modules.base.service.impl;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import javax.annotation.Resource;

import cn.hutool.core.date.DateTime;
import com.jpsoft.bus.modules.base.dao.OrderLogDAO;
import com.jpsoft.bus.modules.base.entity.OrderLog;
import com.jpsoft.bus.modules.base.service.OrderLogService;
import com.jpsoft.bus.modules.bus.entity.PassengerInfo;
import com.jpsoft.bus.modules.bus.entity.VehicleInfo;
import com.jpsoft.bus.modules.bus.service.PassengerInfoService;
import com.jpsoft.bus.modules.pay.service.AlipayService;
import com.jpsoft.bus.modules.pay.service.WechatService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import com.jpsoft.bus.modules.base.dao.OrderInfoDAO;
import com.jpsoft.bus.modules.base.entity.OrderInfo;
import com.jpsoft.bus.modules.base.service.OrderInfoService;
import com.github.pagehelper.Page;
import com.jpsoft.bus.modules.common.dto.Sort;
import com.github.pagehelper.PageHelper;

@Transactional
@Component(value="orderInfoService")
public class OrderInfoServiceImpl implements OrderInfoService {
	@Resource(name="orderInfoDAO")
	private OrderInfoDAO orderInfoDAO;

	@Resource(name="orderLogDAO")
	private OrderLogDAO orderLogDAO;

	@Autowired
	private AlipayService alipayService;

	@Autowired
	private PassengerInfoService passengerInfoService;

	@Autowired
	private WechatService wechatService;

	@Override
	public OrderInfo get(String id) {
		// TODO Auto-generated method stub
		return orderInfoDAO.get(id);
	}

	@Override
	public int insert(OrderInfo model) {
		// TODO Auto-generated method stub
		//model.setId(UUID.randomUUID().toString());

		return orderInfoDAO.insert(model);
	}

	@Override
	public int update(OrderInfo model) {
		// TODO Auto-generated method stub
		return orderInfoDAO.update(model);
	}

	@Override
	public int delete(String id) {
		// TODO Auto-generated method stub
		return orderInfoDAO.delete(id);
	}

	@Override
	public boolean exist(String id) {
		// TODO Auto-generated method stub
		int count = orderInfoDAO.exist(id);

		return count > 0 ? true : false;
	}

	@Override
	public List<OrderInfo> list() {
		// TODO Auto-generated method stub
		return orderInfoDAO.list();
	}

	@Override
	public Page<OrderInfo> pageSearch(Map<String, Object> searchParams, int pageNumber, int pageSize,boolean count,List<Sort> sortList) {
        Page<OrderInfo> page = PageHelper.startPage(pageNumber,pageSize,count).doSelectPage(()->{
            orderInfoDAO.search(searchParams,sortList);
        });

        return page;
	}

	@Override
	public List<OrderInfo> findByPassengerIdAndPayStatus(String passengerId,Integer payStatus) {
		return orderInfoDAO.findByPassengerIdAndPayStatus(passengerId,payStatus);
	}

	@Override
	public OrderInfo findByOutOrderNo(String outTradeNo) {
		return orderInfoDAO.findByOutOrderNo(outTradeNo);
	}

	@Override
	public List<OrderInfo> findByMergeOrderId(String mergeOrderId) {
		return orderInfoDAO.findByMergeOrderId(mergeOrderId);
	}

	@Override
	public OrderInfo findByPassengerIdMergeOrderId(String id, String mergeOrderId) {
		return orderInfoDAO.findByPassengerIdMergeOrderId(id,mergeOrderId);
	}

	@Override
	public OrderInfo findByPassengerId(String passengerId){
		return orderInfoDAO.findByPassengerId(passengerId);
	}

	@Override
	public Boolean refund(String orderId,String userId) throws Exception {

		OrderInfo orderInfo = get(orderId);
		if (orderInfo == null){
			throw new Exception("订单不存在");
		}
		if (orderInfo.getPayStatus() !=20){
			throw new Exception("订单状态错误，不可退款");
		}
		if (!"wechat".equals(orderInfo.getPayName()) && !"alipay".equals(orderInfo.getPayName())){
			throw new Exception("非在线支付，不可在线退款");
		}

		if ("wechat".equals(orderInfo.getPayName())){
			wechatService.wxRefund(orderInfo);
		}

		if ("alipay".equals(orderInfo.getPayName())){
			alipayService.aliPayRefund(orderInfo);
		}

		//退款成功修改乘客记录
		PassengerInfo passengerInfo = passengerInfoService.get(orderInfo.getPassengerId());
		passengerInfo.setPayStatus("0");
		passengerInfoService.update(passengerInfo);

		OrderLog orderLog = new OrderLog();
		orderLog.setId(UUID.randomUUID().toString());
		orderLog.setCreateBy(userId);
		orderLog.setCreateTime(new Date());
		orderLog.setContent("线上退款成功,退款金额："+ orderInfo.getPayFee().toString()+"元");
		orderLog.setOrderId(orderInfo.getId());
		orderLogDAO.insert(orderLog);


		return true;
	}

	@Override
	public BigDecimal paySum(List<VehicleInfo> vehicleInfoList,String companyId, String payName, Date beginOfDay, Date endOfDay,Boolean settlementFlag) {
		return orderInfoDAO.paySum(vehicleInfoList,companyId,payName,beginOfDay,endOfDay,settlementFlag);
	}

	@Override
	public Integer payNum(List<VehicleInfo> vehicleInfoList,String companyId, String payName, Date beginOfDay, Date endOfDay, Boolean settlementFlag) {
		return orderInfoDAO.payNum(vehicleInfoList,companyId,payName,beginOfDay,endOfDay,settlementFlag);
	}

	@Override
	public List<OrderInfo> findBySearchParams(Map<String, Object> searchParams,List<Sort> sortList) {
		return orderInfoDAO.search(searchParams,sortList);
	}
}