package com.jpsoft.bus.modules.bus.callback;

import cn.hutool.core.lang.UUID;
import com.jpsoft.bus.modules.base.entity.MergeOrderInfo;
import com.jpsoft.bus.modules.base.entity.OrderInfo;
import com.jpsoft.bus.modules.base.service.MergeOrderInfoService;
import com.jpsoft.bus.modules.base.service.OrderInfoService;
import com.jpsoft.bus.modules.bus.entity.*;
import com.jpsoft.bus.modules.bus.service.*;
import com.jpsoft.bus.modules.common.utils.CommonUtil;
import com.jpsoft.gps.callback.GpsDataCallback;
import com.jpsoft.gps.utils.GPSUtil;
import com.jpsoft.bus.config.GpsParamConfig;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.joda.time.Minutes;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

@Slf4j
@Component
public class GpsDataCallbackImpl implements GpsDataCallback {
    @Autowired
    private ValueOperations<String, Object> valueOperations;

    @Autowired
    private GpsDataInfoService gpsDataInfoService;

    @Autowired
    private GpsService gpsService;

    @Autowired
    private UserInfoService userInfoService;

    @Autowired
    private PassengerMessageService passengerMessageService;

    @Autowired
    private MergeOrderInfoService mergeOrderInfoService;

    @Autowired
    private OrderInfoService orderInfoService;

    @Autowired
    private StationInfoService stationInfoService;

    @Autowired
    private PassengerInfoService passengerInfoService;

    @Autowired
    private ShiftInfoService shiftInfoService;

    @Autowired
    private VehicleInfoService vehicleInfoService;

    @Autowired
    private GpsParamConfig gpsParamConfig;

    @Autowired
    private RabbitTemplate rabbitTemplate;

    @Override
    public void receive(String deviceNo, String longitude, String latitude,boolean manual) {
        log.warn("保存实时数据:{},{},{}", deviceNo, longitude, latitude);

        valueOperations.set(deviceNo + "_gps", longitude + "," + latitude, 1, TimeUnit.HOURS);

        if (Double.valueOf(longitude) > 0 && Double.valueOf(latitude) > 0) {
            String histKey = deviceNo+"_history_time";
            int intervalMinutes = 1;

            Long time = (Long)valueOperations.get(histKey);

            boolean allowInsert = false;

            if (time==null){
                allowInsert = true;
            }
            else{
                DateTime createTime = new DateTime(time);

                int minutes = Minutes.minutesBetween(createTime, DateTime.now()).getMinutes();

                if (minutes>=intervalMinutes){
                    allowInsert = true;
                }
            }

            GpsDataInfo gpsDataInfo0 = null;
            if(allowInsert) {
                log.warn("保存历史数据:{}", deviceNo);
                GpsDataInfo gpsDataInfo = new GpsDataInfo();
                gpsDataInfo.setId(UUID.randomUUID().toString());
                gpsDataInfo.setDeviceNo(deviceNo);
                gpsDataInfo.setCreateTime(new Date());
                gpsDataInfo.setLatitude(latitude);
                gpsDataInfo.setLongitude(longitude);
                gpsDataInfo.setDelFlag(false);


                gpsDataInfoService.insert(gpsDataInfo);
                gpsDataInfo0 = gpsDataInfo;

                valueOperations.set(histKey, (new Date()).getTime(), 1, TimeUnit.HOURS);
            }

            //更新车辆经纬度
            VehicleInfo v = vehicleInfoService.findByGpsDeviceNo(deviceNo);

            if (v != null) {
                log.warn("更新车辆经纬度:{}->({},{})", v.getLicensePlateNumber(), longitude, latitude);

                String lbsKey = "lbs_" + deviceNo;

                String address = (String) valueOperations.get(lbsKey);

                //10分钟进行一次逆向地址解析
                if (StringUtils.isEmpty(address)) {
                    address = GPSUtil.regeo(longitude, latitude, gpsParamConfig.getKey());
                    valueOperations.set(lbsKey, address, 10, TimeUnit.MINUTES);
                }

                v.setLatestAddress(address);
                v.setLongitude(longitude);//经度
                v.setLatitude(latitude);//纬度

                vehicleInfoService.updateGps(v);

                //更新车辆的当前站点
                try{
                    List<ShiftInfo> shiftInfoList = shiftInfoService.findByVehicleIdAndStatus(v.getId(),"1");

                    if (shiftInfoList.size()>0){
                        ShiftInfo shiftInfo = shiftInfoList.get(0);
                        String stationId = gpsService.getLatelyStation(v.getId(),manual);

                        StationInfo currentStation = stationInfoService.get(stationId);

                        //站点发生变化时
                        if (!shiftInfo.getCurrentStationId().equals(stationId)){
                            shiftInfo.setCurrentStationId(stationId);
                            shiftInfoService.update(shiftInfo);

                            rabbitTemplate.convertAndSend("stationRemindQueue", shiftInfo);

                            //当前站点
                          //  StationInfo currentStation = stationInfoService.get(stationId);
                            //此线路的所有站点
                            List<StationInfo> stationInfoList = stationInfoService.findByRouteId(shiftInfo.getRouteId());

                            int currentStationIndex = currentStation.getSortNo();

                            //线路终点站
                            StationInfo endStation = stationInfoService.get(shiftInfo.getEndStationId());
                            boolean frontToEnd = true;

                            if(endStation.getId().equals(stationInfoList.get(0).getId())){
                                //反向行驶
                                frontToEnd = false;
                            }

                            //查询乘客表是否有过站的情况
                            List<PassengerInfo> passengerInfoList = passengerInfoService.findByShiftStatusPayStatusNotTicketDown(shiftInfo.getId(),"1","1",stationId);

                            if (passengerInfoList.size()>0){
                                for (PassengerInfo passengerInfo : passengerInfoList){
                                    //乘客购票的下车站点
                                    StationInfo downStation = stationInfoService.get(passengerInfo.getTicketDownStationId());

                                    //按照线路顺行并且当前站点大于乘客购票站点，则过站
                                    if (frontToEnd){
                                        if(currentStationIndex > downStation.getSortNo()) {
                                            passengerInfo.setPayStatus("2");
                                        }
                                        else{
                                            passengerInfo.setPayStatus("1");
                                        }

                                        passengerInfoService.update(passengerInfo);
                                    }

                                    //按照线路逆行并且当前站点小于乘客购票站点，则过站
                                    if (!frontToEnd){
                                        if(currentStationIndex < downStation.getSortNo()) {
                                            passengerInfo.setPayStatus("2");
                                        }
                                        else {
                                            passengerInfo.setPayStatus("1");
                                        }

                                        passengerInfoService.update(passengerInfo);
                                    }

                                    //过站发消息
                                    if (passengerInfo.getPayStatus().equals("2")){
                                            List<OrderInfo> orderInfoList = orderInfoService.findByPassengerIdAndPayStatus(passengerInfo.getId(),20);
                                            OrderInfo orderInfo = orderInfoList.get(0);
                                            //综合账单
                                            MergeOrderInfo mergeOrderInfo = mergeOrderInfoService.get(orderInfo.getMergeOrderId());
                                            if (StringUtils.isNotBlank(mergeOrderInfo.getOpenId())){
//                                                UserInfo userInfo = userInfoService.findByOpenId(mergeOrderInfo.getOpenId());
//                                                if (userInfo.getAcceptMessage()){
                                                PassengerMessage passengerMessage = passengerMessageService.findByPassengerId(orderInfo.getPassengerId());

                                                if (passengerMessage ==null){
                                                    passengerMessage = new PassengerMessage();
                                                    passengerMessage.setId(UUID.randomUUID().toString());
                                                    passengerMessage.setOpenId(mergeOrderInfo.getOpenId());
                                                    passengerMessage.setPassengerId(passengerInfo.getId());
                                                    passengerMessage.setCreateTime(new Date());
                                                    passengerMessageService.insert(passengerMessage);
                                                }
                                                else {
                                                    passengerMessage.setReadStatus(false);
                                                    passengerMessageService.update(passengerMessage);
                                                }
//                                                }
                                            }
                                    }
                                }
                            }

                            //查询乘客有过站的情况
                            List<PassengerInfo> passengerInfoList1 = passengerInfoService.findByShiftStatusPayStatusNotTicketDown(shiftInfo.getId(),"1","2",stationId);
                            if (passengerInfoList1.size()>0){
                                for (PassengerInfo passengerInfo : passengerInfoList1){
                                    List<OrderInfo> orderInfoList = orderInfoService.findByPassengerIdAndPayStatus(passengerInfo.getId(),20);
                                    OrderInfo orderInfo = orderInfoList.get(0);
                                    //综合账单
                                    MergeOrderInfo mergeOrderInfo = mergeOrderInfoService.get(orderInfo.getMergeOrderId());
                                    if (StringUtils.isNotBlank(mergeOrderInfo.getOpenId())){

//                                        UserInfo userInfo = userInfoService.findByOpenId(mergeOrderInfo.getOpenId());
//                                        if (userInfo.getAcceptMessage()){
                                            PassengerMessage passengerMessage = passengerMessageService.findByPassengerId(orderInfo.getPassengerId());
                                            if (passengerMessage ==null){
                                                passengerMessage = new PassengerMessage();
                                                passengerMessage.setId(UUID.randomUUID().toString());
                                                passengerMessage.setOpenId(mergeOrderInfo.getOpenId());
                                                passengerMessage.setPassengerId(passengerInfo.getId());
                                                passengerMessage.setCreateTime(new Date());
                                                passengerMessageService.insert(passengerMessage);
                                            }
                                            else {
                                                passengerMessage.setReadStatus(false);
                                                passengerMessageService.update(passengerMessage);
                                            }
//                                        }
                                    }
                                }
                            }
                        }

                        if (gpsDataInfo0 != null){
                            gpsDataInfo0.setStationId(shiftInfo.getCurrentStationId());
                            gpsDataInfoService.update(gpsDataInfo0);
                        }

                        //判断车辆和当前站点的位置关系
//                        int distance = 0;
//
//                        if (currentStation.getRadius() == null){
//                            distance = 100;
//                        }

//                        if (gpsService.matchDistance(currentStation,v,distance)){
//                            shiftInfo.setCurrentStationStatus("1");
//                            shiftInfoService.update(shiftInfo);
//                        }else {
//                            shiftInfo.setCurrentStationStatus("2");
//                            shiftInfoService.update(shiftInfo);
//                        }
                    }
                }catch (Exception ex){
                    log.error(ex.getMessage(),ex);
                }
            }
        }
    }
}
