package com.jpsoft.bus.modules.bus.service.impl;

import cn.hutool.http.HttpRequest;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.google.gson.JsonObject;
import com.jpsoft.bus.modules.bus.service.BaiduService;
import com.jpsoft.bus.modules.common.utils.BaiduAuthUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

/**
 * @author 墨鱼_mo
 * @date 2021-4-6 11:05
 */
@Transactional
@Component(value = "baiduService")
public class BaiduServiceImpl implements BaiduService {

    @Autowired
    private BaiduAuthUtil baiduAuthUtil;


    @Override
    public Long faceSearch(String faceImageUrl, String faceType, String groupId) throws Exception {

        Long personId = null;
        // 请求url
        String url = "https://aip.baidubce.com/rest/2.0/face/v3/search";
        Map<String, Object> map = new HashMap<>();
        map.put("image", faceImageUrl);
        map.put("image_type", faceType);
        map.put("group_id_list", groupId);
        map.put("max_user_num", 1);

        // 注意这里仅为了简化编码每一次请求都去获取access_token，线上环境access_token有过期时间， 客户端可自行缓存，过期后重新获取。
        String accessToken = baiduAuthUtil.getAuth();

        String body = HttpRequest.post(url + "?access_token=" + accessToken)
                .form(map)
                .execute()
                .body();

        JSONObject jsonObject = JSONUtil.parseObj(body);
        String errorMsg = jsonObject.getStr("error_msg");
        Integer errorCode = jsonObject.getInt("error_code");
        if (!"SUCCESS".equals(errorMsg)) {
            throw new Exception(errorMsg);
        } else {
            JSONObject jsonObject1 = jsonObject.getJSONObject("result");

            JSONArray jsonArray = jsonObject1.getJSONArray("user_list");

            Map map1 = (Map) jsonArray.get(0);

            Double score = ((Double) map1.get("score"));

            if (score.compareTo(new Double(90))>0){
                Long userId = Long.parseLong((String) map1.get("user_id"));
                personId = userId;
            }

        }
        return personId;


    }

    @Override
    public void createFaceGroup(String faceGroupId) throws Exception {
        // 请求url
        String url = "https://aip.baidubce.com/rest/2.0/face/v3/faceset/group/add";

        Map<String, Object> map = new HashMap<>();
        map.put("group_id", faceGroupId);

        // 注意这里仅为了简化编码每一次请求都去获取access_token，线上环境access_token有过期时间， 客户端可自行缓存，过期后重新获取。
        String accessToken = baiduAuthUtil.getAuth();

        String body = HttpRequest.post(url + "?access_token=" + accessToken)
                .form(map)
                .execute()
                .body();

        JSONObject jsonObject = JSONUtil.parseObj(body);

        String errorMsg = jsonObject.getStr("error_msg");
        Integer errorCode = jsonObject.getInt("error_code");
        if (!"SUCCESS".equals(errorMsg)) {
            if (errorCode != 223101) {
                throw new Exception(errorMsg);
            }
        }


    }

    @Override
    public void faceUserAdd(String imageUrl, String groupId, String personId, String userInfo) throws Exception {

        // 请求url
        String url = "https://aip.baidubce.com/rest/2.0/face/v3/faceset/user/add";

        Map<String, Object> map = new HashMap<>();
        map.put("image", imageUrl);
        map.put("image_type", "URL");
        map.put("group_id", groupId);
        map.put("user_id", personId);
        map.put("user_info", userInfo);

        // 注意这里仅为了简化编码每一次请求都去获取access_token，线上环境access_token有过期时间， 客户端可自行缓存，过期后重新获取。
        String accessToken = baiduAuthUtil.getAuth();

        String body = HttpRequest.post(url + "?access_token=" + accessToken)
                .form(map)
                .execute()
                .body();
        JSONObject jsonObject = JSONUtil.parseObj(body);
        String errorMsg = jsonObject.getStr("error_msg");
        Integer errorCode = jsonObject.getInt("error_code");
        if (!"SUCCESS".equals(errorMsg)) {
            if (errorCode != 223105) {
                throw new Exception(errorMsg);
            }
        }

    }
}
