package com.jpsoft.bus.modules.bus.service.impl;

import java.math.BigDecimal;
import java.util.*;
import javax.annotation.Resource;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.google.common.collect.Ordering;
import com.jpsoft.bus.config.WxJpsoftConfig;
import com.jpsoft.bus.modules.base.dto.MergeOrderDTO;
import com.jpsoft.bus.modules.base.entity.MergeOrderInfo;
import com.jpsoft.bus.modules.base.entity.OrderInfo;
import com.jpsoft.bus.modules.base.entity.PaymentInfo;
import com.jpsoft.bus.modules.base.service.MergeOrderInfoService;
import com.jpsoft.bus.modules.base.service.OrderInfoService;
import com.jpsoft.bus.modules.base.service.PaymentInfoService;
import com.jpsoft.bus.modules.bus.dto.DriverBuyTicketDTO;
import com.jpsoft.bus.modules.bus.dto.PassengerRecordDTO;
import com.jpsoft.bus.modules.bus.entity.*;
import com.jpsoft.bus.modules.bus.service.*;
import com.jpsoft.bus.modules.common.utils.CommonUtil;
import com.jpsoft.bus.modules.pay.service.AlipayService;
import com.jpsoft.bus.modules.pay.service.WechatService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import com.jpsoft.bus.modules.bus.dao.PassengerInfoDAO;
import com.github.pagehelper.Page;
import com.jpsoft.bus.modules.common.dto.Sort;
import com.github.pagehelper.PageHelper;

@Slf4j
@Transactional
@Component(value = "passengerInfoService")
public class PassengerInfoServiceImpl implements PassengerInfoService {
    @Resource(name = "passengerInfoDAO")
    private PassengerInfoDAO passengerInfoDAO;

    @Autowired
    private MergeOrderInfoService mergeOrderInfoService;

    @Autowired
    private VehicleInfoService vehicleInfoService;

    @Autowired
    private ShiftInfoService shiftInfoService;

    @Autowired
    private StationInfoService stationInfoService;

    @Autowired
    private WxJpsoftConfig wxJpsoftConfig;

    @Autowired
    private PriceInfoService priceInfoService;

    @Autowired
    private CompanyInfoService companyInfoService;

    @Autowired
    private PaymentInfoService paymentInfoService;

    @Autowired
    private WechatService wechatService;

    @Autowired
    private AlipayService alipayService;

    @Autowired
    private OrderInfoService orderInfoService;

    @Autowired
    private BaiduService baiduService;

    @Override
    public PassengerInfo get(String id) {
        // TODO Auto-generated method stub
        return passengerInfoDAO.get(id);
    }

    @Override
    public int insert(PassengerInfo model) {
        // TODO Auto-generated method stub
        //model.setId(UUID.randomUUID().toString());

        return passengerInfoDAO.insert(model);
    }

    @Override
    public int update(PassengerInfo model) {
        // TODO Auto-generated method stub
        return passengerInfoDAO.update(model);
    }

    @Override
    public int delete(String id) {
        // TODO Auto-generated method stub
        return passengerInfoDAO.delete(id);
    }

    @Override
    public boolean exist(String id) {
        // TODO Auto-generated method stub
        int count = passengerInfoDAO.exist(id);

        return count > 0 ? true : false;
    }

    @Override
    public List<PassengerInfo> list() {
        // TODO Auto-generated method stub
        return passengerInfoDAO.list();
    }

    @Override
    public Page<PassengerInfo> pageSearch(Map<String, Object> searchParams, int pageNumber, int pageSize, boolean count, List<Sort> sortList) {
        Page<PassengerInfo> page = PageHelper.startPage(pageNumber, pageSize, count).doSelectPage(() -> {
            passengerInfoDAO.search(searchParams, sortList);
        });

        return page;
    }

    @Override
    public List<PassengerInfo> findByPersonIdShiftIdStatus(Long personId, String shiftId, String status) {
        return passengerInfoDAO.findByPersonIdShiftIdStatus(personId, shiftId, status);
    }

    @Override
    public void passengerFace(VehicleInfo vehicleInfo, ShiftInfo shiftInfo, String retFileUrl, String recordTime,
                              String stationId,Long libId, Long personId,String localImageUrl) throws Exception {
        Date recordDate = DateUtil.parse(recordTime, "yyyy-MM-dd HH:mm:ss");

        //查询是否有已上车的乘客记录
        List<PassengerInfo> passengerInfoList = findByPersonIdShiftIdStatus(personId, shiftInfo.getId(), "1");

        if (passengerInfoList.size() > 0) {
            PassengerInfo passengerInfo = passengerInfoList.get(0);

            boolean allow = true;

            //如果已过站则不改状态
            if("2".equals(passengerInfo.getPayStatus())) {
                log.warn("乘客:{}已过站，不修改状态",passengerInfo.getId());
                allow = false;
            }
            else if("0".equals(passengerInfo.getPayStatus())){
                //未购票，同时已过站
                if (!passengerInfo.getUpStationId().equals(shiftInfo.getCurrentStationId())){
                    log.warn("乘客:{}未购票且已过站，不修改状态", passengerInfo.getId());
                    allow = false;
                }
            }

            if(allow){
                passengerInfo.setLocalImageUrl(localImageUrl);
                passengerInfo.setDownTime(recordDate);
                passengerInfo.setDownStationId(stationId);
                passengerInfo.setStatus("2");
                update(passengerInfo);
            }
        } else {
            //查询此班次是否有已下车的记录
            List<PassengerInfo> passengerInfoList1 = findByPersonIdShiftIdStatus(personId, shiftInfo.getId(), "2");
            if (passengerInfoList1.size() == 0) {
                PassengerInfo passengerInfo = new PassengerInfo();
                passengerInfo.setId(UUID.randomUUID().toString());
                passengerInfo.setVehicleShiftId(shiftInfo.getId());
                passengerInfo.setVehicleLibId(libId);
                passengerInfo.setVehiclePersonId(personId);
                passengerInfo.setUpTime(recordDate);
                passengerInfo.setUpStationId(stationId);
                passengerInfo.setStatus("1");
                passengerInfo.setPayStatus("0");
                passengerInfo.setCreateTime(new Date());

                passengerInfo.setLocalImageUrl(localImageUrl);

                if (StringUtils.isNotEmpty(retFileUrl)) {
                    passengerInfo.setImageUrl(retFileUrl);
                } else {
                    //如果没有上车记录，但有已下车记录，则读取之前记录的头像信息
                    List<PassengerInfo> downList = findByPersonIdShiftIdStatus(personId, shiftInfo.getId(), "2");

                    if (downList.size() > 0) {
                        passengerInfo.setImageUrl(downList.get(0).getImageUrl());
                    }
                }

                insert(passengerInfo);

                //暂时不进行服务端人脸比对
//                String groupId = CommonUtil.getProvinceNum(vehicleInfo.getLicensePlateNumber()) + StrUtil.sub(vehicleInfo.getLicensePlateNumber(), 1, 10);
//
//                if (StringUtils.isNotEmpty(groupId)) {
//                    //将图片上传到百度AI人脸识别,  userInfo 1：司机，2：乘客
//                    baiduService.faceUserAdd(retFileUrl, groupId, String.valueOf(personId), "2");
//                }
            } else {
                PassengerInfo passengerInfo = passengerInfoList1.get(0);
                passengerInfo.setStatus("1");
                update(passengerInfo);
            }
        }
    }

    @Override
    public DriverBuyTicketDTO driverBuyTicket(PassengerInfo passengerInfo, String ticketUpStationId, String ticketDownStationId, String ticketType, String goodsTicket, String totalFee, String payName, String paymentId) throws Exception {

        DriverBuyTicketDTO driverBuyTicketDTO = new DriverBuyTicketDTO();


        //乘客记录添加购票上车点和下车点
		/*passengerInfo.setTicketUpStationId(ticketUpStationId);
		passengerInfo.setTicketDownStationId(ticketDownStationId);
		update(passengerInfo);*/


        //根据支付方式创建订单
        if ("cash".equals(payName) || "ticket".equals(payName)) {
            //创建综合账单
            MergeOrderInfo mergeOrderInfo = new MergeOrderInfo();
            mergeOrderInfo.setId(UUID.randomUUID().toString());
            mergeOrderInfo.setVehicleShiftId(passengerInfo.getVehicleShiftId());
            //当前车辆所属公司
            ShiftInfo shiftInfo = shiftInfoService.get(passengerInfo.getVehicleShiftId());
            VehicleInfo vehicleInfo = vehicleInfoService.get(shiftInfo.getVehicleId());

            mergeOrderInfo.setPayStatus(20);
            mergeOrderInfo.setCompanyId(vehicleInfo.getCompanyId());
            mergeOrderInfo.setOutOrderNo(wxJpsoftConfig.getUrlKey() + com.jpsoft.bus.modules.common.utils.StringUtils.getOutTradeNo());
            mergeOrderInfo.setPayName(payName);
            mergeOrderInfo.setPaymentId(paymentId);
            mergeOrderInfo.setCreateTime(new Date());
            mergeOrderInfoService.insert(mergeOrderInfo);

            OrderInfo orderInfo = new OrderInfo();
            orderInfo.setId(UUID.randomUUID().toString());
            orderInfo.setBody("买车票");
            orderInfo.setTotalFee(new BigDecimal(totalFee));
            orderInfo.setPayStatus(20);
            orderInfo.setPayName(payName);
            orderInfo.setPayFee(new BigDecimal(totalFee));
            orderInfo.setPayTime(new Date());
            orderInfo.setOutOrderNo(com.jpsoft.bus.modules.common.utils.StringUtils.getOutTradeNo());
            orderInfo.setMergeOrderId(mergeOrderInfo.getId());
            orderInfo.setTransactionId(orderInfo.getOutOrderNo());
            orderInfo.setPassengerId(passengerInfo.getId());
            orderInfo.setTicketType(ticketType);
            orderInfo.setCreateTime(new Date());
            orderInfo.setGoodsTicket(new BigDecimal(goodsTicket));
            orderInfo.setTicketUpStationId(ticketUpStationId);
            orderInfo.setTicketDownStationId(ticketDownStationId);
            orderInfoService.insert(orderInfo);

            //乘客记录添加购票上车点和下车点
            passengerInfo.setTicketUpStationId(ticketUpStationId);
            passengerInfo.setTicketDownStationId(ticketDownStationId);
            passengerInfo.setTicketType(ticketType);

            //乘客购票状态修改
            passengerInfo.setPayStatus("1");
            update(passengerInfo);


        } else {

            //创建综合账单
            MergeOrderInfo mergeOrderInfo = new MergeOrderInfo();
            mergeOrderInfo.setId(UUID.randomUUID().toString());
            mergeOrderInfo.setVehicleShiftId(passengerInfo.getVehicleShiftId());
            mergeOrderInfo.setPayStatus(10);

            //当前车辆所属公司
            ShiftInfo shiftInfo = shiftInfoService.get(passengerInfo.getVehicleShiftId());
            VehicleInfo vehicleInfo = vehicleInfoService.get(shiftInfo.getVehicleId());

            mergeOrderInfo.setCompanyId(vehicleInfo.getCompanyId());
            mergeOrderInfo.setOutOrderNo(wxJpsoftConfig.getUrlKey() + com.jpsoft.bus.modules.common.utils.StringUtils.getOutTradeNo());
            mergeOrderInfo.setPaymentId(paymentId);
            mergeOrderInfo.setCreateTime(new Date());
            mergeOrderInfoService.insert(mergeOrderInfo);

            OrderInfo orderInfo = new OrderInfo();
            orderInfo.setId(UUID.randomUUID().toString());
            orderInfo.setBody("买车票");
            orderInfo.setTotalFee(new BigDecimal(totalFee));
            orderInfo.setPayStatus(10);
            orderInfo.setOutOrderNo(com.jpsoft.bus.modules.common.utils.StringUtils.getOutTradeNo());
            orderInfo.setMergeOrderId(mergeOrderInfo.getId());
            orderInfo.setPassengerId(passengerInfo.getId());
            orderInfo.setTicketType(ticketType);
            orderInfo.setGoodsTicket(new BigDecimal(goodsTicket));
            orderInfo.setCreateTime(new Date());
            orderInfo.setTicketUpStationId(ticketUpStationId);
            orderInfo.setTicketDownStationId(ticketDownStationId);
            orderInfoService.insert(orderInfo);

            String codeUrl = "";
            if ("wechat".equals(payName)) {
                codeUrl = wechatService.wxNativePay(mergeOrderInfo.getId());
            }
            if ("alipay".equals(payName)) {
                codeUrl = alipayService.tradePrecreatePay(mergeOrderInfo.getId());
            }

            driverBuyTicketDTO.setId(orderInfo.getId());
            driverBuyTicketDTO.setPayName(payName);
            driverBuyTicketDTO.setTotalFee(orderInfo.getTotalFee());
            driverBuyTicketDTO.setUrl(codeUrl);
            driverBuyTicketDTO.setPayNameStr(orderInfo.getPayNameStr(payName));
        }

        return driverBuyTicketDTO;
    }

    @Override
    public List<PassengerInfo> findByShiftStatusPayStatusNotTicketDown(String shiftId, String status, String payStatus, String ticketDownStationId) {
        return passengerInfoDAO.findByShiftStatusPayStatusNotTicketDown(shiftId, status, payStatus, ticketDownStationId);
    }

    @Override
    public MergeOrderDTO createOrder(VehicleInfo vehicleInfo, ShiftInfo shiftInfo, String openId, String mergeOrderId, String passengerIds, String ticketUpStationId, String ticketDownStationId, String ticketTypes, String goodsTicket, String totalFee) throws Exception {


        MergeOrderDTO mergeOrderDTO = new MergeOrderDTO();

        //乘客id集合
        List<String> passengerList = Arrays.asList(passengerIds.split(","));
        List<String> ticketTypeList = Arrays.asList(ticketTypes.split(","));
        if (passengerList.size() == 0) {
            throw new Exception("乘客不存在");
        }
        if (passengerList.size() != ticketTypeList.size()){
            throw new Exception("乘客数和购票类型数不一致");
        }


        PriceInfo priceInfo = priceInfoService.findByStartStationAndEndStation(ticketUpStationId,ticketDownStationId);
        if (priceInfo == null || priceInfo.getPrice().compareTo(BigDecimal.ZERO) <=0){
            PriceInfo priceInfo1 = priceInfoService.findByStartStationAndEndStation(ticketDownStationId,ticketUpStationId);
            if (priceInfo1 == null || priceInfo1.getPrice().compareTo(BigDecimal.ZERO) <= 0){
                throw new Exception("站点间无费用设置");
            }
            priceInfo = priceInfo1;
        }

        BigDecimal price = priceInfo.getPrice();

        BigDecimal goodTicketPrice = new BigDecimal(goodsTicket);

        BigDecimal total = goodTicketPrice;



       /* BigDecimal totalTicketPrice =price.multiply(new BigDecimal(passengerList.size()));

        BigDecimal goodTicketPrice = new BigDecimal(goodsTicket);

        BigDecimal total = totalTicketPrice.add(goodTicketPrice);
        if (total.compareTo(new BigDecimal(totalFee)) != 0) {
            throw new Exception("票价有误，请重新创建");
        }*/


        String paymentId = "";
        CompanyInfo companyInfo = companyInfoService.get(vehicleInfo.getCompanyId());
        PaymentInfo paymentInfo = paymentInfoService.findByCompanyId(vehicleInfo.getCompanyId());
        if (paymentInfo != null) {
            paymentId = paymentInfo.getId();
        } else {
            if ("1".equals(companyInfo.getType())) {
                throw new Exception("车辆对应公司未配置收款账户");
            }
            if (!"1".equals(companyInfo.getType())) {
                CompanyInfo companyInfo1 = companyInfoService.get(companyInfo.getParentId());
                PaymentInfo paymentInfo1 = paymentInfoService.findByCompanyId(companyInfo1.getId());
                if (paymentInfo1 != null) {
                    paymentId = paymentInfo1.getId();
                }

            }
        }

        if (StringUtils.isBlank(paymentId)) {
            throw new Exception("对应公司收款账户异常，请核验");
        }



        String mergeId = mergeOrderId;
        if (StringUtils.isBlank(mergeOrderId)){
            MergeOrderInfo mergeOrderInfo = new MergeOrderInfo();
            mergeOrderInfo.setId(UUID.randomUUID().toString());
            mergeOrderInfo.setVehicleShiftId(shiftInfo.getId());

            //当前车辆所属公司
           // ShiftInfo shiftInfo = shiftInfoService.get(passengerInfo.getVehicleShiftId());
          //  VehicleInfo vehicleInfo = vehicleInfoService.get(shiftInfo.getVehicleId());

            mergeOrderInfo.setCompanyId(vehicleInfo.getCompanyId());
            mergeOrderInfo.setOpenId(openId);
            mergeOrderInfo.setPayStatus(10);
            mergeOrderInfo.setOutOrderNo(wxJpsoftConfig.getUrlKey() + com.jpsoft.bus.modules.common.utils.StringUtils.getOutTradeNo());
            mergeOrderInfo.setPaymentId(paymentId);
            mergeOrderInfo.setCreateTime(new Date());
            mergeOrderInfoService.insert(mergeOrderInfo);
            mergeId = mergeOrderInfo.getId();
        }

        mergeOrderDTO.setId(mergeId);

        PassengerInfo passengerInfo0 = get(passengerList.get(0));

        String upStationId = passengerInfo0.getUpStationId();

        Boolean goodTicketStatus = false;
        List<PassengerRecordDTO> passengerRecordDTOList = new ArrayList<>();
     //   for (String passengerId : passengerList) {

        for (int n = 0;n<passengerList.size();n++){
            BigDecimal price1 = price;
            String passengerId = passengerList.get(n);
            String ticketType = ticketTypeList.get(n);
            if ("1".equals(ticketType)) {
                price1 = price1.divide(new BigDecimal(2));
            }

            total = total.add(price1);

            PassengerRecordDTO passengerRecordDTO = new PassengerRecordDTO();

            OrderInfo orderInfo0 = orderInfoService.findByPassengerIdMergeOrderId(passengerId,mergeId);
            if (orderInfo0 != null){
                throw new Exception("有乘客已加入此账单，不可重复添加");
            }

            PassengerInfo passengerInfo = get(passengerId);
            if (!passengerInfo.getUpStationId().equals(upStationId)){
                throw new Exception("上车站点不一致不可同时选择");
            }
            if ("1".equals(passengerInfo.getStatus())) {
                if (!"1".equals(passengerInfo.getPayStatus())){
                    //创建订单
                    OrderInfo orderInfo = new OrderInfo();
                    orderInfo.setId(UUID.randomUUID().toString());
                    orderInfo.setTotalFee(price1);
                    orderInfo.setBody("买车票");
                    orderInfo.setPayStatus(10);
                    orderInfo.setOutOrderNo(com.jpsoft.bus.modules.common.utils.StringUtils.getOutTradeNo());
                    orderInfo.setCreateTime(new Date());
                    orderInfo.setPaymentId(paymentId);
                    orderInfo.setPassengerId(passengerId);
                    orderInfo.setTicketType(ticketType);

                    if (!goodTicketStatus){
                        orderInfo.setGoodsTicket(new BigDecimal(goodsTicket));
                        orderInfo.setTotalFee(price1.add(new BigDecimal(goodsTicket)));
                        goodTicketStatus = true;
                    }

                    orderInfo.setMergeOrderId(mergeId);
                    orderInfo.setTicketUpStationId(ticketUpStationId);
                    orderInfo.setTicketDownStationId(ticketDownStationId);
                    orderInfoService.insert(orderInfo);

                    passengerRecordDTO.setId(orderInfo.getId());
                    passengerRecordDTO.setImageUrl(passengerInfo.getImageUrl());

                    //开始站点
                    StationInfo startStation = stationInfoService.get(orderInfo.getTicketUpStationId());
                    //终点站
                    StationInfo endStation = stationInfoService.get(orderInfo.getTicketDownStationId());
                    passengerRecordDTO.setStartStation(startStation.getName());
                    passengerRecordDTO.setEndStation(endStation.getName());
                    passengerRecordDTO.setTicketType(orderInfo.getTicketType());
                    passengerRecordDTO.setTicketTypeName(passengerRecordDTO.getTicketTypeName(passengerRecordDTO.getTicketType()));
                    passengerRecordDTO.setGoodTicket(orderInfo.getGoodsTicket());
                    passengerRecordDTO.setPayTime(DateUtil.format(orderInfo.getPayTime(),"yyyy-MM-dd HH:mm:ss"));
                    passengerRecordDTO.setTotalFee(orderInfo.getTotalFee());
                    passengerRecordDTOList.add(passengerRecordDTO);

                }else {
                    throw new Exception("乘客已购票");
                }



            }else {
                throw new Exception("乘客已下车");
            }
        }

        if (total.compareTo(new BigDecimal(totalFee)) != 0) {
            throw new Exception("票价有误，请重新创建");
        }
        mergeOrderDTO.setPassengerRecordDTOList(passengerRecordDTOList);


        return mergeOrderDTO;
    }

    @Override
    public void updatePassengerOrder(OrderInfo orderInfo, String ticketUpStationId, String ticketDownStationId, String ticketType, String goodsTicket, String totalFee) {

        orderInfo.setTicketUpStationId(ticketUpStationId);
        orderInfo.setTicketDownStationId(ticketDownStationId);
        orderInfo.setTotalFee(new BigDecimal(totalFee));
        orderInfo.setTicketType(ticketType);
        orderInfo.setGoodsTicket(new BigDecimal(goodsTicket));
        orderInfoService.update(orderInfo);
    }

    @Override
    public List<PassengerInfo> findByOpenIdAndStatus(String openId, String status,String payStatus) {
        return passengerInfoDAO.findByOpenIdAndStatus(openId,status,payStatus);
    }

    @Override
    public List<PassengerInfo> findByOpenIdAndVehicleShiftId(String openId, String vehicleShiftId, String status, String payStatus) {
        return passengerInfoDAO.findByOpenIdAndVehicleShiftId(openId,vehicleShiftId,status,payStatus);
    }

    @Override
    public List<PassengerInfo> findByShiftIdAndStatus(String shiftId, String status) {
        return passengerInfoDAO.findByShiftIdAndStatus(shiftId,status);
    }
}