package com.jpsoft.bus.modules.common.utils;

import cn.hutool.core.date.DateUtil;
import com.jpsoft.bus.modules.bus.entity.StationInfo;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.Random;
import java.util.UUID;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 字符串工具类，继承lang3字符串工具类
 * @author L.com
 */
public final class StringUtils extends org.apache.commons.lang3.StringUtils {

	private static final double PI = 3.14159265;
	private static final double EARTH_RADIUS = 6378137;    //赤道半径
	private static final double RAD = Math.PI / 180.0;


	public static String encode(String str){
		String encode=null;
		try {
			encode = URLEncoder.encode(str, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		return encode;
	}

	/**
	 * 获取UUID，去掉`-`的
	 * @return {String}
	 *
	 */
	public static String generateStr () {
		return UUID.randomUUID().toString().replace("-", "");
	}


	/**
	 * 要求外部订单号必须唯一。
	 * @return {String}
	 */
	public synchronized static String getOutTradeNo() {
		/*SimpleDateFormat format = new SimpleDateFormat("MMddHHmmss", Locale.getDefault());
		Date date = new Date();
		String key = format.format(date);
		DateUtil.formatDateTime(new Date());
		Integer ran = (int)((Math.random()*9+1)*100000);*/

		String key = System.currentTimeMillis() + "cxd" + (int) (Math.random() * 900000000 + 100000000);
	//	key = key +ran.toString();
	//	key = key.substring(0, 24);
		return key;
	}


	/**
	 * 提现单号。
	 * @return {String}
	 */
	public synchronized static String getApplicationNo() {
		SimpleDateFormat format = new SimpleDateFormat("MMddHHmmss", Locale.getDefault());
		Date date = new Date();
		String key = format.format(date);
		Integer ran = (int)((Math.random()*9+1)*100000);
		key = "W"+key +ran.toString();
		return key;
	}

	/**
	 * 字符串格式化
	 *
	 * use: format("my name is {0}, and i like {1}!", "L.cm", "java")
	 *
	 * int long use {0,number,#}
	 *
	 * @param s
	 * @param args
	 * @return {String}转换后的字符串
	 */
	public static String format(String s, Object... args) {
		return MessageFormat.format(s, args);
	}

	/**
	 * 替换某个字符
	 * @param str
	 * @param regex
	 * @param args
	 * @return {String}
	 */
	public static String replace(String str,String regex,String... args){
		int length = args.length;
		for (int i = 0; i < length; i++) {
			str=str.replaceFirst(regex, args[i]);
		}
		return str;
	}

	/**
	 * 清理字符串，清理出某些不可见字符
	 * @param txt
	 * @return {String}
	 */
	public static String cleanChars(String txt) {
		return txt.replaceAll("[ 　	`·•�\\f\\t\\v]", "");
	}

	/**
	 *  随机字符串
	 */
	private static final String INT_TEMP = "0123456789";
	private static final String STR_TEMP = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
	private static final String ALL_TEMP = INT_TEMP + STR_TEMP;

	private static final Random RANDOM = new Random();

	/**
	 * 生成的随机数类型
	 */
	public enum RandomType {
		/**
		 * 整数
		 */
		INT,
		/**
		 * 字符串
		 */
		STRING,
		/**
		 * 所有类型
		 */
		ALL
	}

	/**
	 * 随机数生成
	 * @param count
	 * @return {String}
	 */
	public static String random(int count, RandomType randomType) {
		if (count == 0) {
			return "";
		}
		if (count < 0) {
			throw new IllegalArgumentException("Requested random string length " + count + " is less than 0.");
		}
		char[] buffer = new char[count];
		for (int i = 0; i < count; i++) {
			if (randomType.equals(RandomType.INT)) {
				buffer[i] = INT_TEMP.charAt(RANDOM.nextInt(INT_TEMP.length()));
			} else if (randomType.equals(RandomType.STRING)) {
				buffer[i] = STR_TEMP.charAt(RANDOM.nextInt(STR_TEMP.length()));
			}else {
				buffer[i] = ALL_TEMP.charAt(RANDOM.nextInt(ALL_TEMP.length()));
			}
		}
		return new String(buffer);
	}

	/**
	 * 判断是否手机号
	 * @param phone
	 * @return
	 */
	public static boolean isMobilePhone(String phone) {
		boolean flag=true;
		String regex = "^((13[0-9])|(14[5,7,9])|(15([0-3]|[5-9]))|(166)|(17[0,1,3,5,6,7,8])|(18[0-9])|(19[1,8,9]))\\d{8}$";
		if (phone.length() != 11) {
			flag= false;
		} else {
			Pattern p = Pattern.compile(regex);
			Matcher m = p.matcher(phone);
			flag = m.matches();
		}

		return flag;
	}

	public static double getDistance(double lng1, double lat1, double lng2, double lat2)
	{
		double radLat1 = lat1*RAD;
		double radLat2 = lat2*RAD;
		double a = radLat1 - radLat2;
		double b = (lng1 - lng2)*RAD;
		double s = 2 * Math.asin(Math.sqrt(Math.pow(Math.sin(a/2),2) +
				Math.cos(radLat1)*Math.cos(radLat2)*Math.pow(Math.sin(b/2),2)));
		s = s * EARTH_RADIUS;
		s = Math.round(s * 10000) / 10000;
		return s;
	}

	public static String strValue(Object object,String defaultValue) {
		if (object!=null){
			return String.valueOf(object);
		}
		else{
			return defaultValue;
		}
	}


	public static void main(String[] args) {

		int currentIndex = 3;

		int n = 0;
		for (int i=0;i<7;i++) {


			if(i <= currentIndex){
				//正向行驶时，排序比当前站小的站点不用计算
				continue;
			}
			n++;
			if (n>=2){
				break;
			}
		}


	}
}