package com.jpsoft.bus.modules.pay.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.alipay.api.AlipayClient;
import com.alipay.api.DefaultAlipayClient;
import com.alipay.api.request.AlipayTradePrecreateRequest;
import com.alipay.api.request.AlipayTradeRefundRequest;
import com.alipay.api.response.AlipayTradePrecreateResponse;
import com.alipay.api.response.AlipayTradeRefundResponse;
import com.jpsoft.bus.config.AliPayConfig;
import com.jpsoft.bus.modules.base.entity.MergeOrderInfo;
import com.jpsoft.bus.modules.base.entity.OrderInfo;
import com.jpsoft.bus.modules.base.entity.PaymentInfo;
import com.jpsoft.bus.modules.base.service.MergeOrderInfoService;
import com.jpsoft.bus.modules.base.service.OrderInfoService;
import com.jpsoft.bus.modules.base.service.PaymentInfoService;
import com.jpsoft.bus.modules.pay.service.AlipayService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author 墨鱼_mo
 * @date 2021-4-9 14:51
 */
@Transactional
@Component(value = "alipayService")
public class AlipayServiceImpl implements AlipayService {

    @Autowired
    private AliPayConfig aliPayConfig;

    @Autowired
    private OrderInfoService orderInfoService;

    @Autowired
    private PaymentInfoService paymentInfoService;

    @Autowired
    private MergeOrderInfoService mergeOrderInfoService;


    @Override
    public String tradePrecreatePay(String mergeOrderId) throws Exception {

        AlipayClient alipayClient = new DefaultAlipayClient(aliPayConfig.getServiceUrl(), aliPayConfig.getAppId(), aliPayConfig.getPrivateKey(), "json", aliPayConfig.getInputCharset(), aliPayConfig.getZfbPublicKey(), aliPayConfig.getSignType());
        MergeOrderInfo mergeOrderInfo = mergeOrderInfoService.get(mergeOrderId);

        List<OrderInfo> orderInfoList = orderInfoService.findByMergeOrderId(mergeOrderId);
        if (orderInfoList.size() == 0) {
            throw new Exception("订单错误");
        }
        BigDecimal totalFee = BigDecimal.ZERO;
        for (OrderInfo orderInfo : orderInfoList) {
            if (orderInfo.getPayStatus() != 10) {
                throw new Exception("订单错误");
            }
            totalFee = totalFee.add(orderInfo.getTotalFee());
        }


        PaymentInfo paymentInfo = paymentInfoService.get(mergeOrderInfo.getPaymentId());
        AlipayTradePrecreateRequest request = new AlipayTradePrecreateRequest();
        request.setNotifyUrl(paymentInfo.getAlipayNotifyUrl());


        String appAuthToken = paymentInfo.getAppAuthToken();

        Map<String, Object> maps = new HashMap<>();
        maps.put("out_trade_no", mergeOrderInfo.getOutOrderNo());
        maps.put("total_amount", String.valueOf(totalFee));
        maps.put("subject", orderInfoList.get(0).getBody());
        maps.put("timeout_express", "5m");

        JSONObject extendParams = new JSONObject();

        extendParams.put("sys_service_provider_id", paymentInfo.getMchId());
        maps.put("extend_params", extendParams);
        request.putOtherTextParam("app_auth_token", appAuthToken);
        //把订单信息转换为json对象的字符串
        String postdata = JSONObject.toJSONString(maps);
        request.setBizContent(postdata);
        AlipayTradePrecreateResponse response = alipayClient.execute(request);
        String body = response.getBody();
        JSONObject jsonObject = JSONObject.parseObject(body);
        String alipayTradePrecreateResponse = jsonObject.getString("alipay_trade_precreate_response");
        JSONObject jsonResponse = JSONObject.parseObject(alipayTradePrecreateResponse);

        if (!jsonResponse.getString("code").equals("10000")) {
            throw new Exception(jsonResponse.getString("sub_msg"));
        }

        String qr_code = jsonObject.getJSONObject("alipay_trade_precreate_response").getString("qr_code");

        return qr_code;
    }

    @Override
    public Boolean aliPayRefund(OrderInfo orderInfo) throws Exception {




        Integer payStatus = 40;

        MergeOrderInfo mergeOrderInfo = mergeOrderInfoService.get(orderInfo.getMergeOrderId());


        AlipayClient alipayClient = new DefaultAlipayClient(aliPayConfig.getServiceUrl(), aliPayConfig.getAppId(), aliPayConfig.getPrivateKey(), "json", aliPayConfig.getInputCharset(), aliPayConfig.getZfbPublicKey(), aliPayConfig.getSignType());
        AlipayTradeRefundRequest request = new AlipayTradeRefundRequest();


        Map<String, Object> maps = new HashMap<>();
        maps.put("trade_no", orderInfo.getTransactionId());
        maps.put("out_trade_no",mergeOrderInfo.getOutOrderNo());
        maps.put("refund_reason", "正常退款");
        maps.put("refund_amount", orderInfo.getPayFee());


        JSONObject extendParams = new JSONObject();

        extendParams.put("sys_service_provider_id", aliPayConfig.getMchId());
        maps.put("extend_params", extendParams);
        request.putOtherTextParam("app_auth_token", aliPayConfig.getAppAuthToken());
        //把订单信息转换为json对象的字符串
        String postdata = JSONObject.toJSONString(maps);
        request.setBizContent(postdata);


        AlipayTradeRefundResponse response = alipayClient.execute(request);

        //退款成功
        if ("10000".equals(response.getCode())){
            orderInfo.setRefundFee(orderInfo.getPayFee());
            orderInfo.setPayStatus(payStatus);
            orderInfoService.update(orderInfo);

        }else {
            throw new Exception(response.getSubMsg());
        }

        return true;


    }
}
