package com.jpsoft.bus.modules.pay.service.impl;

import com.ijpay.core.enums.SignType;
import com.ijpay.core.enums.TradeType;
import com.ijpay.core.kit.WxPayKit;
import com.ijpay.wxpay.WxPayApi;
import com.ijpay.wxpay.model.RefundModel;
import com.ijpay.wxpay.model.UnifiedOrderModel;
import com.jpsoft.bus.config.WxJpsoftConfig;
import com.jpsoft.bus.modules.base.dto.MergeOrderDTO;
import com.jpsoft.bus.modules.base.entity.MergeOrderInfo;
import com.jpsoft.bus.modules.base.entity.OrderInfo;
import com.jpsoft.bus.modules.base.entity.PaymentInfo;
import com.jpsoft.bus.modules.base.service.MergeOrderInfoService;
import com.jpsoft.bus.modules.base.service.OrderInfoService;
import com.jpsoft.bus.modules.base.service.PaymentInfoService;
import com.jpsoft.bus.modules.pay.service.WechatService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

/**
 * @author 墨鱼_mo
 * @date 2021-4-9 14:51
 */
@Transactional
@Component(value = "wechatService")
public class WechatServiceImpl implements WechatService {

    @Autowired
    private OrderInfoService orderInfoService;

    @Autowired
    private PaymentInfoService paymentInfoService;

    @Autowired
    private MergeOrderInfoService mergeOrderInfoService;

    @Autowired
    private WxJpsoftConfig wxJpsoftConfig;


    @Override
    public String wxNativePay(String mergeOrderId) throws Exception {


        if (StringUtils.isBlank(mergeOrderId)) {
            throw new Exception("订单id为空");
        }

        MergeOrderInfo mergeOrderInfo = mergeOrderInfoService.get(mergeOrderId);

        List<OrderInfo> orderInfoList = orderInfoService.findByMergeOrderId(mergeOrderId);
        if (orderInfoList.size() ==0){
            throw new Exception("订单错误");
        }
        BigDecimal totalFee = BigDecimal.ZERO;
        for (OrderInfo orderInfo : orderInfoList){
            if (orderInfo.getPayStatus() != 10) {
                throw new Exception("订单支付状态异常");
            }
            totalFee = totalFee.add(orderInfo.getTotalFee());
        }



        PaymentInfo paymentInfo = paymentInfoService.get(mergeOrderInfo.getPaymentId());

        String subMchId = paymentInfo.getSubMchId();

        //金额处理
        int wxTotalTee = totalFee.multiply(new BigDecimal(100)).intValue();

        Map<String, String> params = UnifiedOrderModel
                .builder()
                .appid(paymentInfo.getAppId())
                .mch_id(paymentInfo.getMchId())
                .sub_mch_id(subMchId)
                .nonce_str(WxPayKit.generateStr())
                .body(orderInfoList.get(0).getBody())
                .out_trade_no(mergeOrderInfo.getOutOrderNo())
                .total_fee(String.valueOf(wxTotalTee))
                .spbill_create_ip(wxJpsoftConfig.getIp())
                .notify_url(paymentInfo.getWechatNotifyUrl())
                .trade_type(TradeType.NATIVE.getTradeType())
                .build()
                .createSign(wxJpsoftConfig.getMchKey(), SignType.HMACSHA256);

        String xmlResult = WxPayApi.pushOrder(false, params);

        Map<String, String> result = WxPayKit.xmlToMap(xmlResult);

        String returnCode = result.get("return_code");
        String returnMsg = result.get("return_msg");
        System.out.println(returnMsg);
        if (!WxPayKit.codeIsOk(returnCode)) {
            throw new Exception(returnMsg);
        }
        String resultCode = result.get("result_code");
        if (!WxPayKit.codeIsOk(resultCode)) {
            throw new Exception(returnMsg);
        }
        //生成预付订单success
        String qrCodeUrl = result.get("code_url");


        return qrCodeUrl;
    }

    @Override
    public Boolean wxRefund(OrderInfo orderInfo) throws Exception {



        Integer payStatus = 40;

        MergeOrderInfo mergeOrderInfo = mergeOrderInfoService.get(orderInfo.getMergeOrderId());

        PaymentInfo paymentInfo = paymentInfoService.get(mergeOrderInfo.getPaymentId());


        List<OrderInfo> orderInfoList = orderInfoService.findByMergeOrderId(mergeOrderInfo.getId());
        BigDecimal totalDecimal = BigDecimal.ZERO;
        if (orderInfoList.size()>0){
            for (OrderInfo orderInfo1 : orderInfoList){
                if (orderInfo1.getPayStatus() != 10){
                    totalDecimal = totalDecimal.add(orderInfo1.getPayFee());
                }
            }
        }


        String totalFee = totalDecimal.multiply(new BigDecimal(100)).setScale(0).toString();
        String refundFeeStr = orderInfo.getPayFee().multiply(new BigDecimal(100)).setScale(0).toString();


        Map<String, String> params = RefundModel.builder()
                .appid(wxJpsoftConfig.getAppId())
                .mch_id(wxJpsoftConfig.getMchId())
                .sub_mch_id(paymentInfo.getSubMchId())
                .nonce_str(WxPayKit.generateStr())
                .transaction_id(orderInfo.getTransactionId())
                .out_trade_no(mergeOrderInfo.getOutOrderNo())
                .out_refund_no(WxPayKit.generateStr())
                .total_fee(totalFee)
                .refund_fee(refundFeeStr)
                .notify_url(wxJpsoftConfig.getRefundUrl())
                .build()
                .createSign(wxJpsoftConfig.getMchKey(), SignType.HMACSHA256);
        //  .createSign(wxTransferConfig.getKey(), SignType.HMACSHA256);

        //  return WxPayApi.orderRefund(false, params, wxTransferConfig.getCertPath(), wxTransferConfig.getMchId());
        String result = WxPayApi.orderRefund(false, params, wxJpsoftConfig.getCertPath(), wxJpsoftConfig.getMchId());
        Map<String, String> map = WxPayKit.xmlToMap(result);
        String returnCode = map.get("return_code");
        String resultCode = map.get("result_code");
        if (WxPayKit.codeIsOk(returnCode) && WxPayKit.codeIsOk(resultCode)) {

            orderInfo.setRefundFee(orderInfo.getPayFee());
            orderInfo.setPayStatus(payStatus);
            orderInfoService.update(orderInfo);



            return true;
        } else {
            // 退款失败
            throw new Exception("退款失败," + map.get("return_msg") + "," + map.get("err_code_des"));
        }

    }
}
