package com.jpsoft.gps;

import com.jpsoft.gps.callback.GpsDataCallback;
import com.jpsoft.gps.codec.CustomDecoder;
import com.jpsoft.gps.handler.ActivateHandler;
import com.jpsoft.gps.handler.ProcessHandler;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.Channel;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.handler.timeout.ReadTimeoutHandler;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class GpsServer {
    private Channel serverChannel;
    private final int READ_TIME_OUT = 300;

    public void start(int port, GpsDataCallback callback){
        /**
         * NioEventLoop是非守护线程。
         * NioEventLoop运行之后，不会主动退出。
         * 只有调用shutdown系列方法，NioEventLoop才会退出。
         * Netty是一个异步非阻塞框架
         **/
        NioEventLoopGroup bossGroup = new NioEventLoopGroup();
        NioEventLoopGroup workerGroup = new NioEventLoopGroup();

        ServerBootstrap b = new ServerBootstrap();

        b.group(bossGroup,workerGroup)
                .channel(NioServerSocketChannel.class)
                .childHandler(new ChannelInitializer(){
                    @Override
                    protected void initChannel(Channel channel) throws Exception {
                        channel.pipeline().addLast(new ReadTimeoutHandler(READ_TIME_OUT));

                        channel.pipeline().addLast(new ActivateHandler());
                        channel.pipeline().addLast(new CustomDecoder());
                        channel.pipeline().addLast(new ProcessHandler(callback));
                    }
                });

        ChannelFuture channelFuture = b.bind("0.0.0.0",port);
        serverChannel = channelFuture.channel();

        channelFuture.addListener((future)-> {
            if(future.isSuccess()){
                log.warn(String.format("gps server 已监听%d端口",port));
            }
            else{
                log.error(String.format("gps server 已监听%d端口",port));
            }
        });
    }

    public void stop() {
        if (serverChannel!=null && serverChannel.isOpen()){
            serverChannel.close();
        }
    }
}
