package com.jpsoft.bus.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.bus.modules.common.dto.MessageResult;
import com.jpsoft.bus.modules.common.dto.Sort;
import com.jpsoft.bus.modules.base.entity.MergeOrderInfo;
import com.jpsoft.bus.modules.base.service.MergeOrderInfoService;
import com.jpsoft.bus.modules.common.utils.PojoUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 支付订单
 */
@RestController
@RequestMapping("/base/mergeOrderInfo")
@Api(description = "mergeOrderInfo")
public class MergeOrderInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private MergeOrderInfoService mergeOrderInfoService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<MergeOrderInfo> create(){
        MessageResult<MergeOrderInfo> msgResult = new MessageResult<>();

        MergeOrderInfo mergeOrderInfo = new MergeOrderInfo();

        msgResult.setData(mergeOrderInfo);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<MergeOrderInfo> add(@RequestBody MergeOrderInfo mergeOrderInfo,@RequestAttribute String subject){
        MessageResult<MergeOrderInfo> msgResult = new MessageResult<>();

        try {
            mergeOrderInfo.setId(UUID.randomUUID().toString());
            mergeOrderInfo.setDelFlag(false);
            mergeOrderInfo.setCreateBy(subject);
            mergeOrderInfo.setCreateTime(new Date());
            
            int affectCount = mergeOrderInfoService.insert(mergeOrderInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(mergeOrderInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<MergeOrderInfo> edit(@PathVariable("id") String id){
        MessageResult<MergeOrderInfo> msgResult = new MessageResult<>();

        try {
            MergeOrderInfo mergeOrderInfo = mergeOrderInfoService.get(id);

            if (mergeOrderInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(mergeOrderInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<MergeOrderInfo> update(@RequestBody MergeOrderInfo mergeOrderInfo,@RequestAttribute String subject){
        MessageResult<MergeOrderInfo> msgResult = new MessageResult<>();

        try {
            mergeOrderInfo.setUpdateBy(subject);
            mergeOrderInfo.setUpdateTime(new Date());
            
            int affectCount = mergeOrderInfoService.update(mergeOrderInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(mergeOrderInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            MergeOrderInfo mergeOrderInfo = mergeOrderInfoService.get(id);
            mergeOrderInfo.setDelFlag(true);
            mergeOrderInfo.setUpdateBy(subject);
            mergeOrderInfo.setUpdateTime(new Date());

            int affectCount = mergeOrderInfoService.update(mergeOrderInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                MergeOrderInfo mergeOrderInfo = mergeOrderInfoService.get(id);
                mergeOrderInfo.setDelFlag(true);
                mergeOrderInfo.setUpdateBy(subject);
                mergeOrderInfo.setUpdateTime(new Date());

                affectCount += mergeOrderInfoService.update(mergeOrderInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        Page<MergeOrderInfo> page = mergeOrderInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
