package com.jpsoft.bus.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.bus.modules.bus.entity.CompanyInfo;
import com.jpsoft.bus.modules.bus.service.CompanyInfoService;
import com.jpsoft.bus.modules.common.dto.MessageResult;
import com.jpsoft.bus.modules.common.dto.Sort;
import com.jpsoft.bus.modules.base.entity.PaymentInfo;
import com.jpsoft.bus.modules.base.service.PaymentInfoService;
import com.jpsoft.bus.modules.common.utils.PojoUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/paymentInfo")
@Api(description = "paymentInfo")
public class PaymentInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private PaymentInfoService paymentInfoService;

    @Autowired
    private CompanyInfoService companyInfoService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<PaymentInfo> create(){
        MessageResult<PaymentInfo> msgResult = new MessageResult<>();

        PaymentInfo paymentInfo = new PaymentInfo();

        msgResult.setData(paymentInfo);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<PaymentInfo> add(@RequestBody PaymentInfo paymentInfo,@RequestAttribute String subject){
        MessageResult<PaymentInfo> msgResult = new MessageResult<>();

        try {
            paymentInfo.setId(UUID.randomUUID().toString());
            paymentInfo.setDelFlag(false);
            paymentInfo.setCreateBy(subject);
            paymentInfo.setCreateTime(new Date());
            
            int affectCount = paymentInfoService.insert(paymentInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(paymentInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<PaymentInfo> edit(@PathVariable("id") String id){
        MessageResult<PaymentInfo> msgResult = new MessageResult<>();

        try {
            PaymentInfo paymentInfo = paymentInfoService.get(id);

            if (paymentInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(paymentInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<PaymentInfo> update(@RequestBody PaymentInfo paymentInfo,@RequestAttribute String subject){
        MessageResult<PaymentInfo> msgResult = new MessageResult<>();

        try {
            paymentInfo.setUpdateBy(subject);
            paymentInfo.setUpdateTime(new Date());
            
            int affectCount = paymentInfoService.update(paymentInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(paymentInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            PaymentInfo paymentInfo = paymentInfoService.get(id);
            paymentInfo.setDelFlag(true);
            paymentInfo.setUpdateBy(subject);
            paymentInfo.setUpdateTime(new Date());

            int affectCount = paymentInfoService.update(paymentInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                PaymentInfo paymentInfo = paymentInfoService.get(id);
                paymentInfo.setDelFlag(true);
                paymentInfo.setUpdateBy(subject);
                paymentInfo.setUpdateTime(new Date());

                affectCount += paymentInfoService.update(paymentInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String companyId,String name,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        if (StringUtils.isNotEmpty(companyId)) {
            searchParams.put("companyId",companyId);
        }

        if (StringUtils.isNotEmpty(name)) {
            searchParams.put("name","%"+name+"%");
        }

        Page<PaymentInfo> page = paymentInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        for (PaymentInfo paymentInfo:page) {
            CompanyInfo companyInfo = companyInfoService.get(paymentInfo.getCompanyId());
            if(companyInfo!=null){
                paymentInfo.setCompanyName(companyInfo.getName());
            }
        }

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value="修改是否支持微信支付宝支付")
    @PostMapping("updateWechatPayOrAliPay")
    public MessageResult<Integer> updateWechatPayOrAliPay(String id,Boolean wechatPay,Boolean aliPay, @RequestAttribute String subject) {
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            PaymentInfo paymentInfo = paymentInfoService.get(id);

            if(paymentInfo!=null){
                if(wechatPay!=null) {
                    paymentInfo.setIsOpenWechat(wechatPay);
                }
                if(aliPay!=null) {
                    paymentInfo.setIsOpenAlipay(aliPay);
                }

                paymentInfo.setUpdateBy(subject);
                paymentInfo.setUpdateTime(new Date());

                int count = paymentInfoService.update(paymentInfo);

                msgResult.setResult(true);
                msgResult.setData(count);
            }
            else{
                msgResult.setResult(false);
                msgResult.setMessage("更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
