package com.jpsoft.bus.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.bus.modules.base.entity.RegionInfo;
import com.jpsoft.bus.modules.base.service.RegionInfoService;
import com.jpsoft.bus.modules.common.dto.MessageResult;
import com.jpsoft.bus.modules.common.dto.Sort;
import com.jpsoft.bus.modules.common.utils.PojoUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 地区区域信息表
 * sz
 * **/
@RestController
@RequestMapping("/base/regionInfo")
@Api(description = "regionInfo")
public class RegionInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private RegionInfoService regionInfoService;


    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<RegionInfo> create(){
        MessageResult<RegionInfo> msgResult = new MessageResult<>();

        RegionInfo regionInfo = new RegionInfo();

        msgResult.setData(regionInfo);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<RegionInfo> add(@RequestBody RegionInfo regionInfo, @RequestAttribute String subject){
        MessageResult<RegionInfo> msgResult = new MessageResult<>();

        try {
            regionInfo.setId(UUID.randomUUID().toString());
            regionInfo.setDelFlag(false);
            regionInfo.setCreateBy(subject);
            regionInfo.setCreateTime(new Date());
            
            int affectCount = regionInfoService.insert(regionInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(regionInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<RegionInfo> edit(@PathVariable("id") String id){
        MessageResult<RegionInfo> msgResult = new MessageResult<>();

        try {
            RegionInfo regionInfo = regionInfoService.get(id);

            if (regionInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(regionInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<RegionInfo> update(@RequestBody RegionInfo regionInfo, @RequestAttribute String subject){
        MessageResult<RegionInfo> msgResult = new MessageResult<>();

        try {
            regionInfo.setUpdateBy(subject);
            regionInfo.setUpdateTime(new Date());
            
            int affectCount = regionInfoService.update(regionInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(regionInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id, @RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            RegionInfo regionInfo = regionInfoService.get(id);
            regionInfo.setDelFlag(true);
            regionInfo.setUpdateBy(subject);
            regionInfo.setUpdateTime(new Date());

            int affectCount = regionInfoService.update(regionInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList, @RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                RegionInfo regionInfo = regionInfoService.get(id);
                regionInfo.setDelFlag(true);
                regionInfo.setUpdateBy(subject);
                regionInfo.setUpdateTime(new Date());

                affectCount += regionInfoService.update(regionInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="name",defaultValue="") String name,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("b.sort_no","asc"));
        sortList.add(new Sort("a.sort_no","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        if (StringUtils.isNotEmpty(name)) {
            searchParams.put("name","%" + name + "%");
        }

        Page<RegionInfo> page = regionInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }


    @ApiOperation(value="所有地区区域列表")
    @RequestMapping(value = "list",method = RequestMethod.POST)
    public MessageResult<List<RegionInfo>> list(@RequestAttribute String subject){

        MessageResult<List<RegionInfo>> msgResult = new MessageResult<>();

        List<RegionInfo> list = regionInfoService.list();

        msgResult.setResult(true);
        msgResult.setData(list);

        return msgResult;
    }
}
