package com.jpsoft.bus.modules.bus.controller;

import com.github.pagehelper.Page;
import com.jpsoft.bus.modules.base.entity.OrderInfo;
import com.jpsoft.bus.modules.base.service.OrderInfoService;
import com.jpsoft.bus.modules.common.dto.MessageResult;
import com.jpsoft.bus.modules.common.dto.Sort;
import com.jpsoft.bus.modules.bus.entity.PassengerInfo;
import com.jpsoft.bus.modules.bus.service.PassengerInfoService;
import com.jpsoft.bus.modules.common.utils.PojoUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 乘客
 */

@RestController
@RequestMapping("/bus/passengerInfo")
@Api(description = "passengerInfo")
public class PassengerInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private PassengerInfoService passengerInfoService;

    @Autowired
    private OrderInfoService orderInfoService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<PassengerInfo> create(){
        MessageResult<PassengerInfo> msgResult = new MessageResult<>();

        PassengerInfo passengerInfo = new PassengerInfo();

        msgResult.setData(passengerInfo);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<PassengerInfo> add(@RequestBody PassengerInfo passengerInfo,@RequestAttribute String subject){
        MessageResult<PassengerInfo> msgResult = new MessageResult<>();

        try {
            passengerInfo.setId(UUID.randomUUID().toString());
            passengerInfo.setDelFlag(false);
            passengerInfo.setCreateBy(subject);
            passengerInfo.setCreateTime(new Date());
            
            int affectCount = passengerInfoService.insert(passengerInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(passengerInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<PassengerInfo> edit(@PathVariable("id") String id){
        MessageResult<PassengerInfo> msgResult = new MessageResult<>();

        try {
            PassengerInfo passengerInfo = passengerInfoService.get(id);

            if (passengerInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(passengerInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<PassengerInfo> update(@RequestBody PassengerInfo passengerInfo,@RequestAttribute String subject){
        MessageResult<PassengerInfo> msgResult = new MessageResult<>();

        try {
            passengerInfo.setUpdateBy(subject);
            passengerInfo.setUpdateTime(new Date());
            
            int affectCount = passengerInfoService.update(passengerInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(passengerInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            PassengerInfo passengerInfo = passengerInfoService.get(id);
            passengerInfo.setDelFlag(true);
            passengerInfo.setUpdateBy(subject);
            passengerInfo.setUpdateTime(new Date());

            int affectCount = passengerInfoService.update(passengerInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                PassengerInfo passengerInfo = passengerInfoService.get(id);
                passengerInfo.setDelFlag(true);
                passengerInfo.setUpdateBy(subject);
                passengerInfo.setUpdateTime(new Date());

                affectCount += passengerInfoService.update(passengerInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            @RequestParam(value="vehicleShiftId",defaultValue="") String vehicleShiftId,
            @RequestParam(value="vehiclePersonId",defaultValue="") String vehiclePersonId,
            @RequestParam(value="userName",defaultValue="") String userName,
            @RequestParam(value="payStatus",defaultValue="") String payStatus,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("a.create_time","desc"));

        if (StringUtils.isNotEmpty(vehicleShiftId)) {
            searchParams.put("vehicleShiftId",vehicleShiftId);
        }

        if (StringUtils.isNotEmpty(vehiclePersonId)) {
            searchParams.put("vehiclePersonId",Long.valueOf(vehiclePersonId));
        }

        if (StringUtils.isNotEmpty(userName)) {
            searchParams.put("userName","%" + userName + "%");
        }

        if (StringUtils.isNotEmpty(payStatus)) {
            searchParams.put("payStatus",payStatus);
        }

        Page<PassengerInfo> page = passengerInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        for (PassengerInfo passengerInfo:page) {

            OrderInfo orderInfo = orderInfoService.findByPassengerId(passengerInfo.getId());

            if(orderInfo!=null){
                if(("wechat").equals(orderInfo.getPayName())){
                    passengerInfo.setPayName("微信");
                }
                else if(("alipay").equals(orderInfo.getPayName())){
                    passengerInfo.setPayName("支付宝");
                }
                else if(("cash").equals(orderInfo.getPayName())){
                    passengerInfo.setPayName("现金");
                }
                else if(("ticket").equals(orderInfo.getPayName())){
                    passengerInfo.setPayName("车票");
                }
            }


        }


        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
