package com.jpsoft.bus.modules.bus.controller;

import com.github.pagehelper.Page;
import com.jpsoft.bus.config.OSSConfig;
import com.jpsoft.bus.modules.bus.entity.CompanyInfo;
import com.jpsoft.bus.modules.bus.entity.RouteInfo;
import com.jpsoft.bus.modules.bus.entity.StationInfo;
import com.jpsoft.bus.modules.bus.service.RouteInfoService;
import com.jpsoft.bus.modules.bus.service.StationInfoService;
import com.jpsoft.bus.modules.common.dto.MessageResult;
import com.jpsoft.bus.modules.common.dto.Sort;
import com.jpsoft.bus.modules.bus.entity.PriceInfo;
import com.jpsoft.bus.modules.bus.service.PriceInfoService;
import com.jpsoft.bus.modules.common.utils.OSSUtil;
import com.jpsoft.bus.modules.common.utils.POIUtils;
import com.jpsoft.bus.modules.common.utils.PojoUtils;
import com.jpsoft.bus.modules.sys.entity.User;
import com.jpsoft.bus.modules.sys.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 站点之间的价格
 */
@RestController
@RequestMapping("/bus/priceInfo")
@Api(description = "priceInfo")
public class PriceInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private PriceInfoService priceInfoService;
    @Autowired
    private StationInfoService stationInfoService;
    @Autowired
    private RouteInfoService routeInfoService;
    @Autowired
    private UserService userService;
    @Autowired
    private OSSConfig ossConfig;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<PriceInfo> create(){
        MessageResult<PriceInfo> msgResult = new MessageResult<>();

        PriceInfo priceInfo = new PriceInfo();

        msgResult.setData(priceInfo);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<PriceInfo> add(@RequestBody PriceInfo priceInfo,@RequestAttribute String subject){
        MessageResult<PriceInfo> msgResult = new MessageResult<>();

        try {
            priceInfo.setId(UUID.randomUUID().toString());
            priceInfo.setDelFlag(false);
            priceInfo.setCreateBy(subject);
            priceInfo.setCreateTime(new Date());
            
            int affectCount = priceInfoService.insert(priceInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(priceInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<PriceInfo> edit(@PathVariable("id") String id){
        MessageResult<PriceInfo> msgResult = new MessageResult<>();

        try {
            PriceInfo priceInfo = priceInfoService.get(id);

            if (priceInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(priceInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<PriceInfo> update(@RequestBody PriceInfo priceInfo,@RequestAttribute String subject){
        MessageResult<PriceInfo> msgResult = new MessageResult<>();

        try {
            priceInfo.setUpdateBy(subject);
            priceInfo.setUpdateTime(new Date());
            
            int affectCount = priceInfoService.update(priceInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(priceInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            PriceInfo priceInfo = priceInfoService.get(id);
            priceInfo.setDelFlag(true);
            priceInfo.setUpdateBy(subject);
            priceInfo.setUpdateTime(new Date());

            int affectCount = priceInfoService.update(priceInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                PriceInfo priceInfo = priceInfoService.get(id);
                priceInfo.setDelFlag(true);
                priceInfo.setUpdateBy(subject);
                priceInfo.setUpdateTime(new Date());

                affectCount += priceInfoService.update(priceInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        Page<PriceInfo> page = priceInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value="站点价格图列表")
    @RequestMapping(value = "searchRoutePrice",method = RequestMethod.POST)
    public  Map<String,Object> searchRoutePrice(
            @RequestParam(value="routeId",defaultValue="") String routeId,
            @RequestAttribute String subject){

        Map<String,Object> dataMap = new HashMap<>();

        Map<String,Object> searchParams = new HashMap<>();

        try{
            //查询所有站点
            List<StationInfo> stationInfoList = stationInfoService.findByRouteId(routeId);
            List<Map> stationMap = new ArrayList<>();
            List<Map> stationPriceColumns = new ArrayList<>();


            for (StationInfo endSi : stationInfoList){
                Map<String,Object> map = new HashMap<>();
                List<Map> priceList = new ArrayList<>();
                Map<String,Map> stationPriceMap = new HashMap<>();
                for(StationInfo startSi : stationInfoList){
                    Map<String,Object> priceMap = new HashMap<>();
                    PriceInfo priceInfo = priceInfoService.findByStartStationAndEndStation(startSi.getId(),endSi.getId());
                    String priceStr = "0";
                    //BigDecimal price = new BigDecimal(0);
                    if(priceInfo != null){
                        priceStr = priceInfo.getPrice().toString();
                    }

                    priceMap.put("price",priceStr);
                    priceList.add(priceMap);
                    stationPriceMap.put(startSi.getId(),priceMap);
                }

                map.put("id",endSi.getId());
                map.put("name",endSi.getName());
                map.put("priceList",priceList);
                map.put("stationPriceMap",stationPriceMap);


                stationMap.add(map);
                stationPriceColumns.add(map);
            }

            dataMap.put("stationPriceColumns", stationPriceColumns);
            dataMap.put("stationInfoList", stationMap);

        }catch (Exception ex){
            ex.printStackTrace();
            logger.error(ex.getMessage(),ex);
        }

        return dataMap;
    }

    @ApiOperation(value="导入线路")
    @PostMapping("importXls")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "uploadFile",value = "上传文件", required = true,paramType="form", dataType = "__file")
    })
    public MessageResult<String> importXls(
            MultipartFile uploadFile,
            @RequestParam(value="routeId",defaultValue="") String routeId,
            @RequestAttribute String subject){
        MessageResult<String> msgResult = new MessageResult<>();

        try {

            if(StringUtils.isEmpty(routeId)){
                throw new Exception("线路查询失败");
            }

            RouteInfo routeInfo = routeInfoService.get(routeId);
            if(routeInfo == null){
                throw new Exception("线路不存在");
            }

            POIUtils poiUtils = new POIUtils(uploadFile.getInputStream());
            int sheetIndex = 0;
            Sheet sheet1 = poiUtils.getSheetAt(sheetIndex);

            int affectCount = 0;
            int failCount = 0;
            int validateColIndex = 10;




            Map<Integer,String> heardMap = new HashMap<Integer,String>();
            //查第一列 表头的站点名称名称 --  起点站
            if(sheet1.getLastRowNum() > 0){
                int coloumNum = sheet1.getRow(0).getPhysicalNumberOfCells();
                for(int i = 1;i<coloumNum;i++){
                    String startName = (String)poiUtils.getCellValue(sheetIndex,0,i).toString().replace(" ","");
                    heardMap.put(i,startName);
                }
            }



            for(int rowIndex=0 ; rowIndex<=sheet1.getLastRowNum(); rowIndex++){
                try {
                    //第一列为终点站点
                    String endName = (String)poiUtils.getCellValue(sheetIndex,rowIndex,0).toString().replace(" ","");
                    StationInfo endInfo = stationInfoService.findByRouteIdAndName(routeId,endName);
                    if(endInfo == null){
                        continue;
                    }

                    for(int startIndex = 1;startIndex <=heardMap.size();startIndex++){
                        String startName = heardMap.get(startIndex);
                        StationInfo startInfo = stationInfoService.findByRouteIdAndName(routeId,startName);
                        if(startInfo == null){
                            continue;
                        }

                        PriceInfo pi = priceInfoService.findByStartStationAndEndStation(startInfo.getId(),endInfo.getId());

                        String price = (String)poiUtils.getCellValue(sheetIndex,rowIndex,startIndex).toString().replace(" ","");
                        if("0".equals(price)){
                            //金额为0不保存
                            continue;
                        }

                        if(pi == null){
                            pi = new PriceInfo();
                            pi.setId(UUID.randomUUID().toString());
                            pi.setPrice(new BigDecimal(price));
                            pi.setStartStationId(startInfo.getId());
                            pi.setEndStationId(endInfo.getId());

                            pi.setDelFlag(false);
                            pi.setCreateBy(subject);
                            pi.setCreateTime(new Date());
                            priceInfoService.insert(pi);
                        }else{
                            pi.setPrice(new BigDecimal(price));

                            pi.setDelFlag(false);
                            pi.setUpdateBy(subject);
                            pi.setUpdateTime(new Date());
                            priceInfoService.update(pi);
                        }
                        affectCount++;
                    }
                }
                catch(Exception innerEx){
                    logger.error(innerEx.getMessage(),innerEx);
                }
            }

            if (failCount>0){
                //有导入失败的记录
                msgResult.setResult(false);
                msgResult.setMessage("数据成功导入" + affectCount + "条,有" + failCount + "条数据未导入成功，错误原因请查看报表。");

                //todo 只保留错误数据的sheet
                Workbook wb = poiUtils.exportErrorXls(sheetIndex,validateColIndex,1 + affectCount + failCount);

                //todo 将wb保存到oss
                ByteArrayOutputStream output = new ByteArrayOutputStream();
                wb.write(output);

                byte[] buffer = output.toByteArray();
                ByteArrayInputStream input = new ByteArrayInputStream(buffer);

                SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
                String fileName = "error" + sdf.format(new Date()) + ".xls";

                String downloadUrl = OSSUtil.upload(ossConfig,"import",fileName,input);

                //todo 返回导入失败报表下载链接
                msgResult.setData(downloadUrl);
            }
            else{
                msgResult.setResult(true);
                msgResult.setMessage("数据成功导入" + affectCount + "条");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="导出人员")
    @PostMapping("exportXls")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "routeId", value = "线路Id", required = false, paramType = "form", dataType = "String"),
    })
    public MessageResult<String> exportXls(
            @RequestParam(value="routeId",defaultValue="") String routeId,
            @RequestAttribute String subject){
        MessageResult<String> msgResult = new MessageResult<>();
        User user = userService.get(subject);

        try {

            if(StringUtils.isEmpty(routeId)){
                throw new Exception("线路查询失败");
            }

            RouteInfo routeInfo = routeInfoService.get(routeId);
            if(routeInfo == null){
                throw new Exception("线路不存在");
            }

            //站点
            List<StationInfo> stationInfoList = stationInfoService.findByRouteId(routeId);



            XSSFWorkbook workbook = new XSSFWorkbook();
            XSSFSheet sheet = workbook.createSheet();
            //表头
            XSSFRow rowTitle = sheet.createRow(0);
            XSSFCell cellTitle1 = rowTitle.createCell(0);
            cellTitle1.setCellValue("站点");

            //标题列
            for(int i=0;i<stationInfoList.size();i++){
                StationInfo startStation = stationInfoList.get(i);
                XSSFCell cellTitle = rowTitle.createCell(i+1);
                cellTitle.setCellValue(startStation.getName());

                XSSFRow rowContent = sheet.createRow(i + 1);

                XSSFCell cellContent1 = rowContent.createCell(0);
                cellContent1.setCellValue(startStation.getName());

                //第二列开始
                for(int j=0; j<stationInfoList.size(); j++){
                    StationInfo endStation = stationInfoList.get(j);
                    PriceInfo priceInfo = priceInfoService.findByStartStationAndEndStation(startStation.getId(),endStation.getId());

                    XSSFCell cellContent2 = rowContent.createCell(j+1);
                    String priceStr = "0";
                    if(priceInfo != null) {
                        priceStr = priceInfo.getPrice().toString();
                    }
                    cellContent2.setCellValue(priceStr);
                }
            }

            //todo 将wb保存到oss
            ByteArrayOutputStream output = new ByteArrayOutputStream();
            workbook.write(output);

            byte[] buffer = output.toByteArray();
            ByteArrayInputStream input = new ByteArrayInputStream(buffer);

            //格式化
            SimpleDateFormat sim = new SimpleDateFormat("yyyyMMddHHmmss");
            String fileName = "导出数据" + sim.format(new Date()) + ".xlsx";
            String downloadUrl = OSSUtil.upload(ossConfig,"import",fileName,input);

            //todo 返回导入失败报表下载链接

            msgResult.setData(downloadUrl);
            msgResult.setResult(true);
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="根据站点查金额")
    @RequestMapping(value = "findPriceByStation",method = RequestMethod.POST)
    public MessageResult<PriceInfo> list(
            String startStationId,
            String endStationId,
            @RequestAttribute String subject){

        MessageResult<PriceInfo> msgResult = new MessageResult<>();

        PriceInfo priceInfo = priceInfoService.findByStartStationAndEndStation(startStationId,endStationId);
        if(priceInfo == null){
            priceInfo = new PriceInfo();
        }

        msgResult.setResult(true);
        msgResult.setData(priceInfo);

        return msgResult;
    }

}
