package com.jpsoft.bus.modules.bus.controller;

import com.github.pagehelper.Page;
import com.jpsoft.bus.modules.common.dto.MessageResult;
import com.jpsoft.bus.modules.common.dto.Sort;
import com.jpsoft.bus.modules.bus.entity.RouteTimeTable;
import com.jpsoft.bus.modules.bus.service.RouteTimeTableService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.mapping.ResultMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;


@RestController
@RequestMapping("/bus/routeTimeTable")
@Api(description = "routeTimeTable")
public class RouteTimeTableController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private RouteTimeTableService routeTimeTableService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<RouteTimeTable> create(){
        RouteTimeTable dto = new RouteTimeTable();

        MessageResult<RouteTimeTable> messageResult = new MessageResult<>();

        messageResult.setResult(true);
        messageResult.setData(dto);

        return messageResult;
    }

    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<RouteTimeTable> add(@RequestBody RouteTimeTable routeTimeTable,@RequestAttribute String subject){
        MessageResult<RouteTimeTable> msgResult = new MessageResult<>();

        try {
            routeTimeTable.setId(UUID.randomUUID().toString());
            routeTimeTable.setCreateBy(subject);
            routeTimeTable.setCreateTime(new Date());
            routeTimeTable.setDelFlag(false);


            RouteTimeTable item = routeTimeTableService.findByRouteId(routeTimeTable.getRouteId());

            if(item!=null){
                routeTimeTable.setSort(item.getSort()+1);
            }
            else{
                routeTimeTable.setSort(1);
            }

            int affectCount = routeTimeTableService.insert(routeTimeTable);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(routeTimeTable);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<RouteTimeTable> detail(@PathVariable("id") String id){
        MessageResult<RouteTimeTable> msgResult = new MessageResult<>();

        try {
            RouteTimeTable routeTimeTable = routeTimeTableService.get(id);

            if (routeTimeTable != null) {
                msgResult.setResult(true);
                msgResult.setData(routeTimeTable);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<RouteTimeTable> update(@RequestBody RouteTimeTable routeTimeTable,@RequestAttribute String subject){
        MessageResult<RouteTimeTable> msgResult = new MessageResult<>();

        try {

            routeTimeTable.setUpdateBy(subject);
            routeTimeTable.setUpdateTime(new Date());

            int affectCount = routeTimeTableService.update(routeTimeTable);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(routeTimeTable);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="删除用户")
    @PostMapping("delete/{id}")
    public MessageResult<RouteTimeTable> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<RouteTimeTable> msgResult = new MessageResult<>();

        try {
            RouteTimeTable routeTimeTable = routeTimeTableService.get(id);
            routeTimeTable.setDelFlag(true);
            routeTimeTable.setUpdateBy(subject);
            routeTimeTable.setUpdateTime(new Date());

            int affectCount = routeTimeTableService.update(routeTimeTable);

            if (affectCount > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                RouteTimeTable routeTimeTable = routeTimeTableService.get(id);
                routeTimeTable.setDelFlag(true);
                routeTimeTable.setUpdateBy(subject);
                routeTimeTable.setUpdateTime(new Date());

                affectCount += routeTimeTableService.update(routeTimeTable);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Object> pageList(
            String routeId,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            HttpServletRequest request){
        String subject = (String)request.getAttribute("subject");

        //当前用户ID
        System.out.println(subject);

        MessageResult<Object> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("sort_","asc"));

        searchParams.put("routeId",routeId);

        Page<RouteTimeTable> page = routeTimeTableService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(page.getResult());

        return msgResult;
    }
}
