package com.jpsoft.bus.modules.bus.controller;

import com.github.pagehelper.Page;
import com.jpsoft.bus.modules.common.dto.MessageResult;
import com.jpsoft.bus.modules.common.dto.Sort;
import com.jpsoft.bus.modules.bus.entity.ShiftInfo;
import com.jpsoft.bus.modules.bus.service.ShiftInfoService;
import com.jpsoft.bus.modules.common.utils.PojoUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 班次
 */

@RestController
@RequestMapping("/bus/shiftInfo")
@Api(description = "shiftInfo")
public class ShiftInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private ShiftInfoService shiftInfoService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<ShiftInfo> create(){
        MessageResult<ShiftInfo> msgResult = new MessageResult<>();

        ShiftInfo shiftInfo = new ShiftInfo();

        msgResult.setData(shiftInfo);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<ShiftInfo> add(@RequestBody ShiftInfo shiftInfo,@RequestAttribute String subject){
        MessageResult<ShiftInfo> msgResult = new MessageResult<>();

        try {
            shiftInfo.setId(UUID.randomUUID().toString());
            shiftInfo.setDelFlag(false);
            shiftInfo.setCreateBy(subject);
            shiftInfo.setCreateTime(new Date());
            
            int affectCount = shiftInfoService.insert(shiftInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(shiftInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<ShiftInfo> edit(@PathVariable("id") String id){
        MessageResult<ShiftInfo> msgResult = new MessageResult<>();

        try {
            ShiftInfo shiftInfo = shiftInfoService.get(id);

            if (shiftInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(shiftInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<ShiftInfo> update(@RequestBody ShiftInfo shiftInfo,@RequestAttribute String subject){
        MessageResult<ShiftInfo> msgResult = new MessageResult<>();

        try {
            shiftInfo.setUpdateBy(subject);
            shiftInfo.setUpdateTime(new Date());
            
            int affectCount = shiftInfoService.update(shiftInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(shiftInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            ShiftInfo shiftInfo = shiftInfoService.get(id);
            shiftInfo.setDelFlag(true);
            shiftInfo.setUpdateBy(subject);
            shiftInfo.setUpdateTime(new Date());

            int affectCount = shiftInfoService.update(shiftInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                ShiftInfo shiftInfo = shiftInfoService.get(id);
                shiftInfo.setDelFlag(true);
                shiftInfo.setUpdateBy(subject);
                shiftInfo.setUpdateTime(new Date());

                affectCount += shiftInfoService.update(shiftInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            @RequestParam(value="id",defaultValue="") String id,
            @RequestParam(value="vehicleNumber",defaultValue="") String vehicleNumber,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("a.create_time","desc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        if (StringUtils.isNotEmpty(vehicleNumber)) {
            searchParams.put("vehicleNumber","%" + vehicleNumber + "%");
        }

        Page<ShiftInfo> page = shiftInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
