package com.jpsoft.bus.modules.bus.controller;

import com.github.pagehelper.Page;
import com.jpsoft.bus.modules.common.dto.MessageResult;
import com.jpsoft.bus.modules.common.dto.Sort;
import com.jpsoft.bus.modules.bus.entity.UserInfo;
import com.jpsoft.bus.modules.bus.service.UserInfoService;
import com.jpsoft.bus.modules.common.utils.DES3;
import com.jpsoft.bus.modules.common.utils.PojoUtils;
import com.jpsoft.bus.modules.sys.entity.User;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 乘客
 */
@RestController
@RequestMapping("/bus/userInfo")
@Api(description = "userInfo")
public class UserInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private UserInfoService userInfoService;
    @Value("${jwt.secret}")
    private String jwtSecret;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<UserInfo> create(){
        MessageResult<UserInfo> msgResult = new MessageResult<>();

        UserInfo userInfo = new UserInfo();

        msgResult.setData(userInfo);
        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<UserInfo> add(@RequestBody UserInfo userInfo,@RequestAttribute String subject){
        MessageResult<UserInfo> msgResult = new MessageResult<>();

        try {
            userInfo.setDelFlag(false);
            userInfo.setCreateBy(subject);
            userInfo.setCreateTime(new Date());
            DES3 des3 = new DES3();
            userInfo.setPassword(des3.encrypt(jwtSecret,userInfo.getPassword()));

            int affectCount = userInfoService.insert(userInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(userInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);
            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<UserInfo> edit(@PathVariable("id") String id){
        MessageResult<UserInfo> msgResult = new MessageResult<>();

        try {
            UserInfo userInfo = userInfoService.get(Long.parseLong(id));

            if (userInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(userInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<UserInfo> update(@RequestBody UserInfo userInfo,@RequestAttribute String subject){
        MessageResult<UserInfo> msgResult = new MessageResult<>();

        try {
            userInfo.setUpdateBy(subject);
            userInfo.setUpdateTime(new Date());

            int affectCount = userInfoService.update(userInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(userInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            UserInfo userInfo = userInfoService.get(Long.parseLong(id));
            userInfo.setDelFlag(true);
            userInfo.setUpdateBy(subject);
            userInfo.setUpdateTime(new Date());

            int affectCount = userInfoService.update(userInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                UserInfo userInfo = userInfoService.get(Long.parseLong(id));
                userInfo.setDelFlag(true);
                userInfo.setUpdateBy(subject);
                userInfo.setUpdateTime(new Date());

                affectCount += userInfoService.update(userInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            @RequestParam(value="id",defaultValue="") String id,
            @RequestParam(value="name",defaultValue="") String name,
            @RequestParam(value="phone",defaultValue="") String phone,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id",id);
        }

        if (StringUtils.isNotEmpty(name)) {
            searchParams.put("name","%" + name + "%");
        }

        if (StringUtils.isNotEmpty(phone)) {
            searchParams.put("phone","%" + phone + "%");
        }

        Page<UserInfo> page = userInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value="修改用户密码")
    @PostMapping("changePassword")
    public MessageResult<Integer> changePassword(String userId,String newPwd,@RequestAttribute String subject) {

        MessageResult<Integer> msgResult = new MessageResult<>();

        try {

            UserInfo userInfo = userInfoService.get(Long.parseLong(userId));

            DES3 des3 = new DES3();

            userInfo.setPassword(des3.encrypt(jwtSecret, newPwd));
            userInfo.setUpdateBy(subject);
            userInfo.setUpdateTime(new Date());

            int affectCount = userInfoService.update(userInfo);

            if(affectCount>0){
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            }
            else{
                msgResult.setResult(false);
                msgResult.setMessage("修改失败！");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);
            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
