package com.jpsoft.bus.modules.bus.controller;

import com.github.pagehelper.Page;
import com.jpsoft.bus.config.OSSConfig;
import com.jpsoft.bus.modules.bus.entity.CompanyInfo;
import com.jpsoft.bus.modules.bus.entity.GpsDataInfo;
import com.jpsoft.bus.modules.bus.service.CompanyInfoService;
import com.jpsoft.bus.modules.bus.service.GpsDataInfoService;
import com.jpsoft.bus.modules.common.dto.MessageResult;
import com.jpsoft.bus.modules.common.dto.Sort;
import com.jpsoft.bus.modules.bus.entity.VehicleInfo;
import com.jpsoft.bus.modules.bus.service.VehicleInfoService;
import com.jpsoft.bus.modules.common.utils.OSSUtil;
import com.jpsoft.bus.modules.common.utils.POIUtils;
import com.jpsoft.bus.modules.common.utils.PojoUtils;
import com.jpsoft.bus.modules.sys.entity.SysLog;
import com.jpsoft.bus.modules.sys.service.SysLogService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 车辆表
 */
@RestController
@RequestMapping("/bus/vehicleInfo")
@Api(description = "vehicleInfo")
@Slf4j
public class VehicleInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private OSSConfig ossConfig;

    @Autowired
    private VehicleInfoService vehicleInfoService;

    @Autowired
    private CompanyInfoService companyInfoService;

    @Autowired
    private GpsDataInfoService gpsDataInfoService;

    @Autowired
    private SysLogService sysLogService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<VehicleInfo> create(){
        MessageResult<VehicleInfo> msgResult = new MessageResult<>();

        VehicleInfo vehicleInfo = new VehicleInfo();

        msgResult.setData(vehicleInfo);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<VehicleInfo> add(@RequestBody VehicleInfo vehicleInfo,@RequestAttribute String subject){
        MessageResult<VehicleInfo> msgResult = new MessageResult<>();

        try {
            vehicleInfo.setId(UUID.randomUUID().toString());
            vehicleInfo.setStatus("1");
            vehicleInfo.setDelFlag(false);
            vehicleInfo.setCreateBy(subject);
            vehicleInfo.setCreateTime(new Date());
            
            int affectCount = vehicleInfoService.insert(vehicleInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(vehicleInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<VehicleInfo> edit(@PathVariable("id") String id){
        MessageResult<VehicleInfo> msgResult = new MessageResult<>();

        try {
            VehicleInfo vehicleInfo = vehicleInfoService.get(id);

            if (vehicleInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(vehicleInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<VehicleInfo> update(@RequestBody VehicleInfo vehicleInfo,@RequestAttribute String subject){
        MessageResult<VehicleInfo> msgResult = new MessageResult<>();

        try {
            vehicleInfo.setUpdateBy(subject);
            vehicleInfo.setUpdateTime(new Date());
            
            int affectCount = vehicleInfoService.update(vehicleInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(vehicleInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            VehicleInfo vehicleInfo = vehicleInfoService.get(id);
            vehicleInfo.setDelFlag(true);
            vehicleInfo.setUpdateBy(subject);
            vehicleInfo.setUpdateTime(new Date());

            int affectCount = vehicleInfoService.update(vehicleInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                VehicleInfo vehicleInfo = vehicleInfoService.get(id);
                vehicleInfo.setDelFlag(true);
                vehicleInfo.setUpdateBy(subject);
                vehicleInfo.setUpdateTime(new Date());

                affectCount += vehicleInfoService.update(vehicleInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String licensePlateNumber,String companyId,String gpsDeviceNo,String deviceNo,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("b.name_","asc"));
        sortList.add(new Sort("a.create_time","desc"));

        if (StringUtils.isNotEmpty(licensePlateNumber)) {
            searchParams.put("licensePlateNumber","%" + licensePlateNumber + "%");
        }

        if (StringUtils.isNotEmpty(companyId)) {
            searchParams.put("companyId",companyId);
        }

        if (StringUtils.isNotEmpty(gpsDeviceNo)) {
            searchParams.put("gpsDeviceNo","%" + gpsDeviceNo + "%");
        }

        if (StringUtils.isNotEmpty(deviceNo)) {
            searchParams.put("deviceNo","%" + deviceNo + "%");
        }

        Page<VehicleInfo> page = vehicleInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        for (VehicleInfo vehicleInfo:page) {
            CompanyInfo companyInfo = companyInfoService.get(vehicleInfo.getCompanyId());
            if(companyInfo!=null){
                vehicleInfo.setCompanyName(companyInfo.getName());
            }
        }

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }


    @ApiOperation(value="导入车辆")
    @PostMapping("importXls")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "uploadFile",value = "上传文件", required = true,paramType="form", dataType = "__file")
    })
    public MessageResult<String> importXls(MultipartFile uploadFile,
                                           @RequestAttribute String subject){
        MessageResult<String> msgResult = new MessageResult<>();

        try {
            POIUtils poiUtils = new POIUtils(uploadFile.getInputStream());
            int sheetIndex = 0;
            Sheet sheet1 = poiUtils.getSheetAt(sheetIndex);

            int affectCount = 0;
            int failCount = 0;
            int validateColIndex = 10;

            for(int rowIndex=1 ; rowIndex<=sheet1.getLastRowNum(); rowIndex++){
                try {
                    String companyName = (String)poiUtils.getCellValue(sheetIndex,rowIndex,0).toString().replace(" ","");
                    String licensePlateNumber = (String)poiUtils.getCellValue(sheetIndex,rowIndex,1).toString().replace(" ","");
                    String gpsDeviceNo = (String)poiUtils.getCellValue(sheetIndex,rowIndex,2).toString().replace(" ","");
                    String deviceNo = (String)poiUtils.getCellValue(sheetIndex,rowIndex,3).toString().replace(" ","");
                    String loadNumber = (String)poiUtils.getCellValue(sheetIndex,rowIndex,4).toString().replace(" ","");

                    if(StringUtils.isEmpty(companyName)){
                        break;
                    }

                    CompanyInfo companyInfo = companyInfoService.findByName(companyName);

                    if (companyInfo == null) {
                        sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("所属单位不存在!");
                        failCount++;
                        continue;
                    }

                    VehicleInfo vehicleInfo = new VehicleInfo();
                    vehicleInfo.setId(UUID.randomUUID().toString());
                    vehicleInfo.setCompanyId(companyInfo.getId());
                    vehicleInfo.setLicensePlateNumber(licensePlateNumber);
                    vehicleInfo.setGpsDeviceNo(gpsDeviceNo);
                    vehicleInfo.setDeviceNo(deviceNo);
                    vehicleInfo.setStatus("1");

                    if(StringUtils.isNotEmpty(loadNumber)){
                        vehicleInfo.setLoadNumber(Integer.parseInt(loadNumber));
                    }

                    vehicleInfo.setDelFlag(false);
                    vehicleInfo.setCreateBy(subject);
                    vehicleInfo.setCreateTime(new Date());
                    vehicleInfoService.insert(vehicleInfo);

                    affectCount++;

                }
                catch(Exception innerEx){
                    logger.error(innerEx.getMessage(),innerEx);
                }
            }

            if (failCount>0){
                //有导入失败的记录
                msgResult.setResult(false);
                msgResult.setMessage("数据成功导入" + affectCount + "条,有" + failCount + "条数据未导入成功，错误原因请查看报表。");

                //todo 只保留错误数据的sheet
                Workbook wb = poiUtils.exportErrorXls(sheetIndex,validateColIndex,1 + affectCount + failCount);

                //todo 将wb保存到oss
                ByteArrayOutputStream output = new ByteArrayOutputStream();
                wb.write(output);

                byte[] buffer = output.toByteArray();
                ByteArrayInputStream input = new ByteArrayInputStream(buffer);

                String downloadUrl = OSSUtil.upload(ossConfig,"import","error.xls",input);

                //todo 返回导入失败报表下载链接
                msgResult.setData(downloadUrl);
            }
            else{
                msgResult.setResult(true);
                msgResult.setMessage("数据成功导入" + affectCount + "条");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @PostMapping("gpsHistoryList")
    public MessageResult<Map> gpsHistoryList(
            String deviceNo,
            @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss") Date startTime,
            @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss") Date endTime,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        MessageResult<Map> messageResult = new MessageResult<Map>();

        Map<String,Object> searchParams = new HashMap<>();

        searchParams.put("deviceNo",deviceNo);

        if(startTime != null) {
            searchParams.put("startTime", startTime);
        }

        if(endTime != null) {
            searchParams.put("endTime", endTime);
        }

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        if (pageSize>100){
            pageSize = 100;
        }

        Page<GpsDataInfo> page = gpsDataInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        messageResult.setResult(true);
        messageResult.setData(PojoUtils.pageWrapper(page));

        return messageResult;
    }

    @ApiOperation(value="车辆信息一键绑定")
    @RequestMapping(value = "bindBus",method = RequestMethod.POST)
    public MessageResult<Map> bindBus(
            @RequestParam(value="logId",defaultValue="") String logId,
            @RequestAttribute String subject)throws Exception{

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();
        try {

            if (StringUtils.isEmpty(logId)) {
                throw new Exception("未查到日志信息");
            }

            SysLog sysLog = sysLogService.get(Long.valueOf(logId));

            if (sysLog == null) {
                throw new Exception("未查到日志信息");
            }


            String sysData = sysLog.getData();
            String[] data = sysData.split(",");
            if (data.length > 2) {
                //车牌号
                String carNum = data[0];
                //平板设备编号
                String activationCode = data[1];
                //设备编号
                String deviceNo = data[2];
                VehicleInfo vehicleInfo = vehicleInfoService.findByCarNum(carNum);
                if (vehicleInfo == null) {
                    throw new Exception("未查到车辆信息");
                }

                vehicleInfo.setActivationCode(activationCode);
                vehicleInfo.setDeviceNo(deviceNo);
                vehicleInfo.setUpdateBy(subject);
                vehicleInfo.setUpdateTime(new Date());

                vehicleInfoService.update(vehicleInfo);
                msgResult.setResult(true);
                msgResult.setMessage("绑定成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("绑定失败");
            }
        } catch (Exception ex) {
            log.error(ex.getMessage());
            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
