package com.jpsoft.bus.modules.device.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.jpsoft.bus.modules.bus.entity.DeviceCmdInfo;
import com.jpsoft.bus.modules.bus.service.DeviceCmdInfoService;
import com.jpsoft.bus.modules.common.dto.MessageResult;
import com.jpsoft.bus.modules.sys.entity.DataDictionary;
import com.jpsoft.bus.modules.sys.service.DataDictionaryService;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/bus/deviceCmdApi")
@Slf4j
public class DeviceCmdApiController {
    @Autowired
    private DataDictionaryService dataDictionaryService;

    @Autowired
    private DeviceCmdInfoService deviceCmdInfoService;

    @GetMapping("queryPadLatestVersion")
    @ApiOperation(value="查询平板app最新版本")
    public MessageResult<Map> queryLatestVersion(){
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
             DataDictionary dd  = dataDictionaryService.findByName("平板app版本");

             Map<String,String> data = new HashMap<>();

             data.put("version", dd.getValue());
             data.put("versionCode", dd.getExpand1());
             data.put("url", dd.getExpand2());

            messageResult.setData(data);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @GetMapping("queryXunFeiAppId")
    @ApiOperation(value="查询讯飞AppId")
    public MessageResult<Map> queryXunFeiAppId(){
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            DataDictionary dd  = dataDictionaryService.findByName("讯飞AppId");

            Map<String,String> data = new HashMap<>();

            data.put("appId", dd.getValue());

            messageResult.setData(data);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("downloadCommand")
    @ApiOperation(value="下载设备待执行命令")
    public MessageResult<List> downloadCommand(@RequestBody JSONObject requestBody){
        MessageResult<List> messageResult = new MessageResult<>();

        try {
            String licensePlateNumber = requestBody.getString("licensePlateNumber");
            int limit = requestBody.getInteger("limit");

             if (limit>100){
                 limit = 100;
             }

             List<DeviceCmdInfo> data = deviceCmdInfoService.findByLicensePlateNumber(licensePlateNumber,limit);

            //todo 填写具体代码
            messageResult.setData(data);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("uploadCommandExecuteResult")
    @ApiOperation(value="上传命令执行结果")
    public MessageResult<String> uploadCommandExecuteResult(@RequestBody JSONObject requestBody){
        MessageResult<String> messageResult = new MessageResult<>();

        try {
            String licensePlateNumber = requestBody.getString("licensePlateNumber");
            JSONArray resultList = requestBody.getJSONArray("resultList");

            for (int i=0;i<resultList.size();i++) {
                JSONObject result = resultList.getJSONObject(i);

                String id = result.getString("id");
                int responseCode = result.getInteger("responseCode");
                String responseText = result.getString("responseText");

                DeviceCmdInfo deviceCmdInfo = deviceCmdInfoService.get(id);
                deviceCmdInfo.setResponseCode(responseCode);
                deviceCmdInfo.setResponseText(responseText);
                deviceCmdInfo.setStatus(1); //已执行
                deviceCmdInfo.setUpdateTime(new Date());

                deviceCmdInfoService.update(deviceCmdInfo);
            }

            String data = "ok";

            //todo 填写具体代码
            messageResult.setData(data);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }
}
