package com.jpsoft.bus.modules.driver.controller;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.Page;
import com.jpsoft.bus.config.OSSConfig;
import com.jpsoft.bus.modules.base.dto.PassengerOrderDTO;
import com.jpsoft.bus.modules.base.entity.OrderInfo;
import com.jpsoft.bus.modules.base.entity.PaymentInfo;
import com.jpsoft.bus.modules.base.service.OrderInfoService;
import com.jpsoft.bus.modules.base.service.PaymentInfoService;
import com.jpsoft.bus.modules.bus.dto.DriverBuyTicketDTO;
import com.jpsoft.bus.modules.bus.dto.PassengerDetailDTO;
import com.jpsoft.bus.modules.bus.dto.StationStatusDTO;
import com.jpsoft.bus.modules.bus.dto.VehicleStationDTO;
import com.jpsoft.bus.modules.bus.entity.*;
import com.jpsoft.bus.modules.bus.service.*;
import com.jpsoft.bus.modules.common.dto.MessageResult;
import com.jpsoft.bus.modules.common.dto.Sort;
import com.jpsoft.bus.modules.common.utils.*;
import com.jpsoft.bus.modules.sys.entity.SysLog;
import com.jpsoft.bus.modules.sys.service.SysLogService;
import com.jpsoft.gps.callback.GpsDataCallback;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.bouncycastle.crypto.signers.ECDSASigner;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @author 墨鱼_mo
 * @date 2021-4-2 14:45
 */
@RestController
@RequestMapping("/mobile/driverApi")
@Api(value = "driverApi")
@Slf4j
public class DriverApiController {

    @Value("${jwt.secret}")
    private String jwtSecret;

    @Autowired
    private OSSConfig ossConfig;

    @Autowired
    private PaymentInfoService paymentInfoService;

    @Autowired
    private CompanyInfoService companyInfoService;

    @Autowired
    private ShiftInfoService shiftInfoService;

    @Autowired
    private PriceInfoService priceInfoService;

    @Autowired
    private StationSubInfoService stationSubInfoService;

    @Autowired
    private BaiduService baiduService;

    @Autowired
    private GpsDataInfoService gpsDataInfoService;

    @Autowired
    private OrderInfoService orderInfoService;

    @Autowired
    private GpsService gpsService;

    @Autowired
    private PassengerInfoService passengerInfoService;

    @Autowired
    private StationInfoService stationInfoService;

    @Autowired
    private RouteInfoService routeInfoService;

    @Autowired
    private DriverRecordInfoService driverRecordInfoService;

    @Autowired
    private DriverInfoService driverInfoService;

    @Autowired
    private VehicleInfoService vehicleInfoService;

    @Autowired
    private ValueOperations<String,Object> valueOperations;

    @Autowired
    private GpsDataCallback gpsDataCallback;

    @Autowired
    private SysLogService logService;

    @PostMapping("carActivation")
    @ApiOperation(value = "司机端激活")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "licensePlateNumber", value = "车牌号", required = true, paramType = "form"),
        @ApiImplicitParam(name = "padSerialNumber", value = "平板序列号", required = true, paramType = "form"),
        @ApiImplicitParam(name = "pacsSerialNumber", value = "门禁设备序列号", required = true, paramType = "form")
    })
    public MessageResult<Map> carActivation(String licensePlateNumber, String padSerialNumber, String pacsSerialNumber, HttpServletRequest request){
        MessageResult<Map> messageResult = new MessageResult<>();

        try{
            VehicleInfo vehicleInfo = vehicleInfoService.findByCodeAndCarNum(padSerialNumber,licensePlateNumber);

            if (vehicleInfo == null){
                SysLog sysLog = new SysLog();
                sysLog.setPointcut("车辆激活");
                sysLog.setData(licensePlateNumber + "," + padSerialNumber + "," + pacsSerialNumber);
                sysLog.setRemark("车牌号与当前平板未绑定!");
                sysLog.setRemoteIp(HttpUtil.getIpAddress(request));
                sysLog.setCreateTime(new Date());
                logService.insert(sysLog);

                throw new Exception(sysLog.getRemark());
            }
            else if(!pacsSerialNumber.equals(vehicleInfo.getDeviceNo())){
                SysLog sysLog = new SysLog();
                sysLog.setPointcut("车辆激活");
                sysLog.setData(licensePlateNumber + "," + padSerialNumber + "," + pacsSerialNumber);
                sysLog.setRemark("车牌号与当前打卡设备未绑定!");
                sysLog.setRemoteIp(HttpUtil.getIpAddress(request));
                sysLog.setCreateTime(new Date());
                logService.insert(sysLog);

                throw new Exception(sysLog.getRemark());
            }

            vehicleInfo.setStatus("1");
            vehicleInfoService.update(vehicleInfo);

            String token = JwtUtil.createToken(jwtSecret, vehicleInfo.getId(), DateTime.now().plusDays(30).toDate());

            Map<String,Object> map = new HashMap<>();
            map.put("token",token);
            map.put("vehicleInfo",vehicleInfo);
            messageResult.setResult(true);
            messageResult.setCode(200);
            messageResult.setData(map);

        }catch (Exception ex){
            log.error(ex.getMessage(),ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;

    }

    @PostMapping("findByPhone")
    @ApiOperation(value = "手机号查询司机信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "phone", value = "手机号", required = true, paramType = "query")
    })
    public MessageResult<DriverInfo> findByPhone(String phone){

        MessageResult<DriverInfo> messageResult = new MessageResult<>();
        try{

            DriverInfo driverInfo = driverInfoService.findByPhone(phone);


            if (driverInfo == null){
                throw new Exception("司机不存在");
            }

//            List<ShiftInfo> shiftInfo = shiftInfoService.findByDriverIdAndStatus(driverInfo.getId(),"1");
//            if (shiftInfo.size()>0){
//                throw new Exception("此司机有正在运行的班次，请核对后输入");
//            }
            messageResult.setResult(true);
            messageResult.setCode(200);
            messageResult.setData(driverInfo);

        }catch (Exception ex){
            log.error(ex.getMessage(),ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;

    }

    @PostMapping("driverAttendance")
    @ApiOperation(value = "司机考勤")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "driverId", value = "司机id", required = true, paramType = "form"),
            @ApiImplicitParam(name = "recordTime", value = "考勤时间", required = true, paramType = "form"),
            @ApiImplicitParam(name = "photoBase64Data", value = "照片base64编码", required = true, paramType = "form"),
            @ApiImplicitParam(name = "token", value = "令牌", paramType = "form"),
            @ApiImplicitParam(name = "subject", value = "目标(不传)", paramType = "form")
    })
    public MessageResult<DriverRecordInfo> driverAttendance(@RequestBody JSONObject requestBody, @RequestAttribute String subject) {
        MessageResult<DriverRecordInfo> messageResult = new MessageResult<>();

        try {
            Long driverId = requestBody.getLong("driverId");
            String photoBase64Data = requestBody.getString("photoBase64Data");
            String recordTime = requestBody.getString("recordTime");

            VehicleInfo vehicleInfo = vehicleInfoService.get(subject);

            if (vehicleInfo == null){
                throw new Exception("当前车辆不存在");
            }

            DriverInfo driverInfo = driverInfoService.get(driverId);
            if (driverInfo == null){
                throw new Exception("当前司机不存在");
            }

            String retFileUrl = OSSUtil.uploadBase64(photoBase64Data,driverId+".jpg",ossConfig);
           // String retFileUrl = OSSUtil.upload(ossConfig, "/faceImage", driverInfo.getName(), photoFile.getInputStream());
            DriverRecordInfo driverRecordInfo = new DriverRecordInfo();
            driverRecordInfo.setId(UUID.randomUUID().toString());
            driverRecordInfo.setDriverId(driverInfo.getId());
            driverRecordInfo.setLicensePlateNumber(vehicleInfo.getLicensePlateNumber());
            driverRecordInfo.setRecordTime(DateUtil.parse(recordTime,"yyyy-MM-dd HH:mm:ss"));
            driverRecordInfo.setPlace(vehicleInfo.getLatestAddress());
            driverRecordInfo.setLatitude(vehicleInfo.getLatitude());
            driverRecordInfo.setLongitude(vehicleInfo.getLongitude());
            driverRecordInfo.setImageUrl(retFileUrl);
            driverRecordInfo.setCreateTime(new Date());
            driverRecordInfoService.insert(driverRecordInfo);

            messageResult.setData(driverRecordInfo);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(),ex);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @GetMapping("findLastAttendanceRecord")
    @ApiOperation(value="查询司机今日最后考勤打卡")
    public MessageResult<DriverRecordInfo> findLastAttendanceRecord(@RequestAttribute String subject,Long driverId,String licensePlateNumber){
        MessageResult<DriverRecordInfo> messageResult = new MessageResult<>();

        try {
            VehicleInfo vehicleInfo = vehicleInfoService.get(subject);

            Date startTime = DateTime.now().withTimeAtStartOfDay().toDate();

            //vehicleInfo.getLicensePlateNumber()
            DriverRecordInfo driverRecordInfo = driverRecordInfoService.findLastAttendanceRecord(driverId,licensePlateNumber,startTime);

            //todo 填写具体代码
            messageResult.setData(driverRecordInfo);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @PostMapping("busRouteInfo")
    @ApiOperation(value = "车辆线路信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "token", value = "令牌", paramType = "form"),
            @ApiImplicitParam(name = "subject", value = "目标(不传)", paramType = "form")
    })
    public MessageResult<Map> busRouteInfo(String token, @RequestAttribute String subject) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            VehicleInfo vehicleInfo = vehicleInfoService.get(subject);
            if (vehicleInfo == null){
                throw new Exception("当前车辆不存在");
            }
            if (StringUtils.isBlank(vehicleInfo.getRouteId())){
                throw new Exception("当前车辆还未分配线路");
            }
            RouteInfo routeInfo = routeInfoService.get(vehicleInfo.getRouteId());
            if (routeInfo == null){
                throw new Exception("线路错误");
            }
            //始发站
            List<StationInfo> startStationInfoList = stationInfoService.findByRouteIdAndClassify(routeInfo.getId(),1);
            if (startStationInfoList.size() != 1){
                throw new Exception("线路的始发站配置错误");
            }
            //终点站
            List<StationInfo> endStationInfoList = stationInfoService.findByRouteIdAndClassify(routeInfo.getId(),3);
            if (endStationInfoList.size() != 1){
                throw new Exception("线路的终点站配置错误");
            }


            Map<String,Object> map = new HashMap<>();
            map.put("startStationId",startStationInfoList.get(0).getId());
            map.put("startStationName",startStationInfoList.get(0).getName());
            map.put("endStationId",endStationInfoList.get(0).getId());
            map.put("endStationName",endStationInfoList.get(0).getName());

            messageResult.setData(map);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(),ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("createShift")
    @ApiOperation(value = "开班")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "token", value = "令牌", paramType = "form"),
            @ApiImplicitParam(name = "subject", value = "目标(不传)", paramType = "form"),
            @ApiImplicitParam(name = "driverId", value = "司机id", required = true, paramType = "form"),
            @ApiImplicitParam(name = "startStationId", value = "始发站id", required = true, paramType = "form"),
            @ApiImplicitParam(name = "endStationId", value = "终点站id", required = true, paramType = "form")

    })
    public MessageResult<Map> createShift(@RequestBody JSONObject requestBody, @RequestAttribute String subject) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            Long driverId = requestBody.getLong("driverId");
            String startStationId = requestBody.getString("startStationId");
            String endStationId = requestBody.getString("endStationId");

            VehicleInfo vehicleInfo = vehicleInfoService.get(subject);
            if (vehicleInfo == null){
                throw new Exception("当前车辆不存在");
            }
            DriverInfo driverInfo = driverInfoService.get(driverId);
            if (driverInfo == null){
                throw new Exception("司机不存在");
            }

            List<ShiftInfo> shiftInfoList = shiftInfoService.findByDriverIdAndStatus(driverInfo.getId(),"1");

            if (shiftInfoList.size()>0){
//                throw new Exception("已有正在运行的班次!");
                for (ShiftInfo shiftInfo : shiftInfoList) {
                    if(!shiftInfo.getVehicleId().equals(vehicleInfo.getId())) {
                        shiftInfo.setStatus("2");
                        shiftInfo.setUpdateTime(new Date());
                        shiftInfo.setUpdateBy("在其它车上未收班，则自动收班");

                        shiftInfoService.update(shiftInfo);
                    }
                }
            }

            //始发站
            StationInfo start = stationInfoService.get(startStationId);

            //终点站
            StationInfo end = stationInfoService.get(endStationId);

            if (start == null || end == null){
                throw new Exception("站点错误");
            }

            if (!start.getRouteId().equals(vehicleInfo.getRouteId()) || !end.getRouteId().equals(vehicleInfo.getRouteId())){
                throw new Exception("站点错误");
            }

            ShiftInfo shiftInfo = shiftInfoService.createShift(vehicleInfo,driverId,startStationId,endStationId);

            //司机名称
            shiftInfo.setDriverName(driverInfo.getName());

            //车辆车牌
            shiftInfo.setVehicleNumber(vehicleInfo.getLicensePlateNumber());

            //路线名称
            RouteInfo routeInfo = routeInfoService.get(vehicleInfo.getRouteId());

            shiftInfo.setRouteName(routeInfo.getName());

            //起始站
            shiftInfo.setStartStationName(start.getName());

            //当前站点
            StationInfo currentStation = stationInfoService.get(shiftInfo.getCurrentStationId());

            if(currentStation != null) {
                shiftInfo.setCurrentStationName(currentStation.getName());
            }

            //终点站
            shiftInfo.setEndStationName(end.getName());

            List<StationInfo> stationInfoList = stationInfoService.findByRouteId(shiftInfo.getRouteId());

            Map<String,Object> dataMap = new HashMap<>();
            dataMap.put("shiftInfo", shiftInfo);
            dataMap.put("stationInfoList", stationInfoList);

            messageResult.setResult(true);
            messageResult.setData(dataMap);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(),ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("busShiftInfo")
    @ApiOperation(value = "车辆班次信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "token", value = "令牌", paramType = "form"),
            @ApiImplicitParam(name = "subject", value = "目标(不传)", paramType = "form")
    })
    public MessageResult<Map> busShiftInfo(String token, @RequestAttribute String subject) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {

            VehicleInfo vehicleInfo = vehicleInfoService.get(subject);
            if (vehicleInfo == null){
                throw new Exception("当前车辆不存在");
            }

            List<ShiftInfo> shiftInfoList = shiftInfoService.findByVehicleIdAndStatus(vehicleInfo.getId(),"1");
            if (shiftInfoList.size() == 0){
                throw new Exception("车辆没有相关班次信息");
            }

            ShiftInfo shiftInfo = shiftInfoList.get(0);
            //始发站
            StationInfo startStation = stationInfoService.get(shiftInfo.getStartStationId());
            //终点站
            StationInfo endStation = stationInfoService.get(shiftInfo.getEndStationId());

            //当前站点
            StationInfo currentStation = stationInfoService.get(shiftInfo.getCurrentStationId());

            //根据车辆当前坐标
            StationStatusDTO stationStatusDTO = gpsService.queryStation(vehicleInfo.getId());

            Map<String,Object> map = new HashMap<>();
            map.put("startStationId",startStation.getId());
            map.put("startStationName",startStation.getName());
            map.put("endStationId",endStation.getId());
            map.put("endStationName",endStation.getName());

            if(currentStation != null) {
                map.put("currentStationId", currentStation.getId());
                map.put("currentStationName", currentStation.getName());
            }

            map.put("nextStationId",stationStatusDTO.getNextStationId());
            map.put("nextStationName",stationStatusDTO.getNextStationName());

            RouteInfo routeInfo = routeInfoService.get(vehicleInfo.getRouteId());

            if(routeInfo!=null){
                map.put("routeName", routeInfo.getName());
            }

            messageResult.setData(map);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(),ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("existPassenger")
    @ApiOperation(value="查询乘客是否存在")
    public MessageResult<Map> existPassenger(@RequestBody JSONObject requestBody, @RequestAttribute String subject){
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            Map<String,Object> data = new HashMap<>();

            Long personId = requestBody.getLong("personId");

            VehicleInfo vehicleInfo = vehicleInfoService.get(subject);
            if (vehicleInfo == null){
                throw new Exception("当前车辆不存在");
            }

            List<ShiftInfo> shiftInfoList = shiftInfoService.findByVehicleIdAndStatus(vehicleInfo.getId(),"1");
            if (shiftInfoList.size() == 0){
                throw new Exception("车辆没有相关班次信息");
            }

            //在已上车乘客中查询
            List<PassengerInfo> passengerInfos = passengerInfoService.findByPersonIdShiftIdStatus(personId,shiftInfoList.get(0).getId(),"1");

            if (passengerInfos.size()>0){
                messageResult.setData(data);
                messageResult.setResult(true);
            }
            else{
                messageResult.setResult(false);
            }
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("passengerFaceRegister")
    @ApiOperation(value = "乘客刷脸登记")
    public MessageResult<Map> passengerFaceRegister(@RequestBody JSONObject requestBody, @RequestAttribute String subject) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            Long libId = requestBody.getLong("libId");
            Long personId = requestBody.getLong("personId");
            String photoBase64Data = requestBody.getString("photoBase64Data");
            String recordTime = requestBody.getString("recordTime");
            String localImageUrl = requestBody.getString("localImageUrl");

            //同一辆车相同乘客上下车登记间隔1分钟
            String key = subject + "_" + personId;
            boolean absent = valueOperations.setIfAbsent(key,true,1,TimeUnit.MINUTES);

            if(absent) {
                VehicleInfo vehicleInfo = vehicleInfoService.get(subject);
                if (vehicleInfo == null){
                    throw new Exception("当前车辆不存在");
                }

                List<ShiftInfo> shiftInfoList = shiftInfoService.findByVehicleIdAndStatus(vehicleInfo.getId(),"1");
                if (shiftInfoList.size() == 0){
                    throw new Exception("车辆没有相关班次信息");
                }

                ShiftInfo shiftInfo = shiftInfoList.get(0);
                String retFileUrl = "";

                //只有第一次上车时返回登记照
                if(StringUtils.isNotEmpty(photoBase64Data)){
                    retFileUrl = OSSUtil.uploadBase64(photoBase64Data,personId+".jpg",ossConfig);
                }

                String currentStationId = shiftInfo.getCurrentStationId();
                //班次的当前站点和GPS记录对比验证
                //记录时间往后偏移1min
               /* Date time = DateUtil.parse(recordTime,"yyyy-MM-dd HH:mm:ss");
                Date offsetTime = DateUtil.offsetMinute(time,-1);

                //查询gps记录表中此时间段间的记录
                GpsDataInfo gpsDataInfo = gpsDataInfoService.findByDeviceNoAndCreateTime(vehicleInfo.getGpsDeviceNo(),time,offsetTime);

                if (gpsDataInfo != null && StringUtils.isNotBlank(gpsDataInfo.getStationId())){
                    currentStationId = gpsDataInfo.getStationId();
                }*/

                passengerInfoService.passengerFace(vehicleInfo, shiftInfo, retFileUrl, recordTime, currentStationId,libId, personId,localImageUrl);
                messageResult.setResult(true);
                messageResult.setCode(200);
            }
            else{
                messageResult.setResult(false);
                messageResult.setMessage("乘客刷脸过于频繁，不重复登记!");
            }
        } catch (Exception ex) {
            log.error(ex.getMessage(),ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("busShiftTicketList")
    @ApiOperation(value = "车辆班次购票情况列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "token", value = "令牌", paramType = "form"),
            @ApiImplicitParam(name = "subject", value = "目标(不传)", paramType = "form"),
            @ApiImplicitParam(name = "type", value = "1:未购票，2：待下车，3：待补票，4：全部", required = true, paramType = "form"),
            @ApiImplicitParam(name = "status", value = "0-未乘车、1-上车、2-下车", required = false, paramType = "form"),
            @ApiImplicitParam(name = "shiftId", value = "班次编号", paramType = "form")
    })
    public MessageResult<Map> busShiftTicketList(String token,
                                                 @RequestAttribute String subject,
                                                 @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
                                                 @RequestParam(value = "pageSize", defaultValue = "20") int pageSize,
                                                 @RequestParam(value = "type", defaultValue = "1") String type,
                                                 @RequestParam(value = "status", defaultValue = "1") String status,
                                                 @RequestParam(value = "shiftId", defaultValue = "") String shiftId) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            VehicleInfo vehicleInfo = vehicleInfoService.get(subject);
            if (vehicleInfo == null){
                throw new Exception("当前车辆不存在");
            }

            if (pageSize>100){
                pageSize=100;
            }

            ShiftInfo shiftInfo = null;

            if (StringUtils.isNotEmpty(shiftId)){
                shiftInfo = shiftInfoService.get(shiftId);
            }
            else {
                List<ShiftInfo> shiftInfoList = shiftInfoService.findByVehicleIdAndStatus(vehicleInfo.getId(), "1");

                if(shiftInfoList.size()>0) {
                    shiftInfo = shiftInfoList.get(0);
                }
            }

            if (shiftInfo==null){
                throw new Exception("车辆没有相关班次信息");
            }

            Map<String,Object> searchParams = new HashMap<>();
            searchParams.put("vehicleShiftId",shiftInfo.getId());
            searchParams.put("statusList",status.split(","));

            //未购票
            if ("1".equals(type)){
                searchParams.put("payStatus","0");
            }
            //待下车
            if ("2".equals(type)){
                searchParams.put("notPayStatus","0");
                searchParams.put("downStationId",shiftInfo.getCurrentStationId());
            }
            //待补票
            if ("3".equals(type)){
                searchParams.put("payStatus","2");
            }

            List<Sort> sortList = new ArrayList<>();
//            sortList.add(new Sort("status_","asc"));
            sortList.add(new Sort("create_time","desc"));
            Page<PassengerInfo> page = passengerInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

//            for (PassengerInfo passengerInfo : page.getResult()) {
//                List<OrderInfo> orderInfoList = orderInfoService.findByPassengerIdAndPayStatus(passengerInfo.getId(),20);

                //乘客最后一次购票信息
//                if (orderInfoList.size()>0){
//                    passengerInfo.setTicketType(orderInfoList.get(0).getTicketType());
//                }
//            }

            messageResult.setData(PojoUtils.pageWrapper(page));
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(),ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @PostMapping("passengerInfo")
    @ApiOperation(value = "乘客信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "token", value = "令牌", paramType = "form"),
            @ApiImplicitParam(name = "subject", value = "目标(不传)", paramType = "form"),
            @ApiImplicitParam(name = "id", value = "乘客主id", required = true, paramType = "form")
    })
    public MessageResult<PassengerDetailDTO> passengerInfo(String id,String token, @RequestAttribute String subject) {
        MessageResult<PassengerDetailDTO> messageResult = new MessageResult<>();

        try {
            PassengerDetailDTO passengerDetailDTO = new PassengerDetailDTO();
            VehicleInfo vehicleInfo = vehicleInfoService.get(subject);
            if (vehicleInfo == null){
                throw new Exception("当前车辆不存在");
            }

            PassengerInfo passengerInfo = passengerInfoService.get(id);

            if (passengerInfo == null){
                throw new Exception("无乘客记录");
            }

            List<OrderInfo> orderInfoList = orderInfoService.findByPassengerIdAndPayStatus(id,20);

            passengerDetailDTO.setId(id);
            passengerDetailDTO.setVehicleShiftId(passengerInfo.getVehicleShiftId());
            passengerDetailDTO.setVehicleLibId(passengerInfo.getVehicleLibId());
            passengerDetailDTO.setVehiclePersonId(passengerInfo.getVehiclePersonId());
            passengerDetailDTO.setImageUrl(passengerInfo.getImageUrl());
            passengerDetailDTO.setLocalImageUrl(passengerInfo.getLocalImageUrl());
            passengerDetailDTO.setUpTime(passengerInfo.getUpTime());
            passengerDetailDTO.setUpStationId(passengerInfo.getUpStationId());
            passengerDetailDTO.setTicketType(passengerInfo.getTicketType());

            //上车站点
            StationInfo upStationInfo = stationInfoService.get(passengerInfo.getUpStationId());

            if(upStationInfo!=null) {
                passengerDetailDTO.setUpStationName(upStationInfo.getName());
            }

            passengerDetailDTO.setDownStationId(passengerInfo.getDownStationId());
            passengerDetailDTO.setDownTime(passengerInfo.getDownTime());

            //下车站点
            StationInfo downStationInfo = stationInfoService.get(passengerInfo.getDownStationId());

            if(downStationInfo!=null) {
                passengerDetailDTO.setDownStationName(downStationInfo.getName());
            }

            passengerDetailDTO.setTicketUpStationId(passengerInfo.getTicketUpStationId());

            //购票上车站点
            StationInfo ticketUpStationInfo = stationInfoService.get(passengerInfo.getTicketUpStationId());

            if(ticketUpStationInfo!=null) {
                passengerDetailDTO.setTicketUpStationName(ticketUpStationInfo.getName());
            }

            //购票下车站点
            passengerDetailDTO.setTicketDownStationId(passengerInfo.getTicketDownStationId());

            StationInfo ticketDownStationInfo = stationInfoService.get(passengerInfo.getTicketDownStationId());

            if(ticketDownStationInfo!=null) {
                passengerDetailDTO.setTicketDownStationName(ticketDownStationInfo.getName());
            }

            passengerDetailDTO.setPayStatus(passengerInfo.getPayStatus());
            passengerDetailDTO.setPayStatusName(passengerInfo.getPayStatusName());

            //上下车状态
            passengerDetailDTO.setStatus(passengerInfo.getStatus());
            passengerDetailDTO.setStatusName(passengerInfo.getStatusName());

            BigDecimal payFee = BigDecimal.ZERO;

            List<PassengerOrderDTO> passengerOrderDTOList = new ArrayList<>();

            if (orderInfoList.size()>0){
                for (OrderInfo orderInfo : orderInfoList){
                    PassengerOrderDTO passengerOrderDTO = new PassengerOrderDTO();
                    passengerOrderDTO.setTicketType(orderInfo.getTicketType());
                    passengerOrderDTO.setGoodsTicket(orderInfo.getGoodsTicket());
                    passengerOrderDTO.setPayName(orderInfo.getPayName());
                    passengerOrderDTO.setPayNameStr(orderInfo.getPayNameStr(orderInfo.getPayName()));
                    passengerOrderDTO.setTicketTypeStr(passengerOrderDTO.getTicketTypeStr(orderInfo.getTicketType()));
                    passengerOrderDTO.setTotalFee(orderInfo.getTotalFee());
                    passengerOrderDTOList.add(passengerOrderDTO);
                    payFee = payFee.add(orderInfo.getPayFee());
                }

                passengerDetailDTO.setPassengerOrderDTOList(passengerOrderDTOList);
                passengerDetailDTO.setPayFee(payFee);
            }

            messageResult.setData(passengerDetailDTO);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(),ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("changePassengerStatus")
    @ApiOperation(value="修改乘客乘车状态")
    public MessageResult<PassengerInfo> changePassengerStatus(String id,String status,@RequestAttribute String subject){
        MessageResult<PassengerInfo> messageResult = new MessageResult<>();

        try {
            String data = null;

            //todo 填写具体代码
            PassengerInfo passengerInfo = passengerInfoService.get(id);

            passengerInfo.setStatus(status);
            passengerInfo.setUpdateTime(new Date());
            passengerInfo.setUpdateBy(subject);

            passengerInfoService.update(passengerInfo);

            messageResult.setData(passengerInfo);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("driverCreateOrder")
    @ApiOperation(value = "司机为乘客创建订单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "token", value = "令牌", paramType = "form"),
            @ApiImplicitParam(name = "subject", value = "目标(不传)", paramType = "form"),
            @ApiImplicitParam(name = "id", value = "乘客主id", required = true, paramType = "form"),
            @ApiImplicitParam(name = "ticketUpStationId", value = "乘客买票站点id", required = true, paramType = "form"),
            @ApiImplicitParam(name = "ticketDownStationId", value = "乘客买票下车站点id", required = true, paramType = "form"),
            @ApiImplicitParam(name = "ticketType", value = "购票类型（1：儿童票，2：成人票）", required = true, paramType = "form"),
            @ApiImplicitParam(name = "goodsTicket", value = "货票金额", required = true, paramType = "form"),
            @ApiImplicitParam(name = "totalFee", value = "总金额", required = true, paramType = "form"),
            @ApiImplicitParam(name = "payName", value = "cash：现金，ticket：车票，wechat：微信，alipay：支付宝", required = true, paramType = "form"),
    })
    public MessageResult<DriverBuyTicketDTO> driverCreateOrder(String id,String ticketUpStationId,String ticketDownStationId,String ticketType,String goodsTicket,String totalFee,String payName,String token, @RequestAttribute String subject) {
        MessageResult<DriverBuyTicketDTO> messageResult = new MessageResult<>();

        try {

            VehicleInfo vehicleInfo = vehicleInfoService.get(subject);
            if (vehicleInfo == null){
                throw new Exception("当前车辆不存在");
            }

            String paymentId = "";
            CompanyInfo companyInfo = companyInfoService.get(vehicleInfo.getCompanyId());
            PaymentInfo paymentInfo = paymentInfoService.findByCompanyId(vehicleInfo.getCompanyId());
            if (paymentInfo != null){
                paymentId = paymentInfo.getId();
            }
            else {
                if ("1".equals(companyInfo.getType())){
                    throw new Exception("车辆对应公司未配置收款账户");
                }
                if (!"1".equals(companyInfo.getType())){
                    CompanyInfo companyInfo1 = companyInfoService.get(companyInfo.getParentId());
                    PaymentInfo paymentInfo1 = paymentInfoService.findByCompanyId(companyInfo1.getId());
                    if (paymentInfo1 != null){
                        paymentId = paymentInfo1.getId();
                    }

                }
            }

            if (StringUtils.isBlank(paymentId)){
                throw new Exception("对应公司收款账户异常，请核验");
            }


            List<ShiftInfo> shiftInfoList = shiftInfoService.findByVehicleIdAndStatus(vehicleInfo.getId(),"1");
            if (shiftInfoList.size() == 0){
                throw new Exception("车辆没有相关班次信息");
            }

            PassengerInfo passengerInfo = passengerInfoService.get(id);


            DriverBuyTicketDTO driverBuyTicketDTO = passengerInfoService.driverBuyTicket(passengerInfo,ticketUpStationId,ticketDownStationId,ticketType,goodsTicket,totalFee,payName,paymentId);

            messageResult.setData(driverBuyTicketDTO);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(),ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("ticketAmount")
    @ApiOperation(value = "站点之间的车票金额")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "token", value = "令牌", paramType = "form"),
            @ApiImplicitParam(name = "subject", value = "目标(不传)", paramType = "form"),
            @ApiImplicitParam(name = "ticketUpStationId", value = "乘客买票站点id", required = true, paramType = "form"),
            @ApiImplicitParam(name = "ticketDownStationId", value = "乘客买票下车站点id", required = true, paramType = "form"),
            @ApiImplicitParam(name = "ticketType", value = "购票类型（1：儿童票，2：成人票）", required = true, paramType = "form")
    })
    public MessageResult<Map> ticketAmount(String ticketUpStationId,String ticketDownStationId,String ticketType,String token,String subject) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            PriceInfo priceInfo = priceInfoService.findByStartStationAndEndStation(ticketUpStationId,ticketDownStationId);
            if (priceInfo == null || priceInfo.getPrice().compareTo(BigDecimal.ZERO) <=0){
                PriceInfo priceInfo1 = priceInfoService.findByStartStationAndEndStation(ticketDownStationId,ticketUpStationId);
                if (priceInfo1 == null || priceInfo1.getPrice().compareTo(BigDecimal.ZERO) <= 0){
                    throw new Exception("站点间无费用设置");
                }
                priceInfo = priceInfo1;
            }

            BigDecimal price = priceInfo.getPrice();

            if ("1".equals(ticketType)){
                price = price.divide(new BigDecimal(2));
            }

            Map<String,Object> map = new HashMap<>();
            map.put("price",price);
            map.put("distance",priceInfo.getDistance());

            map.put("upStation", stationInfoService.get(ticketUpStationId));
            map.put("downStation", stationInfoService.get(ticketDownStationId));

            messageResult.setData(map);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(),ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("getPayType")
    @ApiOperation(value = "获取可用支付方式")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "token", value = "令牌", paramType = "form"),
            @ApiImplicitParam(name = "subject", value = "目标(不传)", paramType = "form")
    })
    public MessageResult<Map> getPayType(String token, @RequestAttribute String subject) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {

            VehicleInfo vehicleInfo = vehicleInfoService.get(subject);
            if (vehicleInfo == null){
                throw new Exception("当前车辆不存在");
            }

            List<String> list = new ArrayList<>();
            list.add("cash");
            list.add("ticket");
            PaymentInfo paymentInfo = paymentInfoService.findByCompanyId(vehicleInfo.getCompanyId());
            if (paymentInfo != null){
                if (paymentInfo.getIsOpenWechat()){
                    list.add("wechat");
                }
                if (paymentInfo.getIsOpenAlipay()){
                    list.add("alipay");
                }
            }

            Map<String,Object> map = new HashMap<>();
            map.put("list",list);


            messageResult.setData(map);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(),ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @PostMapping("busShiftAllStation")
    @ApiOperation(value = "车辆班次所有站点信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "subject", value = "目标(不传)", paramType = "form")
    })
    public MessageResult<List<VehicleStationDTO>> busShiftAllStation(@RequestAttribute String subject) {
        MessageResult<List<VehicleStationDTO>> messageResult = new MessageResult<>();

        try {
            VehicleInfo vehicleInfo = vehicleInfoService.get(subject);
            if (vehicleInfo == null){
                throw new Exception("当前车辆不存在");
            }

            List<ShiftInfo> shiftInfoList = shiftInfoService.findByVehicleIdAndStatus(vehicleInfo.getId(),"1");
            if (shiftInfoList.size() == 0){
                throw new Exception("车辆没有相关班次信息");
            }

            ShiftInfo shiftInfo = shiftInfoList.get(0);

            List<StationInfo> stationInfoList = stationInfoService.findByRouteId(shiftInfo.getRouteId());

            List<VehicleStationDTO> vehicleStationDTOList = new ArrayList<>();
            if (stationInfoList.size()>0){
                vehicleStationDTOList = stationInfoService.getVehicleStationDTO(shiftInfo,stationInfoList);
            }


            messageResult.setData(vehicleStationDTOList);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(),ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("orderDetailInfo")
    @ApiOperation(value = "查看订单状态")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "token", value = "令牌", paramType = "form"),
            @ApiImplicitParam(name = "subject", value = "目标(不传)", paramType = "form"),
            @ApiImplicitParam(name = "id", value = "订单主id", paramType = "form")
    })
    public MessageResult<OrderInfo> orderDetailInfo(String id,String token,@RequestAttribute String subject) {
        MessageResult<OrderInfo> messageResult = new MessageResult<>();

        try {
            OrderInfo orderInfo = orderInfoService.get(id);

            if (orderInfo == null){
                throw new Exception("订单不存在");
            }

            messageResult.setData(orderInfo);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(),ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("updateCurrentStation")
    @ApiOperation(value="更新当前站点")
    public MessageResult<String> updateCurrentStation(String currentStationId,@RequestAttribute String subject,HttpServletRequest request){
        MessageResult<String> messageResult = new MessageResult<>();

        try {
            VehicleInfo vehicleInfo = vehicleInfoService.get(subject);
            if (vehicleInfo == null){
                throw new Exception("当前车辆不存在");
            }

            List<ShiftInfo> shiftInfoList = shiftInfoService.findByVehicleIdAndStatus(vehicleInfo.getId(),"1");
            if (shiftInfoList.size() == 0){
                throw new Exception("车辆没有相关班次信息");
            }

            ShiftInfo shiftInfo = shiftInfoList.get(0);
            StationInfo currentStation = stationInfoService.get(currentStationId);

            //记录当前站点的校正记录，以便用来记录站点定位
            if(currentStation!=null){
                SysLog sysLog = new SysLog();
                sysLog.setPointcut("站点校正");
                sysLog.setData("车牌号:" + vehicleInfo.getLicensePlateNumber() + ",经纬度:" +  vehicleInfo.getLongitude() + "." + vehicleInfo.getLatitude());
                sysLog.setRemark("当前站点:" + currentStation.getName() + ",id=" + currentStationId);
                sysLog.setRemoteIp(HttpUtil.getIpAddress(request));
                sysLog.setCreateTime(new Date());
                logService.insert(sysLog);
            }
            else{
                throw new Exception("站点:" + currentStationId + "不存在!");
            }

            if(vehicleInfo.getLongitude()!=null && vehicleInfo.getLatitude()!=null) {
                //如果当前车辆与校正站点距离1km内
                if (gpsService.matchDistance(currentStation,vehicleInfo,1000)){
                    //创建一个站点附属站点
                    StationSubInfo stationSubInfo = stationSubInfoService.findByStationIdStartEnd(currentStationId, shiftInfo.getStartStationId(), shiftInfo.getEndStationId());
                    if (stationSubInfo == null) {
                        StationSubInfo stationSubInfo1 = new StationSubInfo();
                        stationSubInfo1.setId(UUID.randomUUID().toString());
                        stationSubInfo1.setStationId(currentStationId);
                        stationSubInfo1.setStartStationId(shiftInfo.getStartStationId());
                        stationSubInfo1.setEndStationId(shiftInfo.getEndStationId());
                        stationSubInfo1.setLongitude(new Double(vehicleInfo.getLongitude()));
                        stationSubInfo1.setLatitude(new Double(vehicleInfo.getLatitude()));
                        stationSubInfo1.setRadius(50);
                        stationSubInfo1.setCreateTime(new Date());
                        stationSubInfoService.insert(stationSubInfo1);
                    } else {
                        stationSubInfo.setLongitude(new Double(vehicleInfo.getLongitude()));
                        stationSubInfo.setLatitude(new Double(vehicleInfo.getLatitude()));
                        stationSubInfo.setUpdateTime(new Date());
                        stationSubInfoService.update(stationSubInfo);
                    }

                    //重新计算当前乘客状态
                    gpsDataCallback.receive(vehicleInfo.getGpsDeviceNo(),
                            String.valueOf(vehicleInfo.getLongitude()),
                            String.valueOf(vehicleInfo.getLatitude()),
                            true);
                }
                else{
                    //当前车辆位置距离站点过于遥远，将车辆直接设置到站点位置，不添加站点入口
                    if(currentStation.getLongitude()!=null && currentStation.getLatitude()!=null) {
                        gpsDataCallback.receive(vehicleInfo.getGpsDeviceNo(),
                                String.valueOf(currentStation.getLongitude()),
                                String.valueOf(currentStation.getLatitude()),
                                true);
                    }
                }
            }

            shiftInfo.setCurrentStationId(currentStationId);
            shiftInfo.setUpdateTime(new Date());
            shiftInfo.setUpdateBy(subject);

            shiftInfoService.update(shiftInfo);

            //todo 填写具体代码
            messageResult.setData(currentStationId);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("finishShift")
    @ApiOperation(value="收班")
    public MessageResult<Date> finishShift(@RequestAttribute String subject){
        MessageResult<Date> messageResult = new MessageResult<>();

        try {
            VehicleInfo vehicleInfo = vehicleInfoService.get(subject);
            if (vehicleInfo == null){
                throw new Exception("当前车辆不存在");
            }

            List<ShiftInfo> shiftInfoList = shiftInfoService.findByVehicleIdAndStatus(vehicleInfo.getId(),"1");
            if (shiftInfoList.size() == 0){
                throw new Exception("车辆没有相关班次信息");
            }
            ShiftInfo shiftInfo = shiftInfoList.get(0);
            shiftInfo.setUpdateBy(subject);
            //收班
            shiftInfoService.finishShift(shiftInfo);



            /*shiftInfo.setStatus("2");
            shiftInfo.setFinishTime(new Date());
            shiftInfo.setUpdateTime(new Date());
            shiftInfo.setUpdateBy(subject);

            shiftInfoService.update(shiftInfo);*/

            //todo 填写具体代码
            messageResult.setData(shiftInfo.getFinishTime());
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("queryBusShiftList")
    @ApiOperation(value="查询车辆班次列表")
    public MessageResult<Map> queryBusShiftList(
            @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
            @RequestParam(value = "pageSize", defaultValue = "20") int pageSize,
            @RequestAttribute String subject){
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            Map<String,Object> searchParams = new HashMap<>();

            searchParams.put("vehicleId", subject);

            List<Sort> sortList = new ArrayList<>();

            sortList.add(new Sort("create_time","desc"));

            if (pageSize>100){
                pageSize=100;
            }

            Page<ShiftInfo> page = shiftInfoService.pageSearch(searchParams,pageIndex,pageSize,true, sortList);

            Map<String,Object> dataMap = PojoUtils.pageWrapper(page);

//            List<Map> mapList = new ArrayList<>();
//
//            VehicleInfo vehicleInfo = vehicleInfoService.get(subject);
//
//            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm");
//
//            for (ShiftInfo shiftInfo : page.getResult()) {
//                Map<String,Object> map = new HashMap<>();
//
//                //查询司机、线路、车牌、起始站、终点站、开班时间、收班时间
//                DriverInfo driverInfo = driverInfoService.get(shiftInfo.getDriverId());
//                RouteInfo routeInfo = routeInfoService.get(shiftInfo.getRouteId());
//                //起始站
//                StationInfo startStation = stationInfoService.get(shiftInfo.getStartStationId());
//                //终点站
//                StationInfo endStation = stationInfoService.get(shiftInfo.getEndStationId());
//
//                if (driverInfo!=null){
//                    map.put("driverName", driverInfo.getName());
//                }
//
//                if (routeInfo!=null) {
//                    map.put("routeName", routeInfo.getName());
//                }
//
//                map.put("licensePlateNumber", vehicleInfo.getLicensePlateNumber());
//
//                if (startStation!=null){
//                    map.put("startStationName", startStation.getName());
//                }
//
//                if(endStation!=null) {
//                    map.put("endStationName", endStation.getName());
//                }
//
//                if (shiftInfo.getCreateTime()!=null){
//                    map.put("createTime", sdf.format(shiftInfo.getCreateTime()));
//                }
//
//                if (shiftInfo.getFinishTime()!=null){
//                    map.put("finishTime", sdf.format(shiftInfo.getFinishTime()));
//                }
//
//                mapList.add(map);
//            }

//            dataMap.put("data", mapList);

            messageResult.setData(dataMap);

            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }
}
