package com.jpsoft.bus.modules.merchant.controller;

import com.github.pagehelper.Page;
import com.jpsoft.bus.modules.bus.entity.CompanyInfo;
import com.jpsoft.bus.modules.bus.service.CompanyInfoService;
import com.jpsoft.bus.modules.common.dto.MessageResult;
import com.jpsoft.bus.modules.common.dto.Sort;
import com.jpsoft.bus.modules.common.utils.PojoUtils;
import com.jpsoft.bus.modules.merchant.entity.AccountInfo;
import com.jpsoft.bus.modules.merchant.entity.CapitalInfo;
import com.jpsoft.bus.modules.merchant.service.AccountInfoService;
import com.jpsoft.bus.modules.merchant.service.CapitalInfoService;
import com.jpsoft.bus.modules.sys.entity.User;
import com.jpsoft.bus.modules.sys.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/merchant/accountInfo")
@Api(description = "商户")
public class AccountInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private AccountInfoService accountInfoService;
    @Autowired
    private CompanyInfoService companyInfoService;
    @Autowired
    private UserService userService;
    @Autowired
    private CapitalInfoService capitalInfoService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<AccountInfo> create(){
        MessageResult<AccountInfo> msgResult = new MessageResult<>();

        AccountInfo accountInfo = new AccountInfo();

        msgResult.setData(accountInfo);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<AccountInfo> add(@RequestBody AccountInfo accountInfo,@RequestAttribute String subject){
        MessageResult<AccountInfo> msgResult = new MessageResult<>();

        try {
            accountInfo.setId(UUID.randomUUID().toString());
            accountInfo.setDelFlag(false);
            accountInfo.setCreateBy(subject);
            accountInfo.setCreateTime(new Date());
            
            int affectCount = accountInfoService.insert(accountInfo);

            if (affectCount > 0) {
                CapitalInfo ci = capitalInfoService.findByMerchantId(accountInfo.getId());
                if(ci == null) {
                    ci = new CapitalInfo();
                    ci.setId(UUID.randomUUID().toString());
                    ci.setMerchantId(accountInfo.getId());
                    ci.setAlreadyWithdrawalAmount(new BigDecimal(0));
                    ci.setOnlineTotalRevenue(new BigDecimal(0));
                    ci.setTotalRevenue(new BigDecimal(0));
                    ci.setWithdrawalAmount(new BigDecimal(0));

                    ci.setDelFlag(false);
                    ci.setCreateBy(subject);
                    ci.setCreateTime(new Date());
                    capitalInfoService.insert(ci);
                }
                msgResult.setResult(true);
                msgResult.setData(accountInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<AccountInfo> edit(@PathVariable("id") String id){
        MessageResult<AccountInfo> msgResult = new MessageResult<>();

        try {
            AccountInfo accountInfo = accountInfoService.get(id);

            if (accountInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(accountInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<AccountInfo> update(@RequestBody AccountInfo accountInfo,@RequestAttribute String subject){
        MessageResult<AccountInfo> msgResult = new MessageResult<>();

        try {
            accountInfo.setUpdateBy(subject);
            accountInfo.setUpdateTime(new Date());
            
            int affectCount = accountInfoService.update(accountInfo);

            if (affectCount > 0) {
                CapitalInfo ci = capitalInfoService.findByMerchantId(accountInfo.getId());
                if(ci == null) {
                    ci = new CapitalInfo();
                    ci.setId(UUID.randomUUID().toString());
                    ci.setMerchantId(accountInfo.getId());
                    ci.setAlreadyWithdrawalAmount(new BigDecimal(0));
                    ci.setOnlineTotalRevenue(new BigDecimal(0));
                    ci.setTotalRevenue(new BigDecimal(0));
                    ci.setWithdrawalAmount(new BigDecimal(0));

                    ci.setDelFlag(false);
                    ci.setCreateBy(subject);
                    ci.setCreateTime(new Date());
                    capitalInfoService.insert(ci);
                }
                msgResult.setResult(true);
                msgResult.setData(accountInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            AccountInfo accountInfo = accountInfoService.get(id);
            accountInfo.setDelFlag(true);
            accountInfo.setUpdateBy(subject);
            accountInfo.setUpdateTime(new Date());

            int affectCount = accountInfoService.update(accountInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                AccountInfo accountInfo = accountInfoService.get(id);
                accountInfo.setDelFlag(true);
                accountInfo.setUpdateBy(subject);
                accountInfo.setUpdateTime(new Date());

                affectCount += accountInfoService.update(accountInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="name",defaultValue="") String name,
            @RequestParam(value="companyId",defaultValue="") String companyId,
            @RequestParam(value="phone",defaultValue="") String phone,
            @RequestParam(value="frozen",defaultValue="") String frozen,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("b.sort_no","asc"));
        sortList.add(new Sort("a.create_time","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        if (StringUtils.isNotEmpty(name)) {
            searchParams.put("name","%" + name + "%");
        }
        if (StringUtils.isNotEmpty(phone)) {
            searchParams.put("phone","%" + phone + "%");
        }
        if (StringUtils.isNotEmpty(frozen)) {
            searchParams.put("frozen",frozen);
        }
        if (StringUtils.isNotEmpty(companyId)) {
            CompanyInfo companyInfo = companyInfoService.get(companyId);
            searchParams.put("companyCode",companyInfo.getCode() + "%");
        }else{
            if (!userService.hasRole(subject, "SYSADMIN")) {
                User user = userService.get(subject);
                CompanyInfo companyInfo = companyInfoService.get(user.getCompanyId());
                searchParams.put("companyCode", companyInfo.getCode() + "%");
            }
        }

        Page<AccountInfo> page = accountInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
