package com.jpsoft.bus.modules.merchant.controller;

import com.github.pagehelper.Page;
import com.jpsoft.bus.modules.common.dto.MessageResult;
import com.jpsoft.bus.modules.common.dto.Sort;
import com.jpsoft.bus.modules.common.utils.PojoUtils;
import com.jpsoft.bus.modules.merchant.entity.CapitalInfo;
import com.jpsoft.bus.modules.merchant.service.CapitalInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/merchant/capitalInfo")
@Api(description = "商户钱包")
public class CapitalInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private CapitalInfoService capitalInfoService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<CapitalInfo> create(){
        MessageResult<CapitalInfo> msgResult = new MessageResult<>();

        CapitalInfo capitalInfo = new CapitalInfo();

        msgResult.setData(capitalInfo);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<CapitalInfo> add(@RequestBody CapitalInfo capitalInfo,@RequestAttribute String subject){
        MessageResult<CapitalInfo> msgResult = new MessageResult<>();

        try {
            capitalInfo.setId(UUID.randomUUID().toString());
            capitalInfo.setDelFlag(false);
            capitalInfo.setCreateBy(subject);
            capitalInfo.setCreateTime(new Date());
            
            int affectCount = capitalInfoService.insert(capitalInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(capitalInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<CapitalInfo> edit(@PathVariable("id") String id){
        MessageResult<CapitalInfo> msgResult = new MessageResult<>();

        try {
            CapitalInfo capitalInfo = capitalInfoService.get(id);

            if (capitalInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(capitalInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<CapitalInfo> update(@RequestBody CapitalInfo capitalInfo,@RequestAttribute String subject){
        MessageResult<CapitalInfo> msgResult = new MessageResult<>();

        try {
            capitalInfo.setUpdateBy(subject);
            capitalInfo.setUpdateTime(new Date());
            
            int affectCount = capitalInfoService.update(capitalInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(capitalInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            CapitalInfo capitalInfo = capitalInfoService.get(id);
            capitalInfo.setDelFlag(true);
            capitalInfo.setUpdateBy(subject);
            capitalInfo.setUpdateTime(new Date());

            int affectCount = capitalInfoService.update(capitalInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                CapitalInfo capitalInfo = capitalInfoService.get(id);
                capitalInfo.setDelFlag(true);
                capitalInfo.setUpdateBy(subject);
                capitalInfo.setUpdateTime(new Date());

                affectCount += capitalInfoService.update(capitalInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        Page<CapitalInfo> page = capitalInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
