package com.jpsoft.bus.modules.merchant.controller;

import com.github.pagehelper.Page;
import com.jpsoft.bus.modules.bus.entity.CompanyInfo;
import com.jpsoft.bus.modules.bus.service.CompanyInfoService;
import com.jpsoft.bus.modules.common.dto.MessageResult;
import com.jpsoft.bus.modules.common.dto.Sort;
import com.jpsoft.bus.modules.common.utils.PojoUtils;
import com.jpsoft.bus.modules.merchant.entity.AccountInfo;
import com.jpsoft.bus.modules.merchant.entity.CapitalInfo;
import com.jpsoft.bus.modules.merchant.entity.WithdrawalRecord;
import com.jpsoft.bus.modules.merchant.service.AccountInfoService;
import com.jpsoft.bus.modules.merchant.service.CapitalInfoService;
import com.jpsoft.bus.modules.merchant.service.WithdrawalRecordService;
import com.jpsoft.bus.modules.sys.entity.User;
import com.jpsoft.bus.modules.sys.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/merchant/withdrawalRecord")
@Api(description = "商户提现")
public class WithdrawalRecordController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private WithdrawalRecordService withdrawalRecordService;
    @Autowired
    private AccountInfoService accountInfoService;
    @Autowired
    private CapitalInfoService capitalInfoService;
    @Autowired
    private CompanyInfoService companyInfoService;
    @Autowired
    private UserService userService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<WithdrawalRecord> create(){
        MessageResult<WithdrawalRecord> msgResult = new MessageResult<>();

        WithdrawalRecord withdrawalRecord = new WithdrawalRecord();

        msgResult.setData(withdrawalRecord);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<WithdrawalRecord> add(@RequestBody WithdrawalRecord withdrawalRecord,@RequestAttribute String subject){
        MessageResult<WithdrawalRecord> msgResult = new MessageResult<>();

        try {
            withdrawalRecord.setId(UUID.randomUUID().toString());
            withdrawalRecord.setDelFlag(false);
            withdrawalRecord.setCreateBy(subject);
            withdrawalRecord.setCreateTime(new Date());
            
            int affectCount = withdrawalRecordService.insert(withdrawalRecord);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(withdrawalRecord);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<WithdrawalRecord> edit(@PathVariable("id") String id){
        MessageResult<WithdrawalRecord> msgResult = new MessageResult<>();

        try {
            WithdrawalRecord withdrawalRecord = withdrawalRecordService.get(id);

            if (withdrawalRecord != null) {
                msgResult.setResult(true);
                msgResult.setData(withdrawalRecord);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<WithdrawalRecord> update(@RequestBody WithdrawalRecord withdrawalRecord,@RequestAttribute String subject){
        MessageResult<WithdrawalRecord> msgResult = new MessageResult<>();

        try {
            withdrawalRecord.setUpdateBy(subject);
            withdrawalRecord.setUpdateTime(new Date());
            
            int affectCount = withdrawalRecordService.update(withdrawalRecord);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(withdrawalRecord);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            WithdrawalRecord withdrawalRecord = withdrawalRecordService.get(id);
            withdrawalRecord.setDelFlag(true);
            withdrawalRecord.setUpdateBy(subject);
            withdrawalRecord.setUpdateTime(new Date());

            int affectCount = withdrawalRecordService.update(withdrawalRecord);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                WithdrawalRecord withdrawalRecord = withdrawalRecordService.get(id);
                withdrawalRecord.setDelFlag(true);
                withdrawalRecord.setUpdateBy(subject);
                withdrawalRecord.setUpdateTime(new Date());

                affectCount += withdrawalRecordService.update(withdrawalRecord);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="name",defaultValue="") String name,
            @RequestParam(value="companyId",defaultValue="") String companyId,
            @RequestParam(value="phone",defaultValue="") String phone,
            @RequestParam(value="status",defaultValue="") String status,
            @RequestParam(value="withdrawalNo",defaultValue="") String withdrawalNo,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        if (StringUtils.isNotEmpty(name)) {
            searchParams.put("name","%" + name + "%");
        }
        if (StringUtils.isNotEmpty(phone)) {
            searchParams.put("phone","%" + phone + "%");
        }
        if (StringUtils.isNotEmpty(status)) {
            searchParams.put("status",status);
        }
        if (StringUtils.isNotEmpty(withdrawalNo)) {
            searchParams.put("withdrawalNo","%" + withdrawalNo + "%");
        }

        if (StringUtils.isNotEmpty(companyId)) {
            CompanyInfo companyInfo = companyInfoService.get(companyId);
            searchParams.put("companyCode",companyInfo.getCode() + "%");
        }else{
            if (!userService.hasRole(subject, "SYSADMIN")) {
                User user = userService.get(subject);
                CompanyInfo companyInfo = companyInfoService.get(user.getCompanyId());
                searchParams.put("companyCode", companyInfo.getCode() + "%");
            }
        }

        Page<WithdrawalRecord> page = withdrawalRecordService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);
        for(WithdrawalRecord wr : page.getResult()){
            AccountInfo ac = accountInfoService.get(wr.getMerchantId());
            wr.setAccountInfo(ac);
            CapitalInfo ci = capitalInfoService.findByMerchantId(wr.getMerchantId());
            wr.setCapitalInfo(ci);
        }

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value="审核")
    @RequestMapping(value = "approval",method = RequestMethod.POST)
    public MessageResult<String> approval(
            String id,String status,
            @RequestAttribute String subject) throws Exception{

        //当前用户ID
        System.out.println(subject);

        MessageResult<String> msgResult = new MessageResult<>();

        try{

            WithdrawalRecord wr = withdrawalRecordService.get(id);
            if(wr == null){
                throw new Exception("查无此项申请!");
            }

            AccountInfo ai = accountInfoService.get(wr.getMerchantId());
            if(ai == null){
                throw new Exception("查无此营收用户!");
            }

            CapitalInfo ci = capitalInfoService.findByMerchantId(wr.getMerchantId());
            if(ci == null){
                ci = new CapitalInfo();
                ci.setId(UUID.randomUUID().toString());
                ci.setMerchantId(wr.getMerchantId());
                ci.setAlreadyWithdrawalAmount(new BigDecimal(0));
                ci.setOnlineTotalRevenue(new BigDecimal(0));
                ci.setTotalRevenue(new BigDecimal(0));
                ci.setWithdrawalAmount(new BigDecimal(0));

                ci.setDelFlag(false);
                ci.setCreateBy(subject);
                ci.setCreateTime(new Date());
                capitalInfoService.insert(ci);

                throw new Exception("该用户账户有误!");
            }else{
                if("1".equals(status)){
                    //withdrawal_amount 可提现金额
                    //already_withdrawal_amount 已提现金额
                    BigDecimal withdrawalAmount = ci.getWithdrawalAmount();
                    BigDecimal alreadyWithdrawalAmount = ci.getAlreadyWithdrawalAmount();
                    //正在提现金额
                    BigDecimal applyWithdrawalAmount = wr.getApplyWithdrawalAmount();

                    if(withdrawalAmount.compareTo(applyWithdrawalAmount) < 0 ) {
                        //小于
                        throw new Exception("可提现金额不足");
                    }else{
                        BigDecimal newWithdrawalAmount =  withdrawalAmount.subtract(applyWithdrawalAmount);
                        BigDecimal newAlreadyWithdrawalAmount =  alreadyWithdrawalAmount.add(applyWithdrawalAmount);

                        ci.setWithdrawalAmount(newWithdrawalAmount);
                        ci.setAlreadyWithdrawalAmount(newAlreadyWithdrawalAmount);
                        ci.setUpdateBy(subject);
                        ci.setUpdateTime(new Date());

                        capitalInfoService.update(ci);
                    }
                }

                wr.setStatus(status);
                wr.setUpdateBy(subject);
                wr.setUpdateTime(new Date());
                wr.setExamineBy(subject);
                wr.setExamineTime(new Date());
                withdrawalRecordService.update(wr);
            }

            msgResult.setResult(true);
            msgResult.setMessage("审核完成");
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
