package com.jpsoft.bus.modules.mobile.controller;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.symmetric.DES;
import cn.hutool.db.sql.Order;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.PropertyMetadata;
import com.github.pagehelper.Page;
import com.jpsoft.bus.config.OSSConfig;
import com.jpsoft.bus.modules.base.dto.MergeOrderDTO;
import com.jpsoft.bus.modules.base.dto.PassengerOrderDTO;
import com.jpsoft.bus.modules.base.entity.*;
import com.jpsoft.bus.modules.base.service.*;
import com.jpsoft.bus.modules.bus.dto.*;
import com.jpsoft.bus.modules.bus.entity.*;
import com.jpsoft.bus.modules.bus.service.*;
import com.jpsoft.bus.modules.common.dto.MessageResult;
import com.jpsoft.bus.modules.common.dto.Sort;
import com.jpsoft.bus.modules.common.utils.*;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.gavaghan.geodesy.Ellipsoid;
import org.gavaghan.geodesy.GlobalCoordinates;
import org.joda.time.DateTime;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import sun.security.krb5.internal.crypto.Des3;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @author 墨鱼_mo
 * @date 2021-3-26 8:57
 */
@RestController
@RequestMapping("/mobile/passengerApi")
@Api(value = "passengerApi")
@Slf4j
public class PassengerApiController {

    @Value("${jwt.secret}")
    private String jwtSecret;

    @Autowired
    private OSSConfig ossConfig;

    @Autowired
    private ProblemFeedbackService problemFeedbackService;

    @Autowired
    private PassengerMessageService passengerMessageService;

    @Autowired
    private RegionInfoService regionInfoService;

    @Autowired
    private BaiduService baiduService;

    @Autowired
    private HelpCenterService helpCenterService;

    @Autowired
    private OrderInfoService orderInfoService;

    @Autowired
    private PassengerInfoService passengerInfoService;

    @Autowired
    private StationRemindService stationRemindService;

    @Autowired
    private MergeOrderInfoService mergeOrderInfoService;

    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    private UserInfoService userInfoService;

    @Autowired
    private ShiftInfoService shiftInfoService;

    @Autowired
    private VehicleInfoService vehicleInfoService;

    @Autowired
    private DriverInfoService driverInfoService;

    @Autowired
    private RouteInfoService routeInfoService;

    @Autowired
    private StationInfoService stationInfoService;

    @PostMapping
    @ApiOperation(value = "通过车牌号查询详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "token", value = "令牌", required = false, paramType = "query"),
            @ApiImplicitParam(name = "subject", value = "令牌", required = false, paramType = "query"),
            @ApiImplicitParam(name = "carNum", value = "车牌号", required = true, paramType = "form")
    })
    public MessageResult<Map> findByCarNum(String token, @RequestAttribute String subject, String carNum) {

        MessageResult<Map> messageResult = new MessageResult<>();
        try {

        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;

    }


    @PostMapping("getVerifyCode")
    @ApiOperation(value = "获取短信验证码(公开接口)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "phone", value = "电话", required = true, paramType = "form")
    })
    public MessageResult<String> getVerifyCode(String phone) {
        MessageResult<String> messageResult = new MessageResult<>();

        try {

            String key = "SMS_" + phone;

            String verifyCode = (String) redisTemplate.opsForValue().get(key);

            if (StringUtils.isEmpty(verifyCode)) {
                verifyCode = SMSUtil.generateNumberString(6);
                JSONObject verifyCodeJSON = new JSONObject();
                verifyCodeJSON.put("code", verifyCode);

                messageResult = SMSUtil.send(phone, "车信达", "SMS_49390047", verifyCodeJSON.toString());

                //设置当前用户的验证码,5分钟内有效
                redisTemplate.opsForValue().set(key, verifyCode, 5, TimeUnit.MINUTES);
            }

            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setResult(false);
            messageResult.setCode(400);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("validateCode")
    @ApiOperation(value = "验证短信验证码(公开接口)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "phone", value = "手机号", required = true, paramType = "form"),
            @ApiImplicitParam(name = "verifyCode", value = "验证码", required = true, paramType = "form"),
            @ApiImplicitParam(name = "openId", value = "微信openId", paramType = "form")
    })
    public MessageResult<Map> validateCode(String phone, String verifyCode, String openId) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            String smsKey = "SMS_" + phone;

            String beforeVerifyCode = (String) redisTemplate.opsForValue().get(smsKey);

            if (StringUtils.isBlank(openId)) {
                throw new Exception("用户标识为空");
            }

            if (StringUtils.isEmpty(beforeVerifyCode)) {
                throw new Exception("验证码已过期!");
            }

            if (!beforeVerifyCode.equals(verifyCode)) {
                throw new Exception("验证码错误!");
            }

            DES3 des3 = new DES3();

            String token;
            Long userId;

            UserInfo userInfo = userInfoService.findByPhone(phone);
            if (userInfo == null) {
                UserInfo userInfo1 = new UserInfo();
                userInfo1.setName("乘客_" + com.jpsoft.bus.modules.common.utils.StringUtils.random(8, com.jpsoft.bus.modules.common.utils.StringUtils.RandomType.ALL));
                userInfo1.setPhone(phone);
                userInfo1.setPassword(des3.encrypt(jwtSecret, phone));
                userInfo1.setAcceptMessage(true);
                userInfo1.setCreateTime(new Date());
                userInfo1.setDelFlag(false);
                userInfo1.setOpenId(openId);
                userInfoService.insert(userInfo1);
                userId = userInfo1.getId();
                userInfo = userInfo1;
            } else {
                userInfo.setOpenId(openId);
                userInfoService.update(userInfo);
                userId = userInfo.getId();
            }
            token = JwtUtil.createToken(jwtSecret, String.valueOf(userId), DateTime.now().plusDays(30).toDate());

            Map<String, Object> map = new HashMap<>();
            map.put("token", token);
            map.put("userInfo", userInfo);

            messageResult.setData(map);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage());
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @PostMapping("getShiftInfo")
    @ApiOperation(value = "扫码获取当前班次")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "车辆id", required = true, paramType = "form")
    })
    public MessageResult<Map> getShiftInfo(String id) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {

            List<ShiftInfo> shiftInfoList = shiftInfoService.findByVehicleIdAndStatus(id, "1");

            if (shiftInfoList.size() == 0) {
                throw new Exception("当前车辆没有正运行的班次");
            }

            //班次信息
            ShiftInfo shiftInfo = shiftInfoList.get(0);

            DriverInfo driverInfo = driverInfoService.get(shiftInfo.getDriverId());

            //车辆信息
            VehicleInfo vehicleInfo = vehicleInfoService.get(id);

            //线路
            RouteInfo routeInfo = routeInfoService.get(shiftInfo.getRouteId());

            //开始站点
            StationInfo startStation = stationInfoService.get(shiftInfo.getStartStationId());
            //结束站点
            StationInfo endStation = stationInfoService.get(shiftInfo.getEndStationId());

            //线路所有站点
            List<StationInfo> stationInfoList = stationInfoService.findByRouteId(shiftInfo.getRouteId());

            //   List<VehicleStationDTO> vehicleStationDTOList = stationInfoService.getVehicleStationDTO(shiftInfo,stationInfoList);


            Map<String, Object> map = new HashMap<>();
            map.put("startStation", startStation.getName());
            map.put("startStationId", startStation.getId());
            map.put("startStationCoordinate", startStation.getLongitude() + "|" + startStation.getLatitude());
            map.put("endStation", endStation.getName());
            map.put("endStationId", endStation.getId());
            map.put("endStationCoordinate", endStation.getLongitude() + "|" + endStation.getLatitude());
            map.put("startTime", routeInfo.getStartTime());
            map.put("endTime", routeInfo.getEndTime());
            map.put("routeName", routeInfo.getName());
            map.put("mapPath", routeInfo.getMapPath());
            map.put("driver", driverInfo);
            map.put("licensePlateNumber", vehicleInfo.getLicensePlateNumber());
            //    map.put("stationList",vehicleStationDTOList);
            messageResult.setData(map);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("getVehicleStationInfo")
    @ApiOperation(value = "获取车辆站点的位置")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "车辆id", required = true, paramType = "form")
    })
    public MessageResult<List<VehicleStationDTO>> getVehicleStationInfo(String id) {
        MessageResult<List<VehicleStationDTO>> messageResult = new MessageResult<>();

        try {

            List<ShiftInfo> shiftInfoList = shiftInfoService.findByVehicleIdAndStatus(id, "1");

            if (shiftInfoList.size() == 0) {
                throw new Exception("当前车辆没有正运行的班次");
            }

            //班次信息
            ShiftInfo shiftInfo = shiftInfoList.get(0);

            //车辆信息
            VehicleInfo vehicleInfo = vehicleInfoService.get(id);

            //线路所有站点
            List<StationInfo> stationInfoList = stationInfoService.findByRouteId(shiftInfo.getRouteId());
            Map<String, Object> map = new HashMap<>();
            List<VehicleStationDTO> vehicleStationDTOList = stationInfoService.getVehicleStationDTO(shiftInfo, stationInfoList);
            messageResult.setData(vehicleStationDTOList);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @PostMapping("getVehicleInfo")
    @ApiOperation(value = "获取车辆的位置")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "车辆id", required = true, paramType = "form")
    })
    public MessageResult<Map> getVehicleInfo(String id) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {

            List<ShiftInfo> shiftInfoList = shiftInfoService.findByVehicleIdAndStatus(id, "1");

            if (shiftInfoList.size() == 0) {
                throw new Exception("当前车辆没有正运行的班次");
            }

            //班次信息
            ShiftInfo shiftInfo = shiftInfoList.get(0);

            //车辆信息
            VehicleInfo vehicleInfo = vehicleInfoService.get(id);
            Map<String, Object> map = new HashMap<>();
            map.put("longitude", vehicleInfo.getLongitude());
            map.put("latitude", vehicleInfo.getLatitude());
            map.put("currentStationId", shiftInfo.getCurrentStationId());
            map.put("currentStationStatus", shiftInfo.getCurrentStationStatus());
            messageResult.setData(map);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("getPassengerInfo")
    @ApiOperation(value = "获取微信在当前车辆添加乘客信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "车辆id", required = true, paramType = "form"),
            @ApiImplicitParam(name = "openId", value = "openId", required = true, paramType = "form")
    })
    public MessageResult<Map> getPassengerInfo(String id, String openId) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {

            boolean supply = false;
            List<ShiftInfo> shiftInfoList = shiftInfoService.findByVehicleIdAndStatus(id, "1");

            if (shiftInfoList.size() == 0) {
                throw new Exception("当前车辆没有正运行的班次");
            }

            //班次信息
            ShiftInfo shiftInfo = shiftInfoList.get(0);

            RouteInfo routeInfo = routeInfoService.get(shiftInfo.getRouteId());

            //车辆信息
            VehicleInfo vehicleInfo = vehicleInfoService.get(id);

            String mergeOrderId = null;
            List<MergeOrderInfo> mergeOrderInfoList = mergeOrderInfoService.findByOpenIdShiftIdPayStatus(openId, shiftInfo.getId(), 10);

            BigDecimal totalAmount = BigDecimal.ZERO;
            List<PassengerOrderDTO> passengerOrderDTOList = new ArrayList<>();
            if (mergeOrderInfoList.size() > 0) {
                MergeOrderInfo mergeOrderInfo = mergeOrderInfoList.get(0);
                List<OrderInfo> orderInfoList = orderInfoService.findByMergeOrderId(mergeOrderInfo.getId());
                mergeOrderId = mergeOrderInfo.getId();

                if (orderInfoList.size() > 0) {
                    for (OrderInfo orderInfo : orderInfoList) {


                        PassengerInfo passengerInfo = passengerInfoService.get(orderInfo.getPassengerId());
                        //购票始发站
                        StationInfo start = stationInfoService.get(orderInfo.getTicketUpStationId());
                        //购票终点站
                        StationInfo end = stationInfoService.get(orderInfo.getTicketDownStationId());
                        PassengerOrderDTO passengerOrderDTO = new PassengerOrderDTO();
                        passengerOrderDTO.setId(orderInfo.getPassengerId());
                        passengerOrderDTO.setImageUrl(passengerInfo.getImageUrl());
                        passengerOrderDTO.setTicketUpStationName(start.getName());
                        passengerOrderDTO.setTicketUpStationId(start.getId());
                        passengerOrderDTO.setTicketDownStationId(end.getId());
                        passengerOrderDTO.setTicketDownStationName(end.getName());
                        passengerOrderDTO.setTicketType(orderInfo.getTicketType());
                        passengerOrderDTO.setTicketTypeStr(passengerOrderDTO.getTicketTypeStr(passengerOrderDTO.getTicketType()));
                        passengerOrderDTO.setGoodsTicket(orderInfo.getGoodsTicket());
                        passengerOrderDTO.setTotalFee(orderInfo.getTotalFee());
                        passengerOrderDTOList.add(passengerOrderDTO);

                        totalAmount = totalAmount.add(orderInfo.getTotalFee());
                    }
                }
            }

            Map<String, Object> searchParams = new HashMap<>();
            searchParams.put("openId", openId);
            searchParams.put("payStatus", "2");
            searchParams.put("shiftStatus", "1");
            searchParams.put("shiftId", shiftInfo.getId());
            searchParams.put("passengerStatus", "1");
            searchParams.put("routeName", routeInfo.getName());

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("create_time", "desc"));

            Page<PassengerMessage> page = passengerMessageService.pageSearch(searchParams, 1, 5, true, sortList);
            if (page.getResult().size() > 0) {
                supply = true;
            }


            Map<String, Object> map = new HashMap<>();
            map.put("supply", supply);
            map.put("totalAmount", totalAmount);
            map.put("list", passengerOrderDTOList);
            map.put("id", mergeOrderId);
            map.put("routeName", routeInfo.getName());
            messageResult.setData(map);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @PostMapping("upload")
    @ApiOperation(value = "人员照片上传")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "photoName", value = "照片名称", required = true, paramType = "form"),
            @ApiImplicitParam(name = "photoFile", value = "照片base64编码", required = true, paramType = "form"),
    })
    public MessageResult<String> upload(
            String photoName, String photoFile) {
        MessageResult<String> messageResult = new MessageResult<>();

        try {


            String retFileUrl = OSSUtil.uploadBase64(photoFile, photoName, ossConfig);

            messageResult.setResult(true);
            messageResult.setData(retFileUrl);
            messageResult.setCode(200);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(e.getMessage());
        }

        return messageResult;
    }


    @PostMapping("getMatchFaceImage")
    @ApiOperation(value = "获取匹配度高的照片")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "faceImageUrl", value = "人脸照片", required = true, paramType = "form"),
            @ApiImplicitParam(name = "id", value = "车辆id", required = true, paramType = "form")
    })
    public MessageResult<Map> matchFaceImage(String faceImageUrl, String id) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            //车辆
            VehicleInfo vehicleInfo = vehicleInfoService.get(id);
            //人脸库id
            String groupId = CommonUtil.getProvinceNum(vehicleInfo.getLicensePlateNumber()) + StrUtil.sub(vehicleInfo.getLicensePlateNumber(), 1, 10);

            Long personId = baiduService.faceSearch(faceImageUrl, "URL", groupId);
            if (personId == null) {
                throw new Exception("无匹配照片");
            }

            Map<String, Object> map = new HashMap<>();
            messageResult.setData(map);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @PostMapping("upBusNoTicketList")
    @ApiOperation(value = "上传未购票乘客列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "车辆id", required = true, paramType = "form")
    })
    public MessageResult<Map> upBusNoTicketList(String id, @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex, @RequestParam(value = "pageSize", defaultValue = "20") int pageSize) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            //车辆
            VehicleInfo vehicleInfo = vehicleInfoService.get(id);

            List<ShiftInfo> shiftInfoList = shiftInfoService.findByVehicleIdAndStatus(id, "1");

            if (shiftInfoList.size() == 0) {
                throw new Exception("当前车辆没有正运行的班次");
            }

            //班次信息
            ShiftInfo shiftInfo = shiftInfoList.get(0);

            Map<String, Object> searchParams = new HashMap<>();
            searchParams.put("vehicleShiftId", shiftInfo.getId());
            searchParams.put("status", "1");
            searchParams.put("payStatus", "0");
            searchParams.put("maxUpTime", DateUtil.offsetMinute(new Date(), -10));
            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("up_time", "desc"));
            Page<PassengerInfo> page = passengerInfoService.pageSearch(searchParams, pageIndex, pageSize, true, sortList);


            messageResult.setData(PojoUtils.pageWrapper(page));
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @PostMapping("createPassengerOrder")
    @ApiOperation(value = "创建乘客订单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "车辆id", required = true, paramType = "form"),
            @ApiImplicitParam(name = "openId", value = "购票人openId", required = true, paramType = "form"),
            @ApiImplicitParam(name = "mergeOrderId", value = "综合车票id", required = true, paramType = "form"),
            @ApiImplicitParam(name = "passengerIds", value = "乘客ids", required = true, paramType = "form"),
            @ApiImplicitParam(name = "ticketTypes", value = "购票类型", required = true, paramType = "form"),
            @ApiImplicitParam(name = "ticketUpStationId", value = "购票起站", required = true, paramType = "form"),
            @ApiImplicitParam(name = "ticketDownStationId", value = "购票终点站", required = true, paramType = "form"),
            @ApiImplicitParam(name = "goodsTicket", value = "货票金额", required = true, paramType = "form"),
            @ApiImplicitParam(name = "totalFee", value = "总金额", required = true, paramType = "form")
    })
    public MessageResult<Map> createPassengerOrder(String id, String openId, String mergeOrderId, String passengerIds, String ticketTypes, String ticketUpStationId, String ticketDownStationId, String goodsTicket, String totalFee) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            //车辆
            VehicleInfo vehicleInfo = vehicleInfoService.get(id);

            List<ShiftInfo> shiftInfoList = shiftInfoService.findByVehicleIdAndStatus(id, "1");

            if (shiftInfoList.size() == 0) {
                throw new Exception("当前车辆没有正运行的班次");
            }

            //班次信息
            ShiftInfo shiftInfo = shiftInfoList.get(0);


            MergeOrderDTO mergeOrderDTO = passengerInfoService.createOrder(vehicleInfo, shiftInfo, openId, mergeOrderId, passengerIds, ticketUpStationId, ticketDownStationId, ticketTypes, goodsTicket, totalFee);

            Map<String, Object> map = new HashMap<>();
            map.put("MergeOrderDTO", mergeOrderDTO);
            messageResult.setData(map);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @PostMapping("deletePassengerOrder")
    @ApiOperation(value = "删除乘客订单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "乘客id", required = true, paramType = "form"),
            @ApiImplicitParam(name = "mergeOrderId", value = "综合账单id", required = true, paramType = "form")
    })
    public MessageResult<Map> deletePassengerOrder(String id, String mergeOrderId) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {


            MergeOrderInfo mergeOrderInfo = mergeOrderInfoService.get(mergeOrderId);
            if (mergeOrderInfo == null) {
                throw new Exception("账单不存在");
            }

            if (mergeOrderInfo.getPayStatus() == 20) {
                throw new Exception("已支付账单不可删除");
            }

            OrderInfo orderInfo = orderInfoService.findByPassengerIdMergeOrderId(id, mergeOrderId);

            if (orderInfo != null) {
                if (orderInfo.getPayStatus() == 20) {
                    throw new Exception("已支付账单不可删除");
                }
                orderInfo.setDelFlag(true);
                orderInfoService.update(orderInfo);
            }


            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("updatePassengerOrder")
    @ApiOperation(value = "修改乘客订单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "乘客id", required = true, paramType = "form"),
            @ApiImplicitParam(name = "mergeOrderId", value = "综合账单id", required = true, paramType = "form"),
            @ApiImplicitParam(name = "ticketUpStationId", value = "购票起站", required = true, paramType = "form"),
            @ApiImplicitParam(name = "ticketDownStationId", value = "购票终点站", required = true, paramType = "form"),
            @ApiImplicitParam(name = "ticketType", value = "购票类型（1：儿童票，2：成人票）", required = true, paramType = "form"),
            @ApiImplicitParam(name = "goodsTicket", value = "货票金额", required = true, paramType = "form"),
            @ApiImplicitParam(name = "totalFee", value = "总金额", required = true, paramType = "form")
    })
    public MessageResult<Map> updatePassengerOrder(String id, String mergeOrderId, String ticketUpStationId, String ticketDownStationId, String ticketType, String goodsTicket, String totalFee) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {


            MergeOrderInfo mergeOrderInfo = mergeOrderInfoService.get(mergeOrderId);
            if (mergeOrderInfo == null) {
                throw new Exception("账单不存在");
            }

            if (mergeOrderInfo.getPayStatus() == 20) {
                throw new Exception("已支付账单不可修改");
            }

            OrderInfo orderInfo = orderInfoService.findByPassengerIdMergeOrderId(id, mergeOrderId);

            if (orderInfo != null) {
                if (orderInfo.getPayStatus() == 20) {
                    throw new Exception("已支付账单不可修改");
                }
            }

            passengerInfoService.updatePassengerOrder(orderInfo, ticketUpStationId, ticketDownStationId, ticketType, goodsTicket, totalFee);


            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @PostMapping("passengerNearbyStation")
    @ApiOperation(value = "乘客附近站点")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "longitude", value = "经度", required = true, paramType = "form"),
            @ApiImplicitParam(name = "latitude", value = "纬度", required = true, paramType = "form")
    })
    public MessageResult<List<NearbyStationDTO>> passengerNearbyStation(String longitude, String latitude, @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
                                                                        @RequestParam(value = "pageSize", defaultValue = "5") int pageSize) {
        MessageResult<List<NearbyStationDTO>> messageResult = new MessageResult<>();

        try {

            Double lo = Double.valueOf(longitude);
            Double la = Double.valueOf(latitude);

            Double maxLo = lo + 0.005D;
            Double minLo = lo - 0.005D;
            Double maxLa = la + 0.004D;
            Double minLa = la - 0.004D;


            Map<String, Object> searchParams = new HashMap<>();
            searchParams.put("maxLongitude", maxLo);
            searchParams.put("minLongitude", minLo);
            searchParams.put("maxLatitude", maxLa);
            searchParams.put("minLatitude", minLa);

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("create_time", "desc"));

            Page<StationInfo> page = stationInfoService.pageSearch(searchParams, pageIndex, pageSize, true, sortList);

            List<StationInfo> stationInfoList = page.getResult();

            List<NearbyStationDTO> nearbyStationDTOList = new ArrayList<>();
            List<String> shiftIdList = new ArrayList<>();
            List<StationInfo> stationInfos0 = new ArrayList<>();
            if (stationInfoList.size() > 0) {
                int dis0 = 600;
                StationInfo stationInfo0 = null;
                for (StationInfo stationInfo : stationInfoList) {
                    //距离
                    GlobalCoordinates source = new GlobalCoordinates(stationInfo.getLatitude(), stationInfo.getLongitude());
                    GlobalCoordinates target = new GlobalCoordinates(Double.valueOf(latitude), Double.valueOf(longitude));

                    double meter1 = CommonUtil.getDistanceMeter(source, target, Ellipsoid.Sphere);

                    int dis = (new Double(meter1)).intValue();
                    if (dis < dis0) {
                        stationInfo0 = stationInfo;
                        dis0 = dis;
                    }
                }
                stationInfos0.add(stationInfo0);


                for (StationInfo stationInfo : stationInfos0) {

                    NearbyStationDTO nearbyStationDTO = new NearbyStationDTO();
                    List<NearbyShiftDTO> nearbyShiftDTOList = new ArrayList<>();

                    RouteInfo routeInfo = routeInfoService.get(stationInfo.getRouteId());

                    List<StationInfo> stationInfoList1 = stationInfoService.findByRouteIdAndClassify(routeInfo.getId(), 1);
                    List<StationInfo> stationInfoList2 = stationInfoService.findByRouteIdAndClassify(routeInfo.getId(), 3);
                    // List<StationInfo> stationInfoList1 = stationInfoService.findByRouteId(routeInfo.getId());

                    nearbyStationDTO.setStationId(stationInfo.getId());
                    nearbyStationDTO.setStationName(stationInfo.getName());

                    for (int n = 0; n < 2; n++) {
                        String startStationId = stationInfoList1.get(0).getId();
                        String startStationName = stationInfoList1.get(0).getName();
                        String endStationId = stationInfoList2.get(0).getId();
                        String endStationName = stationInfoList2.get(0).getName();
                        if (n == 1) {
                            startStationId = stationInfoList2.get(0).getId();
                            startStationName = stationInfoList2.get(0).getName();
                            endStationId = stationInfoList1.get(0).getId();
                            endStationName = stationInfoList1.get(0).getName();
                        }
                        NearbyShiftDTO nearbyShiftDTO = new NearbyShiftDTO();
                        String des = null;
                        nearbyShiftDTO.setRouteId(routeInfo.getId());
                        nearbyShiftDTO.setRouteName(routeInfo.getName());
                        nearbyShiftDTO.setStartStationName(startStationName);
                        nearbyShiftDTO.setStartStationId(startStationId);
                        nearbyShiftDTO.setEndStationName(endStationName);
                        nearbyShiftDTO.setEndStationId(endStationId);

                        List<ShiftInfo> shiftInfoList = shiftInfoService.findByRouteIdAndStatusAndStartStationId(routeInfo.getId(), "1", nearbyShiftDTO.getStartStationId());
                        if (shiftInfoList.size() == 0) {
                            des = "等待发车";
                        } else {

                            List<StationInfo> stationInfos = stationInfoService.findByRouteId(routeInfo.getId());


                            int dis = 0;
                            ShiftInfo shiftInfo0 = null;
                            for (ShiftInfo shiftInfo : shiftInfoList) {


                                VehicleInfo vehicleInfo = vehicleInfoService.get(shiftInfo.getVehicleId());


                                //距离
                                GlobalCoordinates source = new GlobalCoordinates(Double.valueOf(vehicleInfo.getLatitude()), Double.valueOf(vehicleInfo.getLongitude()));
                                GlobalCoordinates target = new GlobalCoordinates(Double.valueOf(latitude), Double.valueOf(longitude));

                                double meter1 = CommonUtil.getDistanceMeter(source, target, Ellipsoid.Sphere);

                                int intMeter = (new Double(meter1)).intValue();

                                if (dis == 0) {
                                    dis = intMeter;
                                    shiftInfo0 = shiftInfo;
                                } else {
                                    if (dis > intMeter) {
                                        dis = intMeter;
                                        shiftInfo0 = shiftInfo;
                                    }
                                }
                            }
                            //当前站
                            StationInfo currentStation = stationInfoService.get(shiftInfo0.getCurrentStationId());

                            nearbyShiftDTO.setCurrentStationName(currentStation.getName());

                            //班次当前站下标
                            int stationIndex = stationInfos.indexOf(currentStation);
                            //行人当前站
                            int personStationIndex = stationInfos.indexOf(stationInfo);

                            //距离详情
                            String distanceStr = null;
                            if (dis >= 1000) {
                                distanceStr = dis / 1000 + "公里";
                            } else {
                                distanceStr = dis + "米";
                            }

                            //判断顺行还是逆行
                            Boolean tmp = false;
                            if (shiftInfo0.getStartStationId().equals(stationInfos.get(0).getId())) {
                                //顺行
                                tmp = true;
                            }
                            if (tmp) {
                                if (personStationIndex > stationIndex) {

                                    int diff = personStationIndex - stationIndex;
                                    des = diff + "站 " + distanceStr;

                                }
                            }
                            if (!tmp) {
                                if (personStationIndex < stationIndex) {
                                    int diff = stationIndex - personStationIndex;
                                    des = diff + "站 " + distanceStr;

                                }
                            }

                            if (personStationIndex == stationIndex) {
                                des = "已到站";
                            }
                        }
                        nearbyShiftDTO.setDes(des);
                        nearbyShiftDTOList.add(nearbyShiftDTO);
                    }
                    nearbyStationDTO.setNearbyShiftDTOList(nearbyShiftDTOList);

                    nearbyStationDTOList.add(nearbyStationDTO);


                }
            }


            messageResult.setData(nearbyStationDTOList);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @PostMapping("nearbyStationInfo")
    @ApiOperation(value = "乘客附近站点详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "站点id", required = true, paramType = "form"),
            @ApiImplicitParam(name = "longitude", value = "经度", required = true, paramType = "form"),
            @ApiImplicitParam(name = "latitude", value = "纬度", required = true, paramType = "form")
    })
    public MessageResult<NearbyStationDTO> nearbyStationInfo(String id, String longitude, String latitude) {
        MessageResult<NearbyStationDTO> messageResult = new MessageResult<>();

        //   List<NearbyStationDTO> nearbyStationDTOList = new ArrayList<>();
        try {


            StationInfo stationInfo = stationInfoService.get(id);


            NearbyStationDTO nearbyStationDTO = new NearbyStationDTO();
            List<NearbyShiftDTO> nearbyShiftDTOList = new ArrayList<>();

            RouteInfo routeInfo = routeInfoService.get(stationInfo.getRouteId());

            List<StationInfo> stationInfoList1 = stationInfoService.findByRouteIdAndClassify(routeInfo.getId(), 1);
            List<StationInfo> stationInfoList2 = stationInfoService.findByRouteIdAndClassify(routeInfo.getId(), 3);
            // List<StationInfo> stationInfoList1 = stationInfoService.findByRouteId(routeInfo.getId());

            nearbyStationDTO.setStationId(stationInfo.getId());
            nearbyStationDTO.setStationName(stationInfo.getName());
            nearbyStationDTO.setLatitude(stationInfo.getLatitude());
            nearbyStationDTO.setLongitude(stationInfo.getLongitude());

            for (int n = 0; n < 2; n++) {
                String startStationId = stationInfoList1.get(0).getId();
                String startStationName = stationInfoList1.get(0).getName();
                String endStationId = stationInfoList2.get(0).getId();
                String endStationName = stationInfoList2.get(0).getName();
                if (n == 1) {
                    startStationId = stationInfoList2.get(0).getId();
                    startStationName = stationInfoList2.get(0).getName();
                    endStationId = stationInfoList1.get(0).getId();
                    endStationName = stationInfoList1.get(0).getName();
                }
                NearbyShiftDTO nearbyShiftDTO = new NearbyShiftDTO();
                String des = null;
                nearbyShiftDTO.setRouteId(routeInfo.getId());
                nearbyShiftDTO.setRouteName(routeInfo.getName());
                nearbyShiftDTO.setStartStationName(startStationName);
                nearbyShiftDTO.setStartStationId(startStationId);
                nearbyShiftDTO.setEndStationName(endStationName);
                nearbyShiftDTO.setEndStationId(endStationId);

                List<ShiftInfo> shiftInfoList = shiftInfoService.findByRouteIdAndStatusAndStartStationId(routeInfo.getId(), "1", nearbyShiftDTO.getStartStationId());
                if (shiftInfoList.size() == 0) {
                    des = "等待发车";
                } else {

                    List<StationInfo> stationInfos = stationInfoService.findByRouteId(routeInfo.getId());


                    int dis = 0;
                    ShiftInfo shiftInfo0 = null;
                    for (ShiftInfo shiftInfo : shiftInfoList) {


                        VehicleInfo vehicleInfo = vehicleInfoService.get(shiftInfo.getVehicleId());


                        //距离
                        GlobalCoordinates source = new GlobalCoordinates(Double.valueOf(vehicleInfo.getLatitude()), Double.valueOf(vehicleInfo.getLongitude()));
                        GlobalCoordinates target = new GlobalCoordinates(Double.valueOf(latitude), Double.valueOf(longitude));

                        double meter1 = CommonUtil.getDistanceMeter(source, target, Ellipsoid.Sphere);

                        int intMeter = (new Double(meter1)).intValue();

                        if (dis == 0) {
                            dis = intMeter;
                            shiftInfo0 = shiftInfo;
                        } else {
                            if (dis > intMeter) {
                                dis = intMeter;
                                shiftInfo0 = shiftInfo;
                            }
                        }
                    }
                    //当前站
                    StationInfo currentStation = stationInfoService.get(shiftInfo0.getCurrentStationId());

                    //班次当前站下标
                    int stationIndex = stationInfos.indexOf(currentStation);
                    //行人当前站
                    int personStationIndex = stationInfos.indexOf(stationInfo);

                    //距离详情
                    String distanceStr = null;
                    if (dis >= 1000) {
                        distanceStr = dis / 1000 + "公里";
                    } else {
                        distanceStr = dis + "米";
                    }

                    //判断顺行还是逆行
                    Boolean tmp = false;
                    if (shiftInfo0.getStartStationId().equals(stationInfos.get(0).getId())) {
                        //顺行
                        tmp = true;
                    }
                    if (tmp) {
                        if (personStationIndex > stationIndex) {

                            int diff = personStationIndex - stationIndex;
                            des = diff + "站 " + distanceStr;

                        }
                    }
                    if (!tmp) {
                        if (personStationIndex < stationIndex) {
                            int diff = stationIndex - personStationIndex;
                            des = diff + "站 " + distanceStr;

                        }
                    }

                    if (personStationIndex == stationIndex) {
                        des = "已到站";
                    }
                }
                nearbyShiftDTO.setDes(des);
                nearbyShiftDTOList.add(nearbyShiftDTO);
            }
            nearbyStationDTO.setNearbyShiftDTOList(nearbyShiftDTOList);

            //查询站点和本人的距离
            //距离
            GlobalCoordinates source = new GlobalCoordinates(Double.valueOf(stationInfo.getLatitude()), Double.valueOf(stationInfo.getLongitude()));
            GlobalCoordinates target = new GlobalCoordinates(Double.valueOf(latitude), Double.valueOf(longitude));

            double dis = CommonUtil.getDistanceMeter(source, target, Ellipsoid.Sphere);

            int intMeter = (new Double(dis)).intValue();

            nearbyStationDTO.setDistance(intMeter);

            messageResult.setData(nearbyStationDTO);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @PostMapping("routeShiftList")
    @ApiOperation(value = "线路上的所有车列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "routeId", value = "线路id", required = true, paramType = "form"),
            @ApiImplicitParam(name = "startStationId", value = "开始站点id", required = true, paramType = "form")
    })
    public MessageResult<List<RouteShiftDTO>> routeShiftList(String routeId, String startStationId) {
        MessageResult<List<RouteShiftDTO>> messageResult = new MessageResult<>();

        try {

            if (StringUtils.isBlank(routeId) || StringUtils.isBlank(startStationId)) {
                throw new Exception("参数错误");
            }

            List<RouteShiftDTO> list = new ArrayList<>();
            List<ShiftInfo> shiftInfoList = shiftInfoService.findByRouteIdAndStatusAndStartStationId(routeId, "1", startStationId);

            if (shiftInfoList.size() > 0) {
                for (ShiftInfo shiftInfo : shiftInfoList) {
                    DriverInfo driverInfo = driverInfoService.get(shiftInfo.getDriverId());
                    RouteShiftDTO routeShiftDTO = new RouteShiftDTO();
                    BeanUtils.copyProperties(shiftInfo, routeShiftDTO);
                    VehicleInfo vehicleInfo = vehicleInfoService.get(shiftInfo.getVehicleId());
                    routeShiftDTO.setLongitude(vehicleInfo.getLongitude());
                    routeShiftDTO.setLatitude(vehicleInfo.getLatitude());
                    routeShiftDTO.setLicensePlateNumber(vehicleInfo.getLicensePlateNumber());
                    routeShiftDTO.setDriverInfo(driverInfo);

                    list.add(routeShiftDTO);
                }
            }


            messageResult.setData(list);
            messageResult.setCode(200);
            messageResult.setResult(true);


        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("vehicleStationDis")
    @ApiOperation(value = "站点车辆距离")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "routeId", value = "线路id", required = true, paramType = "form"),
            @ApiImplicitParam(name = "startStationId", value = "开始站点id", required = true, paramType = "form"),
            @ApiImplicitParam(name = "currentStationId", value = "当前站点id", required = true, paramType = "form")
    })
    public MessageResult<String> vehicleStationDis(String routeId, String startStationId, String currentStationId) {
        MessageResult<String> messageResult = new MessageResult<>();

        try {

            if (StringUtils.isBlank(routeId) || StringUtils.isBlank(startStationId)) {
                throw new Exception("参数错误");
            }
            String des = null;
            List<StationInfo> stationInfoList = stationInfoService.findByRouteId(routeId);

            if (stationInfoList.size() > 0) {
                //是否顺路
                Boolean tmp = true;
                if (!startStationId.equals(stationInfoList.get(0).getId())) {
                    tmp = false;
                }

                List<ShiftInfo> shiftInfoList = shiftInfoService.findByRouteIdAndStatusAndStartStationId(routeId, "1", startStationId);
                //查询的当前站
                StationInfo currentStation = stationInfoService.get(currentStationId);


                if (shiftInfoList.size() > 0) {
                    String vehicleId = "";
                    int n = 1000;
                    for (ShiftInfo shiftInfo : shiftInfoList) {
                        //班次当前站
                        StationInfo stationInfo = stationInfoService.get(shiftInfo.getCurrentStationId());
                        int stationInfoIndex = stationInfoList.indexOf(stationInfo);

                        int currentStationIndex = stationInfoList.indexOf(currentStation);
                        if (tmp) {
                            if (stationInfoIndex <= currentStationIndex) {
                                int m = currentStationIndex - stationInfoIndex;
                                if (m < n) {
                                    n = m;
                                    vehicleId = shiftInfo.getVehicleId();
                                }
                            }

                        }
                        if (!tmp) {
                            if (stationInfoIndex >= currentStationIndex) {
                                int m = stationInfoIndex - currentStationIndex;
                                if (m < n) {
                                    n = m;
                                    vehicleId = shiftInfo.getVehicleId();
                                }
                            }
                        }

                    }

                    if (StringUtils.isNotBlank(vehicleId)) {
                        //车辆
                        VehicleInfo vehicleInfo = vehicleInfoService.get(vehicleId);
                        //距离
                        GlobalCoordinates source = new GlobalCoordinates(Double.valueOf(vehicleInfo.getLatitude()), Double.valueOf(vehicleInfo.getLongitude()));
                        GlobalCoordinates target = new GlobalCoordinates(Double.valueOf(currentStation.getLatitude()), Double.valueOf(currentStation.getLongitude()));

                        double meter1 = CommonUtil.getDistanceMeter(source, target, Ellipsoid.Sphere);

                        int intMeter = (new Double(meter1)).intValue();
                        //距离详情
                        String distanceStr = null;
                        if (intMeter >= 1000) {
                            distanceStr = intMeter / 1000 + "公里";
                        } else {
                            distanceStr = intMeter + "米";
                        }
                        des = n + "站 " + distanceStr;
                    }
                }

            }

            messageResult.setData(des);
            messageResult.setCode(200);
            messageResult.setResult(true);


        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @PostMapping("routeList")
    @ApiOperation(value = "乘客查询线路列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "name", value = "线路名称", required = true, paramType = "form"),
            @ApiImplicitParam(name = "regionId", value = "区域id", required = false, paramType = "form")
    })
    public MessageResult<Map> routeList(String name, String regionId, @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex, @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {

            List<RouteStationDTO> list = new ArrayList<>();
            Map<String, Object> pageMap = new HashMap<>();
            Map<String, Object> searchParams = new HashMap<>();
            searchParams.put("name", "%" + name + "%");
            searchParams.put("regionId", regionId);
            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("create_time", "desc"));
            Page<RouteInfo> page = routeInfoService.pageSearch(searchParams, pageIndex, pageSize, true, sortList);
            List<RouteInfo> routeInfoList = page.getResult();
            if (routeInfoList.size() > 0) {
                pageMap.put("recordsTotal", page.getTotal());
                pageMap.put("recordsFiltered", page.getTotal());
                pageMap.put("totalPage", page.getPages());
                pageMap.put("pageNumber", page.getPageNum());
                pageMap.put("pageSize", page.getPageSize());

                for (RouteInfo routeInfo : routeInfoList) {
                    RouteStationDTO routeStationDTO = new RouteStationDTO();
                    routeStationDTO.setId(routeInfo.getId());
                    routeStationDTO.setName(routeInfo.getName());
                    list.add(routeStationDTO);
                }

            } else {
                Page<StationInfo> page1 = stationInfoService.pageSearch(searchParams, pageIndex, pageSize, true, sortList);
                List<StationInfo> stationInfoList = page1.getResult();

                pageMap.put("recordsTotal", page1.getTotal());
                pageMap.put("recordsFiltered", page1.getTotal());
                pageMap.put("totalPage", page1.getPages());
                pageMap.put("pageNumber", page1.getPageNum());
                pageMap.put("pageSize", page1.getPageSize());
                if (stationInfoList.size() > 0) {
                    for (StationInfo stationInfo : stationInfoList) {
                        RouteStationDTO routeStationDTO = new RouteStationDTO();
                        routeStationDTO.setId(stationInfo.getRouteId());
                        routeStationDTO.setStationId(stationInfo.getId());
                        routeStationDTO.setName(stationInfo.getName());
                        list.add(routeStationDTO);
                    }
                }
            }


            pageMap.put("data", list);


            messageResult.setData(pageMap);
            messageResult.setCode(200);
            messageResult.setResult(true);


        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("routeDetail")
    @ApiOperation(value = "乘客查询线路详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "线路id", required = true, paramType = "form")
    })
    public MessageResult<Map> routeDetail(String id) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {

            RouteInfo routeInfo = routeInfoService.get(id);
            if (routeInfo == null) {
                throw new Exception("线路不存在");
            }

            List<StationInfo> stationInfoList = stationInfoService.findByRouteId(id);


            Map<String, Object> map = new HashMap<>();
            map.put("routeInfo", routeInfo);
            map.put("stationInfoList", stationInfoList);


            messageResult.setData(map);
            messageResult.setCode(200);
            messageResult.setResult(true);


        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("passengerMessage")
    @ApiOperation(value = "乘客过站消息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "openId", value = "openId", paramType = "form")
    })
    public MessageResult<Map> passengerMessage(String openId, @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex, @RequestParam(value = "pageSize", defaultValue = "5") int pageSize) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {

            if (StringUtils.isBlank(openId)) {
                throw new Exception("openId为空");
            }


            Map<String, Object> searchParams = new HashMap<>();
            searchParams.put("openId", openId);
            searchParams.put("payStatus", "2");
            searchParams.put("passengerStatus", "1");
            searchParams.put("shiftStatus", "1");

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("create_time", "desc"));

            Page<PassengerMessage> page = passengerMessageService.pageSearch(searchParams, pageIndex, pageSize, true, sortList);

            List<PassengerMessage> passengerMessageList = page.getResult();
            List<PassengerMessageDTO> list = new ArrayList<>();
            if (passengerMessageList.size() > 0) {
                for (PassengerMessage passengerMessage : passengerMessageList) {
                    PassengerMessageDTO passengerMessageDTO = new PassengerMessageDTO();
                    PassengerInfo passengerInfo = passengerInfoService.get(passengerMessage.getPassengerId());

                    //过站点
                    StationInfo stationInfo = stationInfoService.get(passengerInfo.getTicketDownStationId());
                    ShiftInfo shiftInfo = shiftInfoService.get(passengerInfo.getVehicleShiftId());
                    //线路
                    RouteInfo routeInfo = routeInfoService.get(shiftInfo.getRouteId());

                    passengerMessageDTO.setId(passengerInfo.getId());
                    passengerMessageDTO.setVehicleId(shiftInfo.getVehicleId());
                    passengerMessageDTO.setStationId(stationInfo.getId());
                    passengerMessageDTO.setStationName(stationInfo.getName());
                    passengerMessageDTO.setRouteName(routeInfo.getName());
                    passengerMessageDTO.setSendTime(DateUtil.format(passengerMessage.getCreateTime(), "yyyy-MM-dd HH:mm:ss"));
                    passengerMessageDTO.setStatusName("已过站");
                    passengerMessageDTO.setImageUrl(passengerInfo.getImageUrl());
                    passengerMessageDTO.setTicketType(passengerInfo.getTicketType());
                    passengerMessageDTO.setTicketTypeName(passengerMessageDTO.getTicketTypeName(passengerMessageDTO.getTicketType()));
                    list.add(passengerMessageDTO);

                    passengerMessage.setReadStatus(true);
                    passengerMessageService.update(passengerMessage);

                }
            }

            Map<String, Object> pageMap = new HashMap<>();

            pageMap.put("recordsTotal", page.getTotal());
            pageMap.put("recordsFiltered", page.getTotal());
            pageMap.put("totalPage", page.getPages());
            pageMap.put("pageNumber", page.getPageNum());
            pageMap.put("pageSize", page.getPageSize());
            pageMap.put("data", list);

            messageResult.setData(pageMap);
            messageResult.setCode(200);
            messageResult.setResult(true);


        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("passengerMessageNoReadNum")
    @ApiOperation(value = "乘客未读过站消息数量")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "openId", value = "openId", paramType = "form")
    })
    public MessageResult<Map> passengerMessageNoReadNum(String openId) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {

            if (StringUtils.isBlank(openId)) {
                throw new Exception("openId为空");
            }

            Map<String, Object> searchParams = new HashMap<>();
            searchParams.put("openId", openId);
            searchParams.put("readStatus", false);
            searchParams.put("shiftStatus", "1");
            searchParams.put("passengerStatus", "1");
            searchParams.put("payStatus", "2");

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("create_time", "desc"));

            Page<PassengerMessage> page = passengerMessageService.pageSearch(searchParams, 1, 100, true, sortList);
            List<PassengerMessage> passengerMessageList = page.getResult();


            Map<String, Object> map = new HashMap<>();
            map.put("num", passengerMessageList.size());

            messageResult.setData(map);
            messageResult.setCode(200);
            messageResult.setResult(true);


        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @PostMapping("passengerRecordList")
    @ApiOperation(value = "乘车记录")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "openId", value = "openId", paramType = "form")
    })
    public MessageResult<Map> passengerRecordList(String openId, @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex, @RequestParam(value = "pageSize", defaultValue = "5") int pageSize) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {


            if (StringUtils.isBlank(openId)) {
                throw new Exception("openId为空");
            }

            Map<String, Object> searchParams = new HashMap<>();
            searchParams.put("openId", openId);
            searchParams.put("payStatus", 20);

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("a.create_time", "desc"));

            Page<OrderInfo> page = orderInfoService.pageSearch(searchParams, pageIndex, pageSize, true, sortList);

            List<OrderInfo> orderInfoList = page.getResult();

            List<PassengerRecordDTO> list = new ArrayList<>();
            if (orderInfoList.size() > 0) {
                for (OrderInfo orderInfo : orderInfoList) {
                    PassengerRecordDTO passengerRecordDTO = new PassengerRecordDTO();

                    //乘客记录
                    PassengerInfo passengerInfo = passengerInfoService.get(orderInfo.getPassengerId());
                    passengerRecordDTO.setId(orderInfo.getId());
                    passengerRecordDTO.setImageUrl(passengerInfo.getImageUrl());
                    //开始站点
                    StationInfo startStation = stationInfoService.get(orderInfo.getTicketUpStationId());
                    //终点站
                    StationInfo endStation = stationInfoService.get(orderInfo.getTicketDownStationId());
                    passengerRecordDTO.setStartStation(startStation.getName());
                    passengerRecordDTO.setEndStation(endStation.getName());
                    passengerRecordDTO.setTicketType(orderInfo.getTicketType());
                    passengerRecordDTO.setTicketTypeName(passengerRecordDTO.getTicketTypeName(passengerRecordDTO.getTicketType()));
                    passengerRecordDTO.setGoodTicket(orderInfo.getGoodsTicket());
                    passengerRecordDTO.setPayTime(DateUtil.format(orderInfo.getPayTime(), "yyyy-MM-dd HH:mm:ss"));
                    passengerRecordDTO.setTotalFee(orderInfo.getTotalFee());
                    passengerRecordDTO.setStatus(passengerInfo.getStatus());
                    passengerRecordDTO.setStatusName(passengerRecordDTO.getStatusName(passengerInfo.getStatus()));
                    list.add(passengerRecordDTO);

                }
            }

            Map<String, Object> pageMap = new HashMap<>();

            pageMap.put("recordsTotal", page.getTotal());
            pageMap.put("recordsFiltered", page.getTotal());
            pageMap.put("totalPage", page.getPages());
            pageMap.put("pageNumber", page.getPageNum());
            pageMap.put("pageSize", page.getPageSize());
            pageMap.put("data", list);

            messageResult.setData(pageMap);
            messageResult.setCode(200);
            messageResult.setResult(true);


        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("passengerRecordDetail")
    @ApiOperation(value = "乘车记录详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "id", paramType = "form")
    })
    public MessageResult<PassengerRecordDetailDTO> passengerRecordDetail(String id) {
        MessageResult<PassengerRecordDetailDTO> messageResult = new MessageResult<>();

        try {

            OrderInfo orderInfo = orderInfoService.get(id);

            //乘客信息
            PassengerInfo passengerInfo = passengerInfoService.get(orderInfo.getPassengerId());

            //班次
            ShiftInfo shiftInfo = shiftInfoService.get(passengerInfo.getVehicleShiftId());
            //开始站
            StationInfo startStation = stationInfoService.get(shiftInfo.getStartStationId());
            LonLatDTO lonLatDTO1 = new LonLatDTO();
            lonLatDTO1.setLatitude(startStation.getLatitude());
            lonLatDTO1.setLongitude(startStation.getLongitude());
            //终点站
            StationInfo endStation = stationInfoService.get(shiftInfo.getEndStationId());
            LonLatDTO lonLatDTO2 = new LonLatDTO();
            lonLatDTO2.setLatitude(endStation.getLatitude());
            lonLatDTO2.setLongitude(endStation.getLongitude());

            List<LonLatDTO> routeStationLonLat = new ArrayList<>();
            routeStationLonLat.add(lonLatDTO1);
            routeStationLonLat.add(lonLatDTO2);


            shiftInfo.setStartStationName(startStation.getName());
            shiftInfo.setEndStationName(endStation.getName());

            //线路
            RouteInfo routeInfo = routeInfoService.get(shiftInfo.getRouteId());

            //车辆
            VehicleInfo vehicleInfo = vehicleInfoService.get(shiftInfo.getVehicleId());

            //司机
            DriverInfo driverInfo = driverInfoService.get(shiftInfo.getDriverId());

            //购票上车站点
            StationInfo ticketUpStation = stationInfoService.get(orderInfo.getTicketUpStationId());
            LonLatDTO lonLatDTO3 = new LonLatDTO();
            lonLatDTO3.setLatitude(ticketUpStation.getLatitude());
            lonLatDTO3.setLongitude(ticketUpStation.getLongitude());

            //购票下次站点
            StationInfo ticketDownStation = stationInfoService.get(orderInfo.getTicketDownStationId());
            LonLatDTO lonLatDTO4 = new LonLatDTO();
            lonLatDTO4.setLatitude(ticketDownStation.getLatitude());
            lonLatDTO4.setLongitude(ticketDownStation.getLongitude());

            List<LonLatDTO> ticketUpDownLonLat = new ArrayList<>();
            ticketUpDownLonLat.add(lonLatDTO3);
            ticketUpDownLonLat.add(lonLatDTO4);

            //购票类型
            String ticketTypeName = "成人票";
            if ("1".equals(orderInfo.getTicketType())) {
                ticketTypeName = "儿童票";
            }


            PassengerRecordDetailDTO passengerRecordDetailDTO = new PassengerRecordDetailDTO();

            passengerRecordDetailDTO.setDriverInfo(driverInfo);
            passengerRecordDetailDTO.setRouteInfo(routeInfo);
            passengerRecordDetailDTO.setTripName(ticketUpStation.getName() + "-" + ticketDownStation.getName());
            passengerRecordDetailDTO.setTicketDes(ticketTypeName + " " + orderInfo.getTotalFee().subtract(orderInfo.getGoodsTicket()) + "元");
            passengerRecordDetailDTO.setGoodsTicket(orderInfo.getGoodsTicket() + "元");
            passengerRecordDetailDTO.setTotalFee(orderInfo.getTotalFee());
            passengerRecordDetailDTO.setPayTime(DateUtil.format(orderInfo.getPayTime(), "yyyy-MM-dd HH:mm:ss"));
            passengerRecordDetailDTO.setTicketStatus(passengerInfo.getStatus());
            passengerRecordDetailDTO.setTicketStatusName(passengerRecordDetailDTO.getTicketStatusName(passengerRecordDetailDTO.getTicketStatus()));
            passengerRecordDetailDTO.setShiftInfo(shiftInfo);
            passengerRecordDetailDTO.setVehicleInfo(vehicleInfo);
            passengerRecordDetailDTO.setRouteStationLonLat(routeStationLonLat);
            passengerRecordDetailDTO.setTicketUpDownLonLat(ticketUpDownLonLat);

            messageResult.setData(passengerRecordDetailDTO);
            messageResult.setCode(200);
            messageResult.setResult(true);


        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @PostMapping("problemFeedback")
    @ApiOperation(value = "问题反馈")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "content", value = "问题内容", required = false, paramType = "form"),
            @ApiImplicitParam(name = "picUrl", value = "图片地址", required = false, paramType = "form"),
            @ApiImplicitParam(name = "telephone", value = "联系电话", required = false, paramType = "form"),
            @ApiImplicitParam(name = "subject", value = "目标(不传)", paramType = "form")
    })
    public MessageResult<Map> problemFeedback(String token, @RequestAttribute String subject, String content, String picUrl, String telephone) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            UserInfo userInfo = userInfoService.get(Long.parseLong(subject));
            if (userInfo == null) {
                throw new Exception("请先注册信息");
            }

            if (StringUtils.isBlank(content)) {
                throw new Exception("请输入反馈内容");
            }

            Map<String, Object> dataMap = new HashMap<String, Object>();
            ProblemFeedback problemFeedback = new ProblemFeedback();
            problemFeedback.setId(UUID.randomUUID().toString());
            problemFeedback.setProblemContent(content);
            problemFeedback.setProblemPic(picUrl);
            problemFeedback.setContactTelephone(telephone);
            problemFeedback.setCreateBy(userInfo.getId().toString());
            problemFeedback.setCreateTime(new Date());
            problemFeedbackService.insert(problemFeedback);


            messageResult.setData(dataMap);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("userInfo")
    @ApiOperation(value = "用户信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "subject", value = "目标(不传)", paramType = "form")
    })
    public MessageResult<Map> userPhone(String token, @RequestAttribute String subject) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            UserInfo userInfo = userInfoService.get(Long.parseLong(subject));
            if (userInfo == null) {
                throw new Exception("请先注册信息");
            }

            Map<String, Object> map = new HashMap<>();
            map.put("userInfo", userInfo);

            messageResult.setData(map);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("userUpdatePhone")
    @ApiOperation(value = "用户更新绑定手机号码")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "phone", value = "手机号码", paramType = "form"),
            @ApiImplicitParam(name = "code", value = "验证码", paramType = "form"),
            @ApiImplicitParam(name = "subject", value = "目标(不传)", paramType = "form")
    })
    public MessageResult<Map> userUpdatePhone(String token, @RequestAttribute String subject, String code, String phone) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            UserInfo userInfo = userInfoService.get(Long.parseLong(subject));
            if (userInfo == null) {
                throw new Exception("请先注册信息");
            }

            if (phone.equals(userInfo.getPhone())) {
                throw new Exception("请更换手机号码");
            }

            String smsKey = "SMS_" + phone;

            String beforeVerifyCode = (String) redisTemplate.opsForValue().get(smsKey);

            if (StringUtils.isEmpty(beforeVerifyCode)) {
                throw new Exception("验证码已过期!");
            }

            if (!beforeVerifyCode.equals(code)) {
                throw new Exception("验证码错误!");
            }

            userInfo.setPhone(phone);
            userInfoService.update(userInfo);

            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("updateUserInfo")
    @ApiOperation(value = "更新用户的基本信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "name", value = "用户姓名", paramType = "form"),
            @ApiImplicitParam(name = "faceImage", value = "人脸照片", paramType = "form"),
            @ApiImplicitParam(name = "acceptMessage", value = "是否接受乘车信息（0：不接受，1：接受）", paramType = "form"),
            @ApiImplicitParam(name = "subject", value = "目标(不传)", paramType = "form")
    })
    public MessageResult<Map> updateUserInfo(String token, @RequestAttribute String subject, String name, String faceImage, String acceptMessage) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            UserInfo userInfo = userInfoService.get(Long.parseLong(subject));
            if (userInfo == null) {
                throw new Exception("请先注册信息");
            }

            Boolean accept = false;
            if ("1".equals(acceptMessage)) {
                accept = true;
            }

            userInfo.setName(name);
            userInfo.setFaceImage(faceImage);
            userInfo.setAcceptMessage(accept);
            userInfoService.update(userInfo);

            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setCode(400);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("mergeInfo")
    @ApiOperation(value = "综合账单详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "outOrderNo", value = "商户订单号", paramType = "form")
    })
    public MessageResult<Map> mergeInfo(String outOrderNo) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {

            List<PassengerRecordDTO> list = new ArrayList<>();
            MergeOrderInfo mergeOrderInfo = mergeOrderInfoService.findByOutOrderNo(outOrderNo);
            ShiftInfo shiftInfo = shiftInfoService.get(mergeOrderInfo.getVehicleShiftId());
            if (mergeOrderInfo != null) {
                List<OrderInfo> orderInfoList = orderInfoService.findByMergeOrderId(mergeOrderInfo.getId());
                if (orderInfoList.size() > 0) {
                    for (OrderInfo orderInfo : orderInfoList) {
                        PassengerRecordDTO passengerRecordDTO = new PassengerRecordDTO();

                        //乘客记录
                        PassengerInfo passengerInfo = passengerInfoService.get(orderInfo.getPassengerId());
                        passengerRecordDTO.setId(orderInfo.getId());
                        passengerRecordDTO.setImageUrl(passengerInfo.getImageUrl());
                        //开始站点
                        StationInfo startStation = stationInfoService.get(orderInfo.getTicketUpStationId());
                        //终点站
                        StationInfo endStation = stationInfoService.get(orderInfo.getTicketDownStationId());
                        passengerRecordDTO.setStartStation(startStation.getName());
                        passengerRecordDTO.setEndStation(endStation.getName());
                        passengerRecordDTO.setTicketType(orderInfo.getTicketType());
                        passengerRecordDTO.setTicketTypeName(passengerRecordDTO.getTicketTypeName(passengerRecordDTO.getTicketType()));
                        passengerRecordDTO.setGoodTicket(orderInfo.getGoodsTicket());
                        passengerRecordDTO.setPayTime(DateUtil.format(orderInfo.getPayTime(), "yyyy-MM-dd HH:mm:ss"));
                        passengerRecordDTO.setTotalFee(orderInfo.getTotalFee());
                        list.add(passengerRecordDTO);
                    }
                }
            }
            Map<String, Object> map = new HashMap<>();
            map.put("id", shiftInfo.getVehicleId());
            map.put("list", list);

            messageResult.setData(map);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @PostMapping("passengerShiftRemindList")
    @ApiOperation(value = "乘客当前班次提醒列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "routeId", value = "线路id", paramType = "form"),
            @ApiImplicitParam(name = "openId", value = "openId", paramType = "form")
    })
    public MessageResult<List<StationRemind>> passengerShiftRemindList(String routeId, String openId) {
        MessageResult<List<StationRemind>> messageResult = new MessageResult<>();

        try {

            List<StationRemind> stationRemindList = stationRemindService.findByOpenIdAndRouteId(openId, routeId);

            messageResult.setData(stationRemindList);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @PostMapping("passengerRemindList")
    @ApiOperation(value = "乘客提醒列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "openId", value = "openId", paramType = "form")
    })
    public MessageResult<List<StationRemind>> passengerRemindList(String openId) {
        MessageResult<List<StationRemind>> messageResult = new MessageResult<>();

        try {


            List<StationRemind> stationRemindList = stationRemindService.findByOpenId(openId);
            if (stationRemindList.size() > 0) {
                for (StationRemind stationRemind : stationRemindList) {

                    if (StringUtils.isNotBlank(stationRemind.getVehicleShiftId())) {
                        ShiftInfo shiftInfo = shiftInfoService.get(stationRemind.getVehicleShiftId());
                        if ("1".equals(shiftInfo.getStatus())) {
                            RouteInfo routeInfo = routeInfoService.get(shiftInfo.getRouteId());
                            //终点站
                            StationInfo endStation = stationInfoService.get(shiftInfo.getEndStationId());
                            //提醒站
                            StationInfo remindStation = stationInfoService.get(stationRemind.getRemindStationId());
                            stationRemind.setRouteName(routeInfo.getName());
                            stationRemind.setEndStationName(endStation.getName());
                            stationRemind.setRemindStationName(remindStation.getName());
                        }

                    }
                    if (StringUtils.isNotBlank(stationRemind.getRouteId())) {
                        RouteInfo routeInfo = routeInfoService.get(stationRemind.getRouteId());
                        //终点站
                        StationInfo endStation = stationInfoService.get(stationRemind.getEndStationId());
                        //提醒站
                        StationInfo remindStation = stationInfoService.get(stationRemind.getRemindStationId());
                        stationRemind.setRouteName(routeInfo.getName());
                        stationRemind.setEndStationName(endStation.getName());
                        stationRemind.setRemindStationName(remindStation.getName());
                    }

                }
            }


            messageResult.setData(stationRemindList);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("helpCenterList")
    @ApiOperation(value = "帮助中心列表")
    public MessageResult<List<HelpCenter>> helpCenterList() {
        MessageResult<List<HelpCenter>> messageResult = new MessageResult<>();

        try {

            List<HelpCenter> helpCenterList = helpCenterService.list();

            messageResult.setData(helpCenterList);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @PostMapping("getRegionList")
    @ApiOperation(value = "获取区域列表")
    public MessageResult<List<RegionInfo>> getRegionList() {
        MessageResult<List<RegionInfo>> messageResult = new MessageResult<>();

        try {

            List<RegionInfo> regionInfoList = regionInfoService.findCityRegion();


            messageResult.setData(regionInfoList);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("getOwnerRegion")
    @ApiOperation(value = "获取自己所在区域")
    public MessageResult<RegionInfo> getOwnerRegion(String longitude, String latitude) {
        MessageResult<RegionInfo> messageResult = new MessageResult<>();

        try {

            List<RegionInfo> regionInfoList = regionInfoService.findCityRegion();
            RegionInfo regionInfo0 = regionInfoList.get(0);
            if (regionInfoList.size() > 0) {

                double distance = 0D;
                for (RegionInfo regionInfo : regionInfoList) {
                    if (regionInfo.getLatitude() != null && regionInfo.getLongitude() != null) {

                        GlobalCoordinates source = new GlobalCoordinates(Double.valueOf(latitude), Double.valueOf(longitude));

                        GlobalCoordinates target = new GlobalCoordinates(regionInfo.getLatitude(), regionInfo.getLongitude());

                        double meter1 = CommonUtil.getDistanceMeter(source, target, Ellipsoid.Sphere);
                        if (distance == 0D) {
                            regionInfo0 = regionInfo;
                            distance = meter1;
                        } else {
                            if (distance > meter1) {
                                regionInfo0 = regionInfo;
                                distance = meter1;
                            }
                        }

                    }
                }
            }

            messageResult.setData(regionInfo0);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


}
