package com.jpsoft.bus.modules.mq.listener;

import com.jpsoft.bus.config.WxConfig;
import com.jpsoft.bus.modules.bus.entity.*;
import com.jpsoft.bus.modules.bus.service.*;
import com.jpsoft.bus.modules.common.utils.CommonUtil;
import com.jpsoft.bus.modules.common.utils.WechatMessageUtil;
import lombok.extern.slf4j.Slf4j;
import net.sf.json.JSONObject;
import org.gavaghan.geodesy.Ellipsoid;
import org.gavaghan.geodesy.GlobalCoordinates;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import java.text.DecimalFormat;
import java.util.Date;
import java.util.List;

@Slf4j
@Component
@RabbitListener(queues = "stationRemindQueue")
public class StationRemindListener {
    @Autowired
    private StationInfoService stationInfoService;

    @Autowired
    private VehicleInfoService vehicleInfoService;

    @Autowired
    private DriverInfoService driverInfoService;

    @Autowired
    private StationRemindService stationRemindService;

    @Autowired
    private RouteInfoService routeInfoService;

    @Value("${wx.stationRemindTemplateId}")
    private String stationRemindTemplateId;

    @Autowired
    private WxConfig wxConfig;

    @Autowired
    private WechatMessageUtil wechatMessageUtil;

    @RabbitHandler
    public void process(ShiftInfo shiftInfo) {
        try {
            //查询站点提醒中设置当前站点到站提醒
            List<StationRemind> stationRemindList = stationRemindService.findByAdvanceStationId(shiftInfo.getCurrentStationId());

            RouteInfo routeInfo = routeInfoService.get(shiftInfo.getRouteId());
            StationInfo startStation = stationInfoService.get(shiftInfo.getStartStationId());
            StationInfo endStation = stationInfoService.get(shiftInfo.getEndStationId());

            VehicleInfo vehicleInfo = vehicleInfoService.get(shiftInfo.getVehicleId());
            DriverInfo driverInfo = driverInfoService.get(shiftInfo.getDriverId());

            //当前车辆所在站点
            StationInfo stationInfo = stationInfoService.get(shiftInfo.getCurrentStationId());

            for (StationRemind stationRemind : stationRemindList) {
                try {
                    String remindStr = "上车刷脸听到“感谢乘车”通行，入座扫码手机购票。司机："+driverInfo.getName() + ",电话："+ driverInfo.getPhone();

                    if (stationRemind.getRemindType() != null && stationRemind.getRemindType() == 2) {
                        //下车提醒要判断是不是同一个班次
                        if (!stationRemind.getVehicleShiftId().equals(shiftInfo.getId())) {
                            continue;
                        }
                        else{
                            remindStr = "下车时刷脸听到“感谢乘车”即可下车！";
                        }
                    }

                    String detail = "上车提醒";
                    if (stationRemind.getRemindType() == 2){
                        detail = "下车提醒";
                    }


                    StationInfo remindStation = stationInfoService.get(stationRemind.getRemindStationId());
                    StationInfo advanceStation = stationInfoService.get(stationRemind.getAdvanceStationId());

                    String templateId = stationRemindTemplateId;
                    JSONObject sendData = new JSONObject();

                    JSONObject thing4 = new JSONObject();
                    thing4.put("value", routeInfo.getName() + "(" + startStation.getName() + "," + endStation.getName() + ")");
                    sendData.put("thing4", thing4);

                    JSONObject thing2 = new JSONObject();
                    thing2.put("value", advanceStation.getName());
                    sendData.put("thing2", thing2);

                    JSONObject thing6 = new JSONObject();
                    thing6.put("value", String.format("大约%s",
                            getDistance(remindStation, advanceStation)));
                    sendData.put("thing6", thing6);

                    JSONObject thing3 = new JSONObject();
                    //距离目的站点(%s)还有%s站 remindStation.getName(),stationRemind.getStopInAdvance()
                    thing3.put("value", String.format("%s",remindStr));

                    sendData.put("thing3", thing3);

                 //   wechatMessageUtil.sendSubscribeTemplate(wxConfig.getAppId(), wxConfig.getAppSecret(), templateId, stationRemind.getOpenId(), sendData, "");

                    wechatMessageUtil.sendCarPosition(stationRemind.getOpenId(),detail,vehicleInfo.getLicensePlateNumber(),stationInfo.getName(),remindStr,wxConfig.getPortalUrl()+"/pages/buytickets/selectSite?id="+shiftInfo.getVehicleId(),wxConfig.getAppId(), wxConfig.getAppSecret());

                    stationRemind.setDelFlag(true);
                    stationRemind.setUpdateTime(new Date());

                    stationRemindService.update(stationRemind);
                } catch (Exception ex) {
                    log.error(ex.getMessage(), ex);
                }
            }
        }
        catch (Exception e){
            log.error(e.getMessage(), e);
        }
    }

    private String getDistance(StationInfo remindStation, StationInfo advanceStation) {
        String distance = "";
        GlobalCoordinates source = new GlobalCoordinates(Double.valueOf(advanceStation.getLatitude()), Double.valueOf(advanceStation.getLongitude()));
        GlobalCoordinates target = new GlobalCoordinates(Double.valueOf(remindStation.getLatitude()), Double.valueOf(remindStation.getLongitude()));

        Double meter = CommonUtil.getDistanceMeter(source, target, Ellipsoid.Sphere);
        DecimalFormat df = new DecimalFormat("0.00");

        if (meter>1000){
            distance = df.format(meter/1000) + "千米";
        }
        else{
            distance = meter.intValue() + "米";
        }

        return distance;
    }
}
