package com.jpsoft.bus.modules.pay.alipay;


import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.DefaultAlipayClient;
import com.alipay.api.internal.util.AlipaySignature;
import com.alipay.api.request.AlipaySystemOauthTokenRequest;
import com.alipay.api.request.AlipayTradeCreateRequest;
import com.alipay.api.request.AlipayTradePrecreateRequest;
import com.alipay.api.response.AlipaySystemOauthTokenResponse;
import com.alipay.api.response.AlipayTradeCreateResponse;
import com.alipay.api.response.AlipayTradePrecreateResponse;
import com.ijpay.alipay.AliPayApi;
import com.jpsoft.bus.config.AliPayConfig;
import com.jpsoft.bus.modules.base.entity.MergeOrderInfo;
import com.jpsoft.bus.modules.base.entity.OrderInfo;
import com.jpsoft.bus.modules.base.entity.PaymentInfo;
import com.jpsoft.bus.modules.base.service.MergeOrderInfoService;
import com.jpsoft.bus.modules.base.service.OrderInfoService;
import com.jpsoft.bus.modules.bus.entity.PassengerInfo;
import com.jpsoft.bus.modules.bus.service.PassengerInfoService;
import com.jpsoft.bus.modules.common.dto.MessageResult;
import com.jpsoft.bus.modules.pay.service.AlipayService;
import com.jpsoft.bus.modules.wechat.vo.UserInfo;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author 墨鱼_mo
 * @date 2019-11-26 9:48
 */

@Slf4j
@RequestMapping("/aliPay")
@RestController
public class AlipayController {

    @Autowired
    private OrderInfoService orderInfoService;

    @Autowired
    private AlipayService alipayService;

    @Autowired
    private MergeOrderInfoService mergeOrderInfoService;

    @Autowired
    private PassengerInfoService passengerInfoService;

    @Autowired
    private AliPayConfig aliPayConfig;



    @RequestMapping(value="/payNotify")
    public String payNotify(HttpServletRequest request) {
        try {
            // 获取支付宝POST过来反馈信息

            Map<String, String> params = AliPayApi.toMap(request);

            for (Map.Entry<String, String> entry : params.entrySet()) {
                System.out.println(entry.getKey() + " = " + entry.getValue());
            }

            boolean verifyResult = AlipaySignature.rsaCheckV1(params, aliPayConfig.getZfbPublicKey(), "UTF-8", "RSA2");

            if (verifyResult) {

                String outOrderNo = params.get("out_trade_no");
                MergeOrderInfo mergeOrderInfo = mergeOrderInfoService.findByOutOrderNo(outOrderNo);

                String payFee = params.get("total_amount");
                if (mergeOrderInfo == null) {
                    throw new Exception("支付已完成，但未找到此订单号的流水");
                }

                mergeOrderInfo.setPayStatus(20);
                mergeOrderInfo.setPayName("alipay");
                mergeOrderInfo.setTransactionId(params.get("trade_no"));
                mergeOrderInfo.setUpdateTime(new Date());
                mergeOrderInfoService.update(mergeOrderInfo);

                //同一个班次下的其他未支付删除
                List<MergeOrderInfo> mergeOrderInfoList = mergeOrderInfoService.findByOpenIdShiftIdPayStatus(mergeOrderInfo.getOpenId(),mergeOrderInfo.getVehicleShiftId(),10);
                if (mergeOrderInfoList.size()>0){
                    for (MergeOrderInfo mergeOrderInfo1 : mergeOrderInfoList){
                        if (!mergeOrderInfo1.getId().equals(mergeOrderInfo.getId())){
                            mergeOrderInfo1.setDelFlag(true);
                            mergeOrderInfoService.update(mergeOrderInfo1);
                        }

                    }
                }

                List<OrderInfo> orderInfoList = orderInfoService.findByMergeOrderId(mergeOrderInfo.getId());

                if (orderInfoList.size()>0){

                    for (OrderInfo orderInfo : orderInfoList){
                        orderInfo.setUpdateTime(new Date());
                        orderInfo.setUpdateBy(params.get("buyer_logon_id"));
                        orderInfo.setPayName("alipay");
                        orderInfo.setPayStatus(20);
                        orderInfo.setPayFee(orderInfo.getTotalFee());
                        orderInfo.setTransactionId(params.get("trade_no"));
                        orderInfo.setPayTime(DateUtil.parse(params.get("gmt_payment")));
                        orderInfoService.update(orderInfo);


                        try {
                            new Thread(() -> {
                                PassengerInfo passengerInfo = passengerInfoService.get(orderInfo.getPassengerId());
                                if (passengerInfo != null){
                                    passengerInfo.setTicketUpStationId(orderInfo.getTicketUpStationId());
                                    passengerInfo.setTicketDownStationId(orderInfo.getTicketDownStationId());
                                    passengerInfo.setPayStatus("1");
                                    passengerInfo.setTicketType(orderInfo.getTicketType());
                                    passengerInfoService.update(passengerInfo);
                                }

                                try{
                                    //根据订单类型做各自处理
                                }catch (Exception ex){

                                }

                            }).start();
                        } catch (Exception e) {
                            log.error(e.getMessage(), e);
                        }
                    }
                }





                return "success";
            } else {
                log.error("notify_url 验证失败");
                return "failure";
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error(e.getMessage(), e);
            return "failure";
        }
    }



    @GetMapping(value="findUserInfo/{authCode}")
    public MessageResult findUserInfo(@PathVariable(name="authCode") String authCode){

        System.out.println("获取支付宝的userId");
        try{

            AlipayClient alipayClient = new DefaultAlipayClient(aliPayConfig.getServiceUrl(),aliPayConfig.getAppId(),aliPayConfig.getPrivateKey(),"json",aliPayConfig.getInputCharset(),aliPayConfig.getZfbPublicKey(),aliPayConfig.getSignType());

            AlipaySystemOauthTokenRequest request = new AlipaySystemOauthTokenRequest();
            request.setCode(authCode);
            request.setGrantType("authorization_code");

            AlipaySystemOauthTokenResponse oauthTokenResponse = alipayClient.execute(request);

            System.out.println(oauthTokenResponse.getUserId());

            HashMap<String,Object> dataMap = new HashMap<String,Object>();

            UserInfo userInfo = new UserInfo();
            userInfo.setOpenid(oauthTokenResponse.getUserId());
       //     dataMap.put("openId", oauthTokenResponse.getUserId());

         //   HashMap<String,Object> resultMap = new HashMap<String,Object>();

        //    resultMap.put("userInfo", dataMap);

            return new MessageResult(true, "获取支付宝信息成功", userInfo,200);
        } catch (AlipayApiException e) {



            //处理异常
            e.printStackTrace();
            return new MessageResult(false, "获取支付宝授权失败", "",400);
        } catch (Exception e) {
            //处理异常
            e.printStackTrace();
            return new MessageResult(false, "获取支付宝授权失败", "",400);
        }
    }


    @ApiOperation(value = "支付宝扫码支付")
    @GetMapping("/tradePrecreatePay")
    public MessageResult tradePrecreatePay(String id) {

        MessageResult result = new MessageResult();

        try {

            MergeOrderInfo mergeOrderInfo = mergeOrderInfoService.get(id);

            if (mergeOrderInfo == null) {
                throw new Exception("订单不存在");
            }

            List<OrderInfo> orderInfoList = orderInfoService.findByMergeOrderId(id);

            if (orderInfoList.size() == 0){
                throw new Exception("订单错误");
            }
            for (OrderInfo orderInfo : orderInfoList){
                if (orderInfo.getPayStatus() !=10){
                    throw new Exception("有已完成购票的乘客，请删除");
                }
            }


            String qrCode = alipayService.tradePrecreatePay(id);
            result.setResult(true);

            result.setData(qrCode);

        } catch (Exception e) {
            e.printStackTrace();
            log.error(e.getMessage(), e);
            result.setResult(false);
            result.setMessage(e.getMessage());
        }
        return result;

    }


}
