package com.jpsoft.bus.modules.pay.wechat;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.ijpay.core.enums.SignType;
import com.ijpay.core.enums.TradeType;
import com.ijpay.core.kit.HttpKit;
import com.ijpay.core.kit.WxPayKit;
import com.ijpay.wxpay.WxPayApi;
import com.ijpay.wxpay.model.UnifiedOrderModel;
import com.jfinal.kit.StrKit;
import com.jpsoft.bus.config.WxJpsoftConfig;
import com.jpsoft.bus.modules.base.entity.MergeOrderInfo;
import com.jpsoft.bus.modules.base.entity.OrderInfo;
import com.jpsoft.bus.modules.base.entity.PaymentInfo;
import com.jpsoft.bus.modules.base.service.MergeOrderInfoService;
import com.jpsoft.bus.modules.base.service.OrderInfoService;
import com.jpsoft.bus.modules.base.service.PaymentInfoService;
import com.jpsoft.bus.modules.bus.entity.PassengerInfo;
import com.jpsoft.bus.modules.bus.service.PassengerInfoService;
import com.jpsoft.bus.modules.common.dto.MessageResult;
import com.jpsoft.bus.modules.pay.service.WechatService;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author 墨鱼_mo
 * @date 2019-11-26 10:35
 */

@RequestMapping("/wxPay")
@RestController
@Slf4j
public class WxPayController {

    @Autowired
    private WxJpsoftConfig wxJpsoftConfig;

    @Autowired
    private MergeOrderInfoService mergeOrderInfoService;

    @Autowired
    private PassengerInfoService passengerInfoService;

    @Autowired
    private OrderInfoService orderInfoService;

    @Autowired
    private WechatService wechatService;

    @Autowired
    private PaymentInfoService paymentInfoService;


    @ApiOperation(value = "微信JSAPI支付公用方法")
    @GetMapping("/wxJsapiPay")
    public MessageResult wxJsapiPay(String id) {
        MessageResult msgResult = new MessageResult<>();
        try {
            MergeOrderInfo mergeOrderInfo = mergeOrderInfoService.get(id);

            if (mergeOrderInfo == null) {
                throw new Exception("订单不存在:" + id);
            }

            PaymentInfo paymentInfo = paymentInfoService.get(mergeOrderInfo.getPaymentId());
            String subMchId = paymentInfo.getSubMchId();
            String outOrderNo = mergeOrderInfo.getOutOrderNo();

            List<OrderInfo> orderInfoList = orderInfoService.findByMergeOrderId(id);
            BigDecimal totalFee = BigDecimal.ZERO;
            if (orderInfoList.size() == 0){
                throw new Exception("订单错误");
            }
            for (OrderInfo orderInfo : orderInfoList){
                if (orderInfo.getPayStatus() !=10){
                    throw new Exception("有已完成购票的乘客，请删除");
                }
                totalFee = totalFee.add(orderInfo.getTotalFee());
            }


            //金额处理
            int wxTotalTee = totalFee.multiply(new BigDecimal(100)).intValue();
            Map<String, String> params = UnifiedOrderModel
                    .builder()
                    .appid(wxJpsoftConfig.getAppId())
                    .mch_id(wxJpsoftConfig.getMchId())
                    .sub_mch_id(subMchId)
                    //     .sub_appid(wxConfig.getAppId())
                    .nonce_str(WxPayKit.generateStr())
                    .body(orderInfoList.get(0).getBody())
                    .out_trade_no(outOrderNo)
                    .total_fee(String.valueOf(wxTotalTee))
                    .spbill_create_ip(wxJpsoftConfig.getIp())
                    .notify_url(paymentInfo.getWechatNotifyUrl())
                    .trade_type(TradeType.JSAPI.getTradeType())
                    .sub_openid(mergeOrderInfo.getOpenId())
                    //    .openid(orderInfo.getOpenId())
                    .build()
                    .createSign(wxJpsoftConfig.getMchKey(), SignType.HMACSHA256);

            String xmlResult = WxPayApi.pushOrder(false, params);

            Map<String, String> resultMap = WxPayKit.xmlToMap(xmlResult);
            String returnCode = resultMap.get("return_code");
            String returnMsg = resultMap.get("return_msg");
            if (!WxPayKit.codeIsOk(returnCode)) {
                log.error(returnCode);
                throw new Exception(returnMsg);
            }
            String resultCode = resultMap.get("result_code");
            if (!WxPayKit.codeIsOk(resultCode)) {
                String errCode = resultMap.get("err_code_des");
                log.error(errCode);
                throw new Exception(errCode);
            }

            // 以下字段在 return_code 和 result_code 都为 SUCCESS 的时候有返回

            String prepayId = resultMap.get("prepay_id");

            Map<String, String> packageParams = WxPayKit.prepayIdCreateSign(prepayId, wxJpsoftConfig.getAppId(),
                    wxJpsoftConfig.getMchKey(), SignType.HMACSHA256);

            msgResult.setData(packageParams);
            msgResult.setResult(true);

        } catch (Exception e) {
            e.printStackTrace();
            log.error(e.getMessage(), e);
            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
        }
        return msgResult;
    }

    @RequestMapping(value = "/wxPayNotify", method = {RequestMethod.POST, RequestMethod.GET})
    @ResponseBody
    public String wxPayNotify(HttpServletRequest request) {
        String xmlMsg = HttpKit.readData(request);
        log.warn("支付通知=" + xmlMsg);
        Map<String, String> params = WxPayKit.xmlToMap(xmlMsg);

        String returnCode = params.get("return_code");
        String outTradeNo = params.get("out_trade_no");
        String payTimeStr = params.get("time_end");
        Date payTime = DateUtil.parse(payTimeStr);

        // 注意此处签名方式需与统一下单的签名类型一致 WxPayApiConfigKit.getWxPayApiConfig().getPartnerKey()
        if (WxPayKit.verifyNotify(params, wxJpsoftConfig.getMchKey(), SignType.HMACSHA256)) {
            if (WxPayKit.codeIsOk(returnCode)) {
                // 更新订单信息

                try {

                    MergeOrderInfo mergeOrderInfo = mergeOrderInfoService.findByOutOrderNo(outTradeNo);

                    if (mergeOrderInfo == null) {
                        throw new Exception("支付已完成，但未找到此订单号的流水");
                    }

                    mergeOrderInfo.setPayStatus(20);
                    mergeOrderInfo.setPayName("wechat");
                    mergeOrderInfo.setTransactionId(params.get("transaction_id"));
                    mergeOrderInfo.setUpdateTime(new Date());
                    mergeOrderInfoService.update(mergeOrderInfo);

                    //同一个班次下的其他未支付删除
                    List<MergeOrderInfo> mergeOrderInfoList = mergeOrderInfoService.findByOpenIdShiftIdPayStatus(mergeOrderInfo.getOpenId(),mergeOrderInfo.getVehicleShiftId(),10);
                    if (mergeOrderInfoList.size()>0){
                        for (MergeOrderInfo mergeOrderInfo1 : mergeOrderInfoList){
                            if (!mergeOrderInfo1.getId().equals(mergeOrderInfo.getId())){
                                mergeOrderInfo1.setDelFlag(true);
                                mergeOrderInfoService.update(mergeOrderInfo1);
                            }
                        }
                    }

                    List<OrderInfo> orderInfoList = orderInfoService.findByMergeOrderId(mergeOrderInfo.getId());

                    if (orderInfoList.size()>0){
                        for (OrderInfo orderInfo : orderInfoList){
                            orderInfo.setUpdateTime(new Date());
                            orderInfo.setUpdateBy(params.get("openid"));
                            orderInfo.setPayName("wechat");
                            orderInfo.setPayStatus(20);
                            orderInfo.setPayFee(orderInfo.getTotalFee());
                            orderInfo.setTransactionId(params.get("transaction_id"));
                            orderInfo.setPayTime(payTime);
                            orderInfoService.update(orderInfo);

                            try {
                                PassengerInfo passengerInfo = passengerInfoService.get(orderInfo.getPassengerId());

                                if (passengerInfo != null){
                                    passengerInfo.setTicketUpStationId(orderInfo.getTicketUpStationId());
                                    passengerInfo.setTicketDownStationId(orderInfo.getTicketDownStationId());
                                    passengerInfo.setPayStatus("1");
                                    passengerInfo.setTicketType(orderInfo.getTicketType());
                                    passengerInfoService.update(passengerInfo);
                                }
                            } catch (Exception ex) {
                                log.error(ex.getMessage(), ex);
                            }
                        }
                    }

                    // 发送通知等
                    Map<String, String> xml = new HashMap<String, String>(2);
                    xml.put("return_code", "SUCCESS");
                    xml.put("return_msg", "OK");
                    return WxPayKit.toXml(xml);
                } catch (Exception e) {
                    log.error(e.getMessage(), e);
                    e.printStackTrace();
                }
            }
        }

        return null;
    }


    @ApiOperation(value = "微信扫码支付功能")
    @GetMapping("/wxNativePay")
    @ResponseBody
    public MessageResult wxNativePay(String id) {

        MessageResult msgResult = new MessageResult<>();

        try {

            MergeOrderInfo mergeOrderInfo = mergeOrderInfoService.get(id);

            if (mergeOrderInfo == null) {
                throw new Exception("订单不存在");
            }

            List<OrderInfo> orderInfoList = orderInfoService.findByMergeOrderId(id);

            if (orderInfoList.size() == 0){
                throw new Exception("订单错误");
            }
            for (OrderInfo orderInfo : orderInfoList){
                if (orderInfo.getPayStatus() !=10){
                    throw new Exception("有已完成购票的乘客，请删除");
                }
            }

            String qrCodeUrl = wechatService.wxNativePay(id);
            msgResult.setData(qrCodeUrl);
            msgResult.setResult(true);


        } catch (Exception ex) {
            log.error(ex.getMessage(),ex);
            msgResult.setMessage(ex.getMessage());
            msgResult.setResult(false);
        }
        return msgResult;
    }

}
