package com.jpsoft.bus.modules.sys.controller;


import com.alibaba.fastjson.JSONObject;
import com.jpsoft.bus.modules.common.dto.MessageResult;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.RestTemplate;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.apache.commons.lang3.StringUtils;

import com.jpsoft.bus.modules.common.dto.Sort;

import javax.servlet.http.HttpServletRequest;
import java.nio.charset.Charset;
import java.util.*;

@Slf4j
@RestController
@RequestMapping("/sys/api")
public class ApiController {
    @Value("${springfox.documentation.swagger.v2.host:}")
    private String swaggerV2Host;

    @Autowired
    private RestTemplate restTemplate;

    @ApiOperation("获取后端api列表")
    @GetMapping("selectAll")
    public MessageResult<List> selectAll(HttpServletRequest request){
        MessageResult<List> msgResult = new MessageResult<>();
        
        String basePath;
        String path = request.getContextPath();

        if (StringUtils.isNotEmpty(swaggerV2Host)){
            basePath = request.getScheme() + "://" + swaggerV2Host + "/";

            if(StringUtils.isNotEmpty(path)){
                basePath += path + "/";
            }
        }
        else {
            basePath = request.getScheme() + "://" + request.getServerName() + ":" + request.getServerPort() + path + "/";
        }


        ResponseEntity<String> entity = null;

        try {
            String url = basePath + "v2/api-docs";

            log.warn("api后端地址：" + url);

            entity = restTemplate.getForEntity( url, String.class);

            JSONObject jsonObject = JSONObject.parseObject(entity.getBody());

            JSONObject paths = jsonObject.getJSONObject("paths");

            List<Map> mapList = new ArrayList<>();

            for(String pathKey : paths.keySet()){

                JSONObject methods = paths.getJSONObject(pathKey);

                for(String methodKey : methods.keySet()){
                    String summary = methods.getJSONObject(methodKey).getString("summary");

                    Map<String,Object> map = new HashMap<>();

                    map.put("path", pathKey);
                    map.put("method", methodKey);
                    map.put("summary", summary);

                    mapList.add(map);
                }
            }

            msgResult.setData(mapList);
            msgResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);
            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
